@@ -2,7 +2,7 @@ use std::rc::Rc;
 
 use floem::{
     event::EventListener,
-    keyboard::ModifiersState,
+    keyboard::Modifiers,
     reactive::{RwSignal, Scope},
     style::{CursorStyle, Display, Position},
     view::View,
@@ -68,12 +68,13 @@ impl KeyPressFocus for AboutData {
         &self,
         command: &crate::command::LapceCommand,
         _count: Option<usize>,
-        _mods: ModifiersState,
+        _mods: Modifiers,
     ) -> crate::command::CommandExecuted {
         match &command.kind {
             CommandKind::Workbench(_) => {}
             CommandKind::Edit(_) => {}
             CommandKind::Move(_) => {}
+            CommandKind::Scroll(_) => {}
             CommandKind::Focus(cmd) => {
                 if cmd == &FocusCommand::ModalClose {
                     self.close();
@@ -100,56 +101,56 @@ pub fn about_popup(window_tab_data: Rc<WindowTabData>) -> impl View {
 
     exclusive_popup(window_tab_data, about_data.visible, move || {
         stack((
-            svg(move || (*config.get()).logo_svg()).style(move |s| {
+            svg(move || (config.get()).logo_svg()).style(move |s| {
                 s.size(logo_size, logo_size)
-                    .color(*config.get().get_color(LapceColor::EDITOR_FOREGROUND))
+                    .color(config.get().color(LapceColor::EDITOR_FOREGROUND))
             }),
             label(|| "Lapce".to_string()).style(move |s| {
                 s.font_bold()
                     .margin_top(10.0)
-                    .color(*config.get().get_color(LapceColor::EDITOR_FOREGROUND))
+                    .color(config.get().color(LapceColor::EDITOR_FOREGROUND))
             }),
             label(|| format!("Version: {}", VERSION)).style(move |s| {
                 s.margin_top(10.0)
-                    .color(*config.get().get_color(LapceColor::EDITOR_DIM))
+                    .color(config.get().color(LapceColor::EDITOR_DIM))
             }),
             web_link(
                 || "Website".to_string(),
                 || AboutUri::LAPCE.to_string(),
-                move || *config.get().get_color(LapceColor::EDITOR_LINK),
+                move || config.get().color(LapceColor::EDITOR_LINK),
                 internal_command,
             )
             .style(|s| s.margin_top(20.0)),
             web_link(
                 || "GitHub".to_string(),
                 || AboutUri::GITHUB.to_string(),
-                move || *config.get().get_color(LapceColor::EDITOR_LINK),
+                move || config.get().color(LapceColor::EDITOR_LINK),
                 internal_command,
             )
             .style(|s| s.margin_top(10.0)),
             web_link(
                 || "Discord".to_string(),
                 || AboutUri::DISCORD.to_string(),
-                move || *config.get().get_color(LapceColor::EDITOR_LINK),
+                move || config.get().color(LapceColor::EDITOR_LINK),
                 internal_command,
             )
             .style(|s| s.margin_top(10.0)),
             web_link(
                 || "Matrix".to_string(),
                 || AboutUri::MATRIX.to_string(),
-                move || *config.get().get_color(LapceColor::EDITOR_LINK),
+                move || config.get().color(LapceColor::EDITOR_LINK),
                 internal_command,
             )
             .style(|s| s.margin_top(10.0)),
             label(|| "Attributions".to_string()).style(move |s| {
                 s.font_bold()
-                    .color(*config.get().get_color(LapceColor::EDITOR_DIM))
+                    .color(config.get().color(LapceColor::EDITOR_DIM))
                     .margin_top(40.0)
             }),
             web_link(
                 || "Codicons (CC-BY-4.0)".to_string(),
                 || AboutUri::CODICONS.to_string(),
-                move || *config.get().get_color(LapceColor::EDITOR_LINK),
+                move || config.get().color(LapceColor::EDITOR_LINK),
                 internal_command,
             )
             .style(|s| s.margin_top(10.0)),
@@ -174,8 +175,8 @@ fn exclusive_popup<V: View + 'static>(
                         .padding_horiz(100.0)
                         .border(1.0)
                         .border_radius(6.0)
-                        .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
-                        .background(*config.get_color(LapceColor::PANEL_BACKGROUND))
+                        .border_color(config.color(LapceColor::LAPCE_BORDER))
+                        .background(config.color(LapceColor::PANEL_BACKGROUND))
                 })
                 .on_event_stop(EventListener::PointerDown, move |_| {}),
         )
@@ -204,7 +205,7 @@ fn exclusive_popup<V: View + 'static>(
         .background(
             config
                 .get()
-                .get_color(LapceColor::LAPCE_DROPDOWN_SHADOW)
+                .color(LapceColor::LAPCE_DROPDOWN_SHADOW)
                 .with_alpha_factor(0.5),
         )
     })
