@@ -1,9 +1,12 @@
 use std::{path::PathBuf, rc::Rc, sync::Arc};
 
-use floem::{keyboard::ModifiersState, peniko::kurbo::Vec2};
+use floem::{
+    keyboard::Modifiers, peniko::kurbo::Vec2, views::editor::command::Command,
+};
 use indexmap::IndexMap;
 use lapce_core::command::{
-    EditCommand, FocusCommand, MotionModeCommand, MoveCommand, MultiSelectionCommand,
+    EditCommand, FocusCommand, MotionModeCommand, MoveCommand,
+    MultiSelectionCommand, ScrollCommand,
 };
 use lapce_rpc::{
     dap_types::{DapId, RunDebugConfig},
@@ -19,7 +22,7 @@ use strum_macros::{Display, EnumIter, EnumMessage, EnumString, IntoStaticStr};
 use crate::{
     alert::AlertButton,
     debug::RunDebugMode,
-    doc::Document,
+    doc::Doc,
     editor::location::EditorLocation,
     editor_tab::EditorTabChild,
     id::EditorTabId,
@@ -27,6 +30,8 @@ use crate::{
     workspace::LapceWorkspace,
 };
 
+pub use floem::views::editor::command::CommandExecuted;
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct LapceCommand {
     pub kind: CommandKind,
@@ -38,6 +43,7 @@ pub enum CommandKind {
     Workbench(LapceWorkbenchCommand),
     Edit(EditCommand),
     Move(MoveCommand),
+    Scroll(ScrollCommand),
     Focus(FocusCommand),
     MotionMode(MotionModeCommand),
     MultiSelection(MultiSelectionCommand),
@@ -49,6 +55,7 @@ impl CommandKind {
             CommandKind::Workbench(cmd) => cmd.get_message(),
             CommandKind::Edit(cmd) => cmd.get_message(),
             CommandKind::Move(cmd) => cmd.get_message(),
+            CommandKind::Scroll(cmd) => cmd.get_message(),
             CommandKind::Focus(cmd) => cmd.get_message(),
             CommandKind::MotionMode(cmd) => cmd.get_message(),
             CommandKind::MultiSelection(cmd) => cmd.get_message(),
@@ -60,17 +67,26 @@ impl CommandKind {
             CommandKind::Workbench(cmd) => cmd.into(),
             CommandKind::Edit(cmd) => cmd.into(),
             CommandKind::Move(cmd) => cmd.into(),
+            CommandKind::Scroll(cmd) => cmd.into(),
             CommandKind::Focus(cmd) => cmd.into(),
             CommandKind::MotionMode(cmd) => cmd.into(),
             CommandKind::MultiSelection(cmd) => cmd.into(),
         }
     }
 }
-
-#[derive(PartialEq, Eq)]
-pub enum CommandExecuted {
-    Yes,
-    No,
+impl From<Command> for CommandKind {
+    fn from(cmd: Command) -> Self {
+        use Command::*;
+        match cmd {
+            Edit(edit) => CommandKind::Edit(edit),
+            Move(movement) => CommandKind::Move(movement),
+            Scroll(scroll) => CommandKind::Scroll(scroll),
+            MotionMode(motion_mode) => CommandKind::MotionMode(motion_mode),
+            MultiSelection(multi_selection) => {
+                CommandKind::MultiSelection(multi_selection)
+            }
+        }
+    }
 }
 
 pub fn lapce_internal_commands() -> IndexMap<String, LapceCommand> {
@@ -100,6 +116,14 @@ pub fn lapce_internal_commands() -> IndexMap<String, LapceCommand> {
         commands.insert(c.to_string(), command);
     }
 
+    for c in ScrollCommand::iter() {
+        let command = LapceCommand {
+            kind: CommandKind::Scroll(c.clone()),
+            data: None,
+        };
+        commands.insert(c.to_string(), command);
+    }
+
     for c in FocusCommand::iter() {
         let command = LapceCommand {
             kind: CommandKind::Focus(c.clone()),
@@ -283,9 +307,10 @@ pub enum LapceWorkbenchCommand {
     #[strum(message = "Connect to SSH Host")]
     ConnectSshHost,
 
-    #[strum(serialize = "connect_wsl")]
-    #[strum(message = "Connect to WSL")]
-    ConnectWsl,
+    #[cfg(windows)]
+    #[strum(serialize = "connect_wsl_host")]
+    #[strum(message = "Connect to WSL Host")]
+    ConnectWslHost,
 
     #[strum(serialize = "disconnect_remote")]
     #[strum(message = "Disconnect From Remote")]
@@ -453,6 +478,10 @@ pub enum LapceWorkbenchCommand {
     #[strum(message = "Change current file language")]
     ChangeFileLanguage,
 
+    #[strum(serialize = "change_file_line_ending")]
+    #[strum(message = "Change current file line ending")]
+    ChangeFileLineEnding,
+
     #[strum(serialize = "next_editor_tab")]
     #[strum(message = "Next Editor Tab")]
     NextEditorTab,
@@ -506,6 +535,10 @@ pub enum LapceWorkbenchCommand {
     #[strum(serialize = "previous_error")]
     PreviousError,
 
+    #[strum(message = "Diff Files")]
+    #[strum(serialize = "diff_files")]
+    DiffFiles,
+
     #[strum(serialize = "quit")]
     #[strum(message = "Quit Editor")]
     Quit,
@@ -524,6 +557,23 @@ pub enum InternalCommand {
     OpenFileChanges {
         path: PathBuf,
     },
+    ReloadFileExplorer,
+    /// Test whether a file/directory can be created at that path
+    TestPathCreation {
+        new_path: PathBuf,
+    },
+    FinishRenamePath {
+        current_path: PathBuf,
+        new_path: PathBuf,
+    },
+    FinishNewNode {
+        is_dir: bool,
+        path: PathBuf,
+    },
+    FinishDuplicate {
+        source: PathBuf,
+        path: PathBuf,
+    },
     GoToLocation {
         location: EditorLocation,
     },
@@ -605,12 +655,12 @@ pub enum InternalCommand {
     FindEditorCommand {
         command: LapceCommand,
         count: Option<usize>,
-        mods: ModifiersState,
+        mods: Modifiers,
     },
     ReplaceEditorCommand {
         command: LapceCommand,
         count: Option<usize>,
-        mods: ModifiersState,
+        mods: Modifiers,
     },
     FocusEditorTab {
         editor_tab_id: EditorTabId,
@@ -642,7 +692,10 @@ pub enum InternalCommand {
     },
     HideAlert,
     SaveScratchDoc {
-        doc: Rc<Document>,
+        doc: Rc<Doc>,
+    },
+    SaveScratchDoc2 {
+        doc: Rc<Doc>,
     },
     UpdateProxyStatus {
         status: ProxyStatus,
@@ -655,6 +708,10 @@ pub enum InternalCommand {
         volt_id: VoltID,
     },
     ResetBlinkCursor,
+    OpenDiffFiles {
+        left_path: PathBuf,
+        right_path: PathBuf,
+    },
 }
 
 #[derive(Clone)]
