@@ -1,3 +1,4 @@
+use ::core::slice;
 use std::{
     collections::HashMap,
     path::{Path, PathBuf},
@@ -14,12 +15,13 @@ use once_cell::sync::Lazy;
 use parking_lot::RwLock;
 use serde::Deserialize;
 use strum::VariantNames;
+use tracing::error;
 
 use self::{
     color::LapceColor,
     color_theme::{ColorThemeConfig, ThemeColor, ThemeColorPreference},
     core::CoreConfig,
-    editor::{EditorConfig, SCALE_OR_SIZE_LIMIT},
+    editor::{EditorConfig, WrapStyle, SCALE_OR_SIZE_LIMIT},
     icon::LapceIcons,
     icon_theme::IconThemeConfig,
     svg::SvgStore,
@@ -49,6 +51,30 @@ static DEFAULT_CONFIG: Lazy<config::Config> = Lazy::new(LapceConfig::default_con
 static DEFAULT_LAPCE_CONFIG: Lazy<LapceConfig> =
     Lazy::new(LapceConfig::default_lapce_config);
 
+static DEFAULT_DARK_THEME_CONFIG: Lazy<config::Config> = Lazy::new(|| {
+    config::Config::builder()
+        .add_source(config::File::from_str(
+            DEFAULT_DARK_THEME,
+            config::FileFormat::Toml,
+        ))
+        .build()
+        .unwrap()
+});
+
+/// The default theme is the dark theme.
+static DEFAULT_DARK_THEME_COLOR_CONFIG: Lazy<ColorThemeConfig> = Lazy::new(|| {
+    let (_, theme) =
+        LapceConfig::load_color_theme_from_str(DEFAULT_DARK_THEME).unwrap();
+    theme.get::<ColorThemeConfig>("color-theme")
+    .expect("Failed to load default dark theme. This is likely due to a missing or misnamed field in dark-theme.toml")
+});
+static DEFAULT_ICON_THEME_CONFIG: Lazy<IconThemeConfig> = Lazy::new(|| {
+    let (_, theme) =
+        LapceConfig::load_icon_theme_from_str(DEFAULT_ICON_THEME).unwrap();
+    theme.get::<IconThemeConfig>("icon-theme")
+    .expect("Failed to load default icon theme. This is likely due to a missing or misnamed field in icon-theme.toml")
+});
+
 /// Used for creating a `DropdownData` for a setting
 #[derive(Debug, Clone)]
 pub struct DropdownInfo {
@@ -57,7 +83,7 @@ pub struct DropdownInfo {
     pub items: im::Vector<String>,
 }
 
-#[derive(Clone, Deserialize, Default)]
+#[derive(Debug, Clone, Deserialize, Default)]
 #[serde(rename_all = "kebab-case")]
 pub struct LapceConfig {
     #[serde(skip)]
@@ -66,15 +92,13 @@ pub struct LapceConfig {
     pub ui: UIConfig,
     pub editor: EditorConfig,
     pub terminal: TerminalConfig,
+    #[serde(default)]
     pub color_theme: ColorThemeConfig,
+    #[serde(default)]
     pub icon_theme: IconThemeConfig,
     #[serde(flatten)]
     pub plugins: HashMap<String, HashMap<String, serde_json::Value>>,
     #[serde(skip)]
-    pub default_color_theme: ColorThemeConfig,
-    #[serde(skip)]
-    pub default_icon_theme: IconThemeConfig,
-    #[serde(skip)]
     pub color: ThemeColor,
     #[serde(skip)]
     pub available_color_themes: HashMap<String, (String, config::Config)>,
@@ -91,18 +115,30 @@ pub struct LapceConfig {
     color_theme_list: im::Vector<String>,
     #[serde(skip)]
     icon_theme_list: im::Vector<String>,
+    /// The couple names for the wrap style
+    #[serde(skip)]
+    wrap_style_list: im::Vector<String>,
 }
 
 impl LapceConfig {
-    pub fn load(workspace: &LapceWorkspace, disabled_volts: &[VoltID]) -> Self {
+    pub fn load(
+        workspace: &LapceWorkspace,
+        disabled_volts: &[VoltID],
+        extra_plugin_paths: &[PathBuf],
+    ) -> Self {
         let config = Self::merge_config(workspace, None, None);
-        let mut lapce_config: LapceConfig = config
-            .try_deserialize()
-            .unwrap_or_else(|_| DEFAULT_LAPCE_CONFIG.clone());
+        let mut lapce_config: LapceConfig = match config.try_deserialize() {
+            Ok(config) => config,
+            Err(error) => {
+                error!("Failed to deserialize configuration file: {error}");
+                DEFAULT_LAPCE_CONFIG.clone()
+            }
+        };
 
         lapce_config.available_color_themes =
-            Self::load_color_themes(disabled_volts);
-        lapce_config.available_icon_themes = Self::load_icon_themes(disabled_volts);
+            Self::load_color_themes(disabled_volts, extra_plugin_paths);
+        lapce_config.available_icon_themes =
+            Self::load_icon_themes(disabled_volts, extra_plugin_paths);
         lapce_config.resolve_theme(workspace);
 
         lapce_config.color_theme_list = lapce_config
@@ -121,6 +157,13 @@ impl LapceConfig {
             .collect();
         lapce_config.icon_theme_list.sort();
 
+        lapce_config.wrap_style_list = im::vector![
+            WrapStyle::None.to_string(),
+            WrapStyle::EditorWidth.to_string(),
+            // TODO: WrapStyle::WrapColumn.to_string(),
+            WrapStyle::WrapWidth.to_string()
+        ];
+
         lapce_config.terminal.get_indexed_colors();
 
         lapce_config
@@ -132,9 +175,13 @@ impl LapceConfig {
         icon_theme_config: Option<config::Config>,
     ) -> config::Config {
         let mut config = DEFAULT_CONFIG.clone();
+
         if let Some(theme) = color_theme_config {
+            // TODO: use different color theme basis if the theme declares its color preference
+            // differently
             config = config::Config::builder()
                 .add_source(config.clone())
+                .add_source(DEFAULT_DARK_THEME_CONFIG.clone())
                 .add_source(theme)
                 .build()
                 .unwrap_or_else(|_| config.clone());
@@ -171,7 +218,7 @@ impl LapceConfig {
             }
             LapceWorkspaceType::RemoteSSH(_) => {}
             #[cfg(windows)]
-            LapceWorkspaceType::RemoteWSL => {}
+            LapceWorkspaceType::RemoteWSL(_) => {}
         }
 
         config
@@ -196,27 +243,15 @@ impl LapceConfig {
 
     fn default_lapce_config() -> LapceConfig {
         let mut default_lapce_config: LapceConfig =
-            DEFAULT_CONFIG.clone().try_deserialize().unwrap();
+            DEFAULT_CONFIG.clone().try_deserialize().expect("Failed to deserialize default config, this likely indicates a missing or misnamed field in settings.toml");
+        default_lapce_config.color_theme = DEFAULT_DARK_THEME_COLOR_CONFIG.clone();
+        default_lapce_config.icon_theme = DEFAULT_ICON_THEME_CONFIG.clone();
         default_lapce_config.resolve_colors(None);
         default_lapce_config
     }
 
     fn resolve_theme(&mut self, workspace: &LapceWorkspace) {
-        let mut default_lapce_config = DEFAULT_LAPCE_CONFIG.clone();
-        if let Some((_, color_theme_config)) = self
-            .available_color_themes
-            .get(&self.core.color_theme.to_lowercase())
-        {
-            if let Ok(mut theme_lapce_config) = config::Config::builder()
-                .add_source(DEFAULT_CONFIG.clone())
-                .add_source(color_theme_config.clone())
-                .build()
-                .and_then(|theme| theme.try_deserialize::<LapceConfig>())
-            {
-                theme_lapce_config.resolve_colors(Some(&default_lapce_config));
-                default_lapce_config = theme_lapce_config;
-            }
-        }
+        let default_lapce_config = DEFAULT_LAPCE_CONFIG.clone();
 
         let color_theme_config = self
             .available_color_themes
@@ -257,17 +292,18 @@ impl LapceConfig {
             }
         }
         self.resolve_colors(Some(&default_lapce_config));
-        self.default_color_theme = default_lapce_config.color_theme.clone();
-        self.default_icon_theme = default_lapce_config.icon_theme.clone();
         self.update_id();
     }
 
     fn load_color_themes(
         disabled_volts: &[VoltID],
+        extra_plugin_paths: &[PathBuf],
     ) -> HashMap<String, (String, config::Config)> {
         let mut themes = Self::load_local_themes().unwrap_or_default();
 
-        for (key, theme) in Self::load_plugin_color_themes(disabled_volts) {
+        for (key, theme) in
+            Self::load_plugin_color_themes(disabled_volts, extra_plugin_paths)
+        {
             themes.insert(key, theme);
         }
 
@@ -281,6 +317,10 @@ impl LapceConfig {
         themes
     }
 
+    pub fn default_color_theme(&self) -> &ColorThemeConfig {
+        &DEFAULT_DARK_THEME_COLOR_CONFIG
+    }
+
     /// Set the active color theme.
     /// Note that this does not save the config.
     pub fn set_color_theme(&mut self, workspace: &LapceWorkspace, theme: &str) {
@@ -304,16 +344,19 @@ impl LapceConfig {
     /// # Panics
     /// If the color was not able to be found in either theme, which may be indicative that
     /// it is misspelled or needs to be added to the base-theme.
-    pub fn get_color(&self, name: &str) -> &Color {
-        self.color
-            .ui
-            .get(name)
-            .unwrap_or_else(|| panic!("Key not found: {name}"))
+    pub fn color(&self, name: &str) -> Color {
+        match self.color.ui.get(name) {
+            Some(c) => *c,
+            None => {
+                error!("Failed to find key: {name}");
+                Color::HOT_PINK
+            }
+        }
     }
 
     /// Retrieve a color value whose key starts with "style."
-    pub fn get_style_color(&self, name: &str) -> Option<&Color> {
-        self.color.syntax.get(name)
+    pub fn style_color(&self, name: &str) -> Option<Color> {
+        self.color.syntax.get(name).copied()
     }
 
     pub fn completion_color(
@@ -339,14 +382,14 @@ impl LapceConfig {
             _ => "string",
         };
 
-        self.get_style_color(theme_str).cloned()
+        self.style_color(theme_str)
     }
 
     fn resolve_colors(&mut self, default_config: Option<&LapceConfig>) {
         self.color.base = self
             .color_theme
             .base
-            .resolve(default_config.map(|c| &c.color.base));
+            .resolve(default_config.map(|c| &c.color_theme.base));
         self.color.ui = self
             .color_theme
             .resolve_ui_color(&self.color.base, default_config.map(|c| &c.color.ui));
@@ -355,8 +398,8 @@ impl LapceConfig {
             default_config.map(|c| &c.color.syntax),
         );
 
-        let fg = self.get_color(LapceColor::EDITOR_FOREGROUND);
-        let bg = self.get_color(LapceColor::EDITOR_BACKGROUND);
+        let fg = self.color(LapceColor::EDITOR_FOREGROUND);
+        let bg = self.color(LapceColor::EDITOR_BACKGROUND);
         let is_light = fg.r as u32 + fg.g as u32 + fg.b as u32
             > bg.r as u32 + bg.g as u32 + bg.b as u32;
         let high_contrast = self.color_theme.high_contrast.unwrap_or(false);
@@ -395,6 +438,8 @@ impl LapceConfig {
         Some((name.to_lowercase(), (name, config)))
     }
 
+    /// Load the given theme by its contents.  
+    /// Returns `(name, theme fields)`
     fn load_color_theme_from_str(s: &str) -> Option<(String, config::Config)> {
         let config = config::Config::builder()
             .add_source(config::File::from_str(s, config::FileFormat::Toml))
@@ -407,11 +452,12 @@ impl LapceConfig {
 
     fn load_icon_themes(
         disabled_volts: &[VoltID],
+        extra_plugin_paths: &[PathBuf],
     ) -> HashMap<String, (String, config::Config, Option<PathBuf>)> {
         let mut themes = HashMap::new();
 
         for (key, (name, theme, path)) in
-            Self::load_plugin_icon_themes(disabled_volts)
+            Self::load_plugin_icon_themes(disabled_volts, extra_plugin_paths)
         {
             themes.insert(key, (name, theme, Some(path)));
         }
@@ -435,9 +481,10 @@ impl LapceConfig {
 
     fn load_plugin_color_themes(
         disabled_volts: &[VoltID],
+        extra_plugin_paths: &[PathBuf],
     ) -> HashMap<String, (String, config::Config)> {
         let mut themes: HashMap<String, (String, config::Config)> = HashMap::new();
-        for meta in find_all_volts() {
+        for meta in find_all_volts(extra_plugin_paths) {
             if disabled_volts.contains(&meta.id()) {
                 continue;
             }
@@ -456,10 +503,11 @@ impl LapceConfig {
 
     fn load_plugin_icon_themes(
         disabled_volts: &[VoltID],
+        extra_plugin_paths: &[PathBuf],
     ) -> HashMap<String, (String, config::Config, PathBuf)> {
         let mut themes: HashMap<String, (String, config::Config, PathBuf)> =
             HashMap::new();
-        for meta in find_all_volts() {
+        for meta in find_all_volts(extra_plugin_paths) {
             if disabled_volts.contains(&meta.id()) {
                 continue;
             }
@@ -540,19 +588,20 @@ impl LapceConfig {
         });
 
         svg.unwrap_or_else(|| {
-            let name = self.default_icon_theme.ui.get(icon).unwrap();
+            let name = DEFAULT_ICON_THEME_CONFIG.ui.get(icon).unwrap();
             self.svg_store.write().get_default_svg(name)
         })
     }
 
-    pub fn file_svg(&self, path: &Path) -> (String, Option<&Color>) {
+    pub fn files_svg(&self, paths: &[&Path]) -> (String, Option<Color>) {
         let svg = self
             .icon_theme
-            .resolve_path_to_icon(path)
+            .resolve_path_to_icon(paths)
             .and_then(|p| self.svg_store.write().get_svg_on_disk(&p));
+
         if let Some(svg) = svg {
             let color = if self.icon_theme.use_editor_color.unwrap_or(false) {
-                Some(self.get_color(LapceColor::LAPCE_ICON_ACTIVE))
+                Some(self.color(LapceColor::LAPCE_ICON_ACTIVE))
             } else {
                 None
             };
@@ -560,11 +609,15 @@ impl LapceConfig {
         } else {
             (
                 self.ui_svg(LapceIcons::FILE),
-                Some(self.get_color(LapceColor::LAPCE_ICON_ACTIVE)),
+                Some(self.color(LapceColor::LAPCE_ICON_ACTIVE)),
             )
         }
     }
 
+    pub fn file_svg(&self, path: &Path) -> (String, Option<Color>) {
+        self.files_svg(slice::from_ref(&path))
+    }
+
     pub fn symbol_svg(&self, kind: &SymbolKind) -> Option<String> {
         let kind_str = match *kind {
             SymbolKind::ARRAY => LapceIcons::SYMBOL_KIND_ARRAY,
@@ -644,37 +697,37 @@ impl LapceConfig {
 
     pub fn terminal_get_color(
         &self,
-        color: &alacritty_terminal::ansi::Color,
+        color: &alacritty_terminal::vte::ansi::Color,
         colors: &alacritty_terminal::term::color::Colors,
     ) -> Color {
         match color {
-            alacritty_terminal::ansi::Color::Named(color) => {
+            alacritty_terminal::vte::ansi::Color::Named(color) => {
                 self.terminal_get_named_color(color)
             }
-            alacritty_terminal::ansi::Color::Spec(rgb) => {
+            alacritty_terminal::vte::ansi::Color::Spec(rgb) => {
                 Color::rgb8(rgb.r, rgb.g, rgb.b)
             }
-            alacritty_terminal::ansi::Color::Indexed(index) => {
+            alacritty_terminal::vte::ansi::Color::Indexed(index) => {
                 if let Some(rgb) = colors[*index as usize] {
                     return Color::rgb8(rgb.r, rgb.g, rgb.b);
                 }
-                const NAMED_COLORS: [alacritty_terminal::ansi::NamedColor; 16] = [
-                    alacritty_terminal::ansi::NamedColor::Black,
-                    alacritty_terminal::ansi::NamedColor::Red,
-                    alacritty_terminal::ansi::NamedColor::Green,
-                    alacritty_terminal::ansi::NamedColor::Yellow,
-                    alacritty_terminal::ansi::NamedColor::Blue,
-                    alacritty_terminal::ansi::NamedColor::Magenta,
-                    alacritty_terminal::ansi::NamedColor::Cyan,
-                    alacritty_terminal::ansi::NamedColor::White,
-                    alacritty_terminal::ansi::NamedColor::BrightBlack,
-                    alacritty_terminal::ansi::NamedColor::BrightRed,
-                    alacritty_terminal::ansi::NamedColor::BrightGreen,
-                    alacritty_terminal::ansi::NamedColor::BrightYellow,
-                    alacritty_terminal::ansi::NamedColor::BrightBlue,
-                    alacritty_terminal::ansi::NamedColor::BrightMagenta,
-                    alacritty_terminal::ansi::NamedColor::BrightCyan,
-                    alacritty_terminal::ansi::NamedColor::BrightWhite,
+                const NAMED_COLORS: [alacritty_terminal::vte::ansi::NamedColor; 16] = [
+                    alacritty_terminal::vte::ansi::NamedColor::Black,
+                    alacritty_terminal::vte::ansi::NamedColor::Red,
+                    alacritty_terminal::vte::ansi::NamedColor::Green,
+                    alacritty_terminal::vte::ansi::NamedColor::Yellow,
+                    alacritty_terminal::vte::ansi::NamedColor::Blue,
+                    alacritty_terminal::vte::ansi::NamedColor::Magenta,
+                    alacritty_terminal::vte::ansi::NamedColor::Cyan,
+                    alacritty_terminal::vte::ansi::NamedColor::White,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightBlack,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightRed,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightGreen,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightYellow,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightBlue,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightMagenta,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightCyan,
+                    alacritty_terminal::vte::ansi::NamedColor::BrightWhite,
                 ];
                 if (*index as usize) < NAMED_COLORS.len() {
                     self.terminal_get_named_color(&NAMED_COLORS[*index as usize])
@@ -687,98 +740,98 @@ impl LapceConfig {
 
     fn terminal_get_named_color(
         &self,
-        color: &alacritty_terminal::ansi::NamedColor,
+        color: &alacritty_terminal::vte::ansi::NamedColor,
     ) -> Color {
         let (color, alpha) = match color {
-            alacritty_terminal::ansi::NamedColor::Cursor => {
+            alacritty_terminal::vte::ansi::NamedColor::Cursor => {
                 (LapceColor::TERMINAL_CURSOR, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Foreground => {
+            alacritty_terminal::vte::ansi::NamedColor::Foreground => {
                 (LapceColor::TERMINAL_FOREGROUND, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Background => {
+            alacritty_terminal::vte::ansi::NamedColor::Background => {
                 (LapceColor::TERMINAL_BACKGROUND, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Blue => {
+            alacritty_terminal::vte::ansi::NamedColor::Blue => {
                 (LapceColor::TERMINAL_BLUE, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Green => {
+            alacritty_terminal::vte::ansi::NamedColor::Green => {
                 (LapceColor::TERMINAL_GREEN, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Yellow => {
+            alacritty_terminal::vte::ansi::NamedColor::Yellow => {
                 (LapceColor::TERMINAL_YELLOW, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Red => {
+            alacritty_terminal::vte::ansi::NamedColor::Red => {
                 (LapceColor::TERMINAL_RED, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::White => {
+            alacritty_terminal::vte::ansi::NamedColor::White => {
                 (LapceColor::TERMINAL_WHITE, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Black => {
+            alacritty_terminal::vte::ansi::NamedColor::Black => {
                 (LapceColor::TERMINAL_BLACK, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Cyan => {
+            alacritty_terminal::vte::ansi::NamedColor::Cyan => {
                 (LapceColor::TERMINAL_CYAN, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::Magenta => {
+            alacritty_terminal::vte::ansi::NamedColor::Magenta => {
                 (LapceColor::TERMINAL_MAGENTA, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightBlue => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightBlue => {
                 (LapceColor::TERMINAL_BRIGHT_BLUE, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightGreen => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightGreen => {
                 (LapceColor::TERMINAL_BRIGHT_GREEN, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightYellow => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightYellow => {
                 (LapceColor::TERMINAL_BRIGHT_YELLOW, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightRed => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightRed => {
                 (LapceColor::TERMINAL_BRIGHT_RED, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightWhite => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightWhite => {
                 (LapceColor::TERMINAL_BRIGHT_WHITE, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightBlack => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightBlack => {
                 (LapceColor::TERMINAL_BRIGHT_BLACK, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightCyan => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightCyan => {
                 (LapceColor::TERMINAL_BRIGHT_CYAN, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightMagenta => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightMagenta => {
                 (LapceColor::TERMINAL_BRIGHT_MAGENTA, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::BrightForeground => {
+            alacritty_terminal::vte::ansi::NamedColor::BrightForeground => {
                 (LapceColor::TERMINAL_FOREGROUND, 1.0)
             }
-            alacritty_terminal::ansi::NamedColor::DimBlack => {
+            alacritty_terminal::vte::ansi::NamedColor::DimBlack => {
                 (LapceColor::TERMINAL_BLACK, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimRed => {
+            alacritty_terminal::vte::ansi::NamedColor::DimRed => {
                 (LapceColor::TERMINAL_RED, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimGreen => {
+            alacritty_terminal::vte::ansi::NamedColor::DimGreen => {
                 (LapceColor::TERMINAL_GREEN, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimYellow => {
+            alacritty_terminal::vte::ansi::NamedColor::DimYellow => {
                 (LapceColor::TERMINAL_YELLOW, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimBlue => {
+            alacritty_terminal::vte::ansi::NamedColor::DimBlue => {
                 (LapceColor::TERMINAL_BLUE, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimMagenta => {
+            alacritty_terminal::vte::ansi::NamedColor::DimMagenta => {
                 (LapceColor::TERMINAL_MAGENTA, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimCyan => {
+            alacritty_terminal::vte::ansi::NamedColor::DimCyan => {
                 (LapceColor::TERMINAL_CYAN, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimWhite => {
+            alacritty_terminal::vte::ansi::NamedColor::DimWhite => {
                 (LapceColor::TERMINAL_WHITE, 0.66)
             }
-            alacritty_terminal::ansi::NamedColor::DimForeground => {
+            alacritty_terminal::vte::ansi::NamedColor::DimForeground => {
                 (LapceColor::TERMINAL_FOREGROUND, 0.66)
             }
         };
-        (*self.get_color(color)).with_alpha_factor(alpha)
+        self.color(color).with_alpha_factor(alpha)
     }
 
     /// Get the dropdown information for the specific setting, used for the settings UI.
@@ -802,6 +855,18 @@ impl LapceConfig {
                     .unwrap_or(0),
                 items: self.icon_theme_list.clone(),
             }),
+            ("editor", "wrap-style") => Some(DropdownInfo {
+                // TODO: it would be better to have the text not be the default kebab-case when
+                // displayed in settings, but we would need to map back from the dropdown's value
+                // or index.
+                active_index: self
+                    .wrap_style_list
+                    .iter()
+                    .flat_map(|w| WrapStyle::try_from_str(w))
+                    .position(|w| w == self.editor.wrap_style)
+                    .unwrap_or(0),
+                items: self.wrap_style_list.clone(),
+            }),
             ("ui", "tab-close-button") => Some(DropdownInfo {
                 active_index: self.ui.tab_close_button as usize,
                 items: ui::TabCloseButton::VARIANTS
@@ -868,6 +933,16 @@ impl LapceConfig {
         key: &str,
         value: toml_edit::Value,
     ) -> Option<()> {
+        // TODO: This is a hack to fix the fact that terminal default profile is saved in a
+        // different manner than other fields. As it is per-operating-system.
+        // Thus we have to instead set the terminal.default-profile.{OS}
+        // It would be better to not need a special hack.
+        let (parent, key) = if parent == "terminal" && key == "default-profile" {
+            ("terminal.default-profile", std::env::consts::OS)
+        } else {
+            (parent, key)
+        };
+
         let mut main_table = Self::get_file_table().unwrap_or_default();
 
         // Find the container table
