@@ -1,3 +1,4 @@
+use floem::views::editor::text::RenderWhitespace;
 use serde::{Deserialize, Serialize};
 use structdesc::FieldNames;
 
@@ -14,6 +15,45 @@ pub enum ClickMode {
     DoubleClickAll,
 }
 
+#[derive(Debug, Clone, Copy, Serialize, Deserialize, Default, PartialEq)]
+#[serde(rename_all = "kebab-case")]
+pub enum WrapStyle {
+    /// No wrapping
+    None,
+    /// Wrap at the editor width
+    #[default]
+    EditorWidth,
+    // /// Wrap at the wrap-column
+    // WrapColumn,
+    /// Wrap at a specific width
+    WrapWidth,
+}
+impl WrapStyle {
+    pub fn as_str(&self) -> &'static str {
+        match self {
+            WrapStyle::None => "none",
+            WrapStyle::EditorWidth => "editor-width",
+            // WrapStyle::WrapColumn => "wrap-column",
+            WrapStyle::WrapWidth => "wrap-width",
+        }
+    }
+
+    pub fn try_from_str(s: &str) -> Option<Self> {
+        match s {
+            "none" => Some(WrapStyle::None),
+            "editor-width" => Some(WrapStyle::EditorWidth),
+            // "wrap-column" => Some(WrapStyle::WrapColumn),
+            "wrap-width" => Some(WrapStyle::WrapWidth),
+            _ => None,
+        }
+    }
+}
+impl ToString for WrapStyle {
+    fn to_string(&self) -> String {
+        self.as_str().to_string()
+    }
+}
+
 #[derive(FieldNames, Debug, Clone, Deserialize, Serialize, Default)]
 #[serde(rename_all = "kebab-case")]
 pub struct EditorConfig {
@@ -43,6 +83,12 @@ pub struct EditorConfig {
         desc = "Set the minimum number of visible lines above and below the cursor"
     )]
     pub cursor_surrounding_lines: usize,
+    #[field_names(desc = "The kind of wrapping to perform")]
+    pub wrap_style: WrapStyle,
+    // #[field_names(desc = "The number of columns to wrap at")]
+    // pub wrap_column: usize,
+    #[field_names(desc = "The number of pixels to wrap at")]
+    pub wrap_width: usize,
     #[field_names(
         desc = "Show code context like functions and classes at the top of editor when scroll"
     )]
@@ -80,6 +126,11 @@ pub struct EditorConfig {
     )]
     pub format_on_save: bool,
 
+    #[field_names(
+        desc = "Whether newlines should be automatically converted to the current line ending"
+    )]
+    pub normalize_line_endings: bool,
+
     #[field_names(desc = "If matching brackets are highlighted")]
     pub highlight_matching_brackets: bool,
 
@@ -103,6 +154,12 @@ pub struct EditorConfig {
         desc = "Whether error lens should go to the end of view line, or only to the end of the diagnostic"
     )]
     pub error_lens_end_of_line: bool,
+    #[field_names(
+        desc = "Whether error lens should extend over multiple lines. If false, it will have newlines stripped."
+    )]
+    pub error_lens_multiline: bool,
+    // TODO: Error lens but put entirely on the next line
+    // TODO: error lens with indentation matching.
     #[field_names(
         desc = "Set error lens font family. If empty, it uses the inlay hint font family."
     )]
@@ -115,6 +172,8 @@ pub struct EditorConfig {
         desc = "If the editor should display the completion item as phantom text"
     )]
     pub enable_completion_lens: bool,
+    #[field_names(desc = "If the editor should display inline completions")]
+    pub enable_inline_completion: bool,
     #[field_names(
         desc = "Set completion lens font family. If empty, it uses the inlay hint font family."
     )]
@@ -138,7 +197,7 @@ pub struct EditorConfig {
     #[field_names(
         desc = "How the editor should render whitespace characters.\nOptions: none, all, boundary, trailing."
     )]
-    pub render_whitespace: String,
+    pub render_whitespace: RenderWhitespace,
     #[field_names(desc = "Whether the editor show indent guide.")]
     pub show_indent_guide: bool,
     #[field_names(
@@ -163,6 +222,14 @@ pub struct EditorConfig {
         desc = "Set the default number of visible lines above and below the diff block (-1 for infinite)"
     )]
     pub diff_context_lines: i32,
+    #[field_names(desc = "Whether the editor colorizes brackets")]
+    pub bracket_pair_colorization: bool,
+    #[field_names(desc = "Bracket colorization Limit")]
+    pub bracket_colorization_limit: u64,
+    #[field_names(
+        desc = "Glob patterns for excluding files and folders (in file explorer)"
+    )]
+    pub files_exclude: String,
 }
 
 impl EditorConfig {
