@@ -10,23 +10,24 @@ use floem::{
         Color,
     },
     reactive::{create_memo, create_rw_signal, Memo, ReadSignal, RwSignal, Scope},
+    views::editor::id::EditorId,
 };
 use lapce_rpc::plugin::VoltID;
 use serde::{Deserialize, Serialize};
 
 use crate::{
     config::{color::LapceColor, icon::LapceIcons, LapceConfig},
-    doc::{DocContent, Document},
+    doc::{Doc, DocContent},
     editor::{
         diff::{DiffEditorData, DiffEditorInfo},
         location::EditorLocation,
         EditorData, EditorInfo,
     },
     id::{
-        DiffEditorId, EditorId, EditorTabId, KeymapId, SettingsId, SplitId,
+        DiffEditorId, EditorTabId, KeymapId, SettingsId, SplitId,
         ThemeColorSettingsId, VoltViewId,
     },
-    main_split::MainSplitData,
+    main_split::{Editors, MainSplitData},
     plugin::PluginData,
     window_tab::WindowTabData,
 };
@@ -49,8 +50,8 @@ impl EditorTabChildInfo {
     ) -> EditorTabChild {
         match &self {
             EditorTabChildInfo::Editor(editor_info) => {
-                let editor_data = editor_info.to_data(data, editor_tab_id);
-                EditorTabChild::Editor(editor_data.editor_id)
+                let editor_id = editor_info.to_data(data, editor_tab_id);
+                EditorTabChild::Editor(editor_id)
             }
             EditorTabChildInfo::DiffEditor(diff_editor_info) => {
                 let diff_editor_data = diff_editor_info.to_data(data, editor_tab_id);
@@ -120,14 +121,8 @@ impl EditorTabInfo {
 }
 
 pub enum EditorTabChildSource {
-    Editor {
-        path: PathBuf,
-        doc: Rc<Document>,
-    },
-    DiffEditor {
-        left: Rc<Document>,
-        right: Rc<Document>,
-    },
+    Editor { path: PathBuf, doc: Rc<Doc> },
+    DiffEditor { left: Rc<Doc>, right: Rc<Doc> },
     NewFileEditor,
     Settings,
     ThemeColorSettings,
@@ -176,9 +171,7 @@ impl EditorTabChild {
                 let editor_data = data
                     .main_split
                     .editors
-                    .get_untracked()
-                    .get(editor_id)
-                    .cloned()
+                    .editor_untracked(*editor_id)
                     .unwrap();
                 EditorTabChildInfo::Editor(editor_data.editor_info(data))
             }
@@ -203,7 +196,7 @@ impl EditorTabChild {
 
     pub fn view_info(
         &self,
-        editors: RwSignal<im::HashMap<EditorId, Rc<EditorData>>>,
+        editors: Editors,
         diff_editors: RwSignal<im::HashMap<DiffEditorId, DiffEditorData>>,
         plugin: PluginData,
         config: ReadSignal<Arc<LapceConfig>>,
@@ -211,10 +204,9 @@ impl EditorTabChild {
         match self.clone() {
             EditorTabChild::Editor(editor_id) => create_memo(move |_| {
                 let config = config.get();
-                let editor_data =
-                    editors.with(|editors| editors.get(&editor_id).cloned());
+                let editor_data = editors.editor(editor_id);
                 let path = if let Some(editor_data) = editor_data {
-                    let doc = editor_data.view.doc.get();
+                    let doc = editor_data.doc_signal().get();
                     let (content, is_pristine, confirmed) = (
                         doc.content.get(),
                         doc.buffer.with(|b| b.is_pristine()),
@@ -238,7 +230,7 @@ impl EditorTabChild {
                         let (svg, color) = config.file_svg(&path);
                         (
                             svg,
-                            color.cloned(),
+                            color,
                             path.file_name()
                                 .unwrap_or_default()
                                 .to_string_lossy()
@@ -249,7 +241,7 @@ impl EditorTabChild {
                     }
                     None => (
                         config.ui_svg(LapceIcons::FILE),
-                        Some(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE)),
+                        Some(config.color(LapceColor::LAPCE_ICON_ACTIVE)),
                         "local".to_string(),
                         create_rw_signal(true),
                         true,
@@ -267,41 +259,68 @@ impl EditorTabChild {
                 let config = config.get();
                 let diff_editor_data = diff_editors
                     .with(|diff_editors| diff_editors.get(&diff_editor_id).cloned());
-                let confirmed = diff_editor_data.as_ref().map(|d| d.right.confirmed);
-                let path = if let Some(diff_editor_data) = diff_editor_data {
-                    let (content, is_pristine) =
-                        diff_editor_data.right.view.doc.with(|doc| {
-                            (doc.content.get(), doc.buffer.with(|b| b.is_pristine()))
-                        });
-                    match content {
-                        DocContent::File { path, .. } => Some((path, is_pristine)),
-                        DocContent::Local => None,
-                        DocContent::History(_) => None,
-                        DocContent::Scratch { name, .. } => {
-                            Some((PathBuf::from(name), is_pristine))
-                        }
-                    }
-                } else {
-                    None
-                };
-                let (icon, color, path, is_pristine) = match path {
-                    Some((path, is_pritine)) => {
+                let confirmed = diff_editor_data.as_ref().map(|d| d.confirmed);
+
+                let info = diff_editor_data
+                    .map(|diff_editor_data| {
+                        [diff_editor_data.left, diff_editor_data.right].map(|data| {
+                            let (content, is_pristine) =
+                                data.doc_signal().with(|doc| {
+                                    (
+                                        doc.content.get(),
+                                        doc.buffer.with(|b| b.is_pristine()),
+                                    )
+                                });
+                            match content {
+                                DocContent::File { path, .. } => {
+                                    Some((path, is_pristine))
+                                }
+                                DocContent::Local => None,
+                                DocContent::History(_) => None,
+                                DocContent::Scratch { name, .. } => {
+                                    Some((PathBuf::from(name), is_pristine))
+                                }
+                            }
+                        })
+                    })
+                    .unwrap_or([None, None]);
+
+                let (icon, color, path, is_pristine) = match info {
+                    [Some((path, is_pristine)), None]
+                    | [None, Some((path, is_pristine))] => {
                         let (svg, color) = config.file_svg(&path);
                         (
                             svg,
-                            color.cloned(),
+                            color,
                             format!(
                                 "{} (Diff)",
                                 path.file_name()
                                     .unwrap_or_default()
                                     .to_string_lossy()
                             ),
-                            is_pritine,
+                            is_pristine,
                         )
                     }
-                    None => (
+                    [Some((left_path, left_is_pristine)), Some((right_path, right_is_pristine))] =>
+                    {
+                        let (svg, color) =
+                            config.files_svg(&[&left_path, &right_path]);
+                        let [left_file_name, right_file_name] =
+                            [&left_path, &right_path].map(|path| {
+                                path.file_name()
+                                    .unwrap_or_default()
+                                    .to_string_lossy()
+                            });
+                        (
+                            svg,
+                            color,
+                            format!("{left_file_name} - {right_file_name} (Diff)"),
+                            left_is_pristine && right_is_pristine,
+                        )
+                    }
+                    [None, None] => (
                         config.ui_svg(LapceIcons::FILE),
-                        Some(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE)),
+                        Some(config.color(LapceColor::LAPCE_ICON_ACTIVE)),
                         "local".to_string(),
                         true,
                     ),
@@ -318,7 +337,7 @@ impl EditorTabChild {
                 let config = config.get();
                 EditorTabChildViewInfo {
                     icon: config.ui_svg(LapceIcons::SETTINGS),
-                    color: Some(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE)),
+                    color: Some(config.color(LapceColor::LAPCE_ICON_ACTIVE)),
                     path: "Settings".to_string(),
                     confirmed: None,
                     is_pristine: true,
@@ -328,7 +347,7 @@ impl EditorTabChild {
                 let config = config.get();
                 EditorTabChildViewInfo {
                     icon: config.ui_svg(LapceIcons::SYMBOL_COLOR),
-                    color: Some(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE)),
+                    color: Some(config.color(LapceColor::LAPCE_ICON_ACTIVE)),
                     path: "Theme Colors".to_string(),
                     confirmed: None,
                     is_pristine: true,
@@ -338,7 +357,7 @@ impl EditorTabChild {
                 let config = config.get();
                 EditorTabChildViewInfo {
                     icon: config.ui_svg(LapceIcons::KEYBOARD),
-                    color: Some(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE)),
+                    color: Some(config.color(LapceColor::LAPCE_ICON_ACTIVE)),
                     path: "Keyboard Shortcuts".to_string(),
                     confirmed: None,
                     is_pristine: true,
@@ -361,7 +380,7 @@ impl EditorTabChild {
                     .unwrap_or_else(|| id.name.clone());
                 EditorTabChildViewInfo {
                     icon: config.ui_svg(LapceIcons::EXTENSIONS),
-                    color: Some(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE)),
+                    color: Some(config.color(LapceColor::LAPCE_ICON_ACTIVE)),
                     path: display_name,
                     confirmed: None,
                     is_pristine: true,
@@ -387,24 +406,21 @@ pub struct EditorTabData {
 impl EditorTabData {
     pub fn get_editor(
         &self,
-        editors: &im::HashMap<EditorId, Rc<EditorData>>,
+        editors: Editors,
         path: &Path,
-    ) -> Option<(usize, Rc<EditorData>)> {
+    ) -> Option<(usize, EditorData)> {
         for (i, child) in self.children.iter().enumerate() {
             if let (_, _, EditorTabChild::Editor(editor_id)) = child {
-                if let Some(editor) = editors.get(editor_id) {
-                    let is_path =
-                        editor.view.doc.get_untracked().content.with_untracked(
-                            |content| {
-                                if let DocContent::File { path: p, .. } = content {
-                                    p == path
-                                } else {
-                                    false
-                                }
-                            },
-                        );
+                if let Some(editor) = editors.editor_untracked(*editor_id) {
+                    let is_path = editor.doc().content.with_untracked(|content| {
+                        if let DocContent::File { path: p, .. } = content {
+                            p == path
+                        } else {
+                            false
+                        }
+                    });
                     if is_path {
-                        return Some((i, editor.clone()));
+                        return Some((i, editor));
                     }
                 }
             }
@@ -414,13 +430,13 @@ impl EditorTabData {
 
     pub fn get_unconfirmed_editor_tab_child(
         &self,
-        editors: &im::HashMap<EditorId, Rc<EditorData>>,
+        editors: Editors,
         diff_editors: &im::HashMap<EditorId, DiffEditorData>,
     ) -> Option<(usize, EditorTabChild)> {
         for (i, (_, _, child)) in self.children.iter().enumerate() {
             match child {
                 EditorTabChild::Editor(editor_id) => {
-                    if let Some(editor) = editors.get(editor_id) {
+                    if let Some(editor) = editors.editor_untracked(*editor_id) {
                         let confirmed = editor.confirmed.get_untracked();
                         if !confirmed {
                             return Some((i, child.clone()));
@@ -429,11 +445,8 @@ impl EditorTabData {
                 }
                 EditorTabChild::DiffEditor(diff_editor_id) => {
                     if let Some(diff_editor) = diff_editors.get(diff_editor_id) {
-                        let left_confirmed =
-                            diff_editor.left.confirmed.get_untracked();
-                        let right_confirmed =
-                            diff_editor.right.confirmed.get_untracked();
-                        if !left_confirmed && !right_confirmed {
+                        let confirmed = diff_editor.confirmed.get_untracked();
+                        if !confirmed {
                             return Some((i, child.clone()));
                         }
                     }
@@ -444,24 +457,6 @@ impl EditorTabData {
         None
     }
 
-    pub fn get_unconfirmed_editor(
-        &self,
-        editors: &im::HashMap<EditorId, RwSignal<EditorData>>,
-    ) -> Option<(usize, RwSignal<EditorData>)> {
-        for (i, child) in self.children.iter().enumerate() {
-            if let (_, _, EditorTabChild::Editor(editor_id)) = child {
-                if let Some(editor) = editors.get(editor_id) {
-                    let e = editor.get_untracked();
-                    let confirmed = e.confirmed.get_untracked();
-                    if !confirmed {
-                        return Some((i, *editor));
-                    }
-                }
-            }
-        }
-        None
-    }
-
     pub fn tab_info(&self, data: &WindowTabData) -> EditorTabInfo {
         let info = EditorTabInfo {
             active: self.active,
