@@ -5,16 +5,16 @@ use floem::{
         kurbo::{Point, Rect},
         Color,
     },
-    prop_extracter,
+    prop_extractor,
     reactive::create_effect,
     style::{FontFamily, FontSize, LineHeight, Style, TextColor},
-    taffy::prelude::Node,
-    view::{View, ViewData},
+    taffy::prelude::NodeId,
+    view::{AnyWidget, View, ViewData, Widget},
     Renderer,
 };
 
-prop_extracter! {
-    Extracter {
+prop_extractor! {
+    Extractor {
         color: TextColor,
         font_size: FontSize,
         font_family: FontFamily,
@@ -37,17 +37,17 @@ pub fn focus_text(
 
     create_effect(move |_| {
         let new_text = text();
-        id.update_state(FocusTextState::Text(new_text), false);
+        id.update_state(FocusTextState::Text(new_text));
     });
 
     create_effect(move |_| {
         let focus_color = focus_color();
-        id.update_state(FocusTextState::FocusColor(focus_color), false);
+        id.update_state(FocusTextState::FocusColor(focus_color));
     });
 
     create_effect(move |_| {
         let focus_indices = focus_indices();
-        id.update_state(FocusTextState::FocusIndices(focus_indices), false);
+        id.update_state(FocusTextState::FocusIndices(focus_indices));
     });
 
     FocusText {
@@ -72,11 +72,11 @@ pub struct FocusText {
     text_layout: Option<TextLayout>,
     focus_color: Color,
     focus_indices: Vec<usize>,
-    text_node: Option<Node>,
+    text_node: Option<NodeId>,
     available_text: Option<String>,
     available_width: Option<f32>,
     available_text_layout: Option<TextLayout>,
-    style: Extracter,
+    style: Extractor,
 }
 
 impl FocusText {
@@ -165,10 +165,19 @@ impl FocusText {
 }
 
 impl View for FocusText {
-    fn id(&self) -> floem::id::Id {
-        self.id
+    fn view_data(&self) -> &ViewData {
+        &self.data
     }
 
+    fn view_data_mut(&mut self) -> &mut ViewData {
+        &mut self.data
+    }
+
+    fn build(self) -> AnyWidget {
+        Box::new(self)
+    }
+}
+impl Widget for FocusText {
     fn view_data(&self) -> &ViewData {
         &self.data
     }
@@ -209,7 +218,7 @@ impl View for FocusText {
     fn layout(
         &mut self,
         cx: &mut floem::context::LayoutCx,
-    ) -> floem::taffy::prelude::Node {
+    ) -> floem::taffy::prelude::NodeId {
         cx.layout_node(self.id, true, |cx| {
             if self.text_layout.is_none() {
                 self.set_text_layout();
