@@ -9,8 +9,8 @@ use floem::{
     style::CursorStyle,
     view::View,
     views::{
-        container, label, list, scroll, stack, text, virtual_list, Decorators,
-        VirtualListDirection, VirtualListItemSize,
+        container, dyn_stack, label, scroll, stack, text, virtual_stack, Decorators,
+        VirtualDirection, VirtualItemSize,
     },
 };
 use lapce_core::mode::Modes;
@@ -18,10 +18,12 @@ use lapce_core::mode::Modes;
 use crate::{
     command::LapceCommand,
     config::{color::LapceColor, LapceConfig},
-    editor::EditorData,
-    id::EditorId,
-    keypress::{keymap::KeyMap, KeyPress, KeyPressData},
-    text_input::text_input,
+    keypress::{
+        keymap::{KeyMap, KeyMapPress},
+        KeyPressData,
+    },
+    main_split::Editors,
+    text_input::TextInputBuilder,
     window_tab::CommonData,
 };
 
@@ -29,10 +31,10 @@ use crate::{
 pub struct KeymapPicker {
     cmd: RwSignal<Option<LapceCommand>>,
     keymap: RwSignal<Option<KeyMap>>,
-    keys: RwSignal<Vec<(KeyPress, bool)>>,
+    keys: RwSignal<Vec<(KeyMapPress, bool)>>,
 }
 
-pub fn keymap_view(common: Rc<CommonData>) -> impl View {
+pub fn keymap_view(editors: Editors, common: Rc<CommonData>) -> impl View {
     let config = common.config;
     let keypress = common.keypress;
     let ui_line_height_memo = common.ui_line_height;
@@ -45,8 +47,8 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
     };
 
     let cx = Scope::current();
-    let editor = EditorData::new_local(cx, EditorId::next(), common.clone());
-    let doc = editor.view.doc;
+    let text_input_view = TextInputBuilder::new().build(cx, editors, common.clone());
+    let doc = text_input_view.doc_signal();
 
     let items = move || {
         let doc = doc.get();
@@ -57,17 +59,33 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
             .iter()
             .filter_map(|keymap| {
                 let cmd = keypress.commands.get(&keymap.command).cloned()?;
-                let match_pattern =
-                    cmd.kind.str().replace('_', " ").contains(&pattern)
-                        || cmd
-                            .kind
-                            .desc()
-                            .map(|desc| desc.to_lowercase().contains(&pattern))
-                            .unwrap_or(false);
-                if !match_pattern {
-                    return None;
+
+                let cmd_name_contains_pattern =
+                    cmd.kind.str().replace('_', " ").contains(&pattern);
+                let cmd_desc_contains_pattern = cmd
+                    .kind
+                    .desc()
+                    .map(|desc| desc.to_lowercase().contains(&pattern))
+                    .unwrap_or(false);
+                let shortcut_contains_pattern = keymap
+                    .key
+                    .iter()
+                    .any(|k| k.label().trim().to_lowercase().contains(&pattern));
+                let when_contains_pattern = keymap
+                    .when
+                    .as_ref()
+                    .map(|when| when.to_lowercase().contains(&pattern))
+                    .unwrap_or(false);
+
+                if cmd_name_contains_pattern
+                    || cmd_desc_contains_pattern
+                    || shortcut_contains_pattern
+                    || when_contains_pattern
+                {
+                    Some((cmd, Some(keymap.clone())))
+                } else {
+                    None
                 }
-                Some((cmd, Some(keymap.clone())))
             })
             .collect::<im::Vector<(LapceCommand, Option<KeyMap>)>>();
         items.extend(keypress.commands_without_keymap.iter().filter_map(|cmd| {
@@ -115,13 +133,11 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                         .flex_basis(0.0)
                         .flex_grow(1.0)
                         .border_right(1.0)
-                        .border_color(
-                            *config.get().get_color(LapceColor::LAPCE_BORDER),
-                        )
+                        .border_color(config.get().color(LapceColor::LAPCE_BORDER))
                 }),
                 {
                     let keymap = keymap.clone();
-                    list(
+                    dyn_stack(
                         move || {
                             keymap
                                 .as_ref()
@@ -129,7 +145,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                                     keymap
                                         .key
                                         .iter()
-                                        .map(|key| key.label().trim().to_string())
+                                        .map(|key| key.label())
                                         .filter(|l| !l.is_empty())
                                         .collect::<Vec<String>>()
                                 })
@@ -144,9 +160,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                                     .border(1.0)
                                     .border_radius(3.0)
                                     .border_color(
-                                        *config
-                                            .get()
-                                            .get_color(LapceColor::LAPCE_BORDER),
+                                        config.get().color(LapceColor::LAPCE_BORDER),
                                     )
                             })
                         },
@@ -158,7 +172,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                             .height_pct(100.0)
                             .border_right(1.0)
                             .border_color(
-                                *config.get().get_color(LapceColor::LAPCE_BORDER),
+                                config.get().color(LapceColor::LAPCE_BORDER),
                             )
                     })
                 },
@@ -184,7 +198,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                                 .collect::<Vec<String>>()
                         })
                         .unwrap_or_default();
-                    list(
+                    dyn_stack(
                         move || modes.clone(),
                         |m| m.clone(),
                         move |mode| {
@@ -195,9 +209,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                                     .border(1.0)
                                     .border_radius(3.0)
                                     .border_color(
-                                        *config
-                                            .get()
-                                            .get_color(LapceColor::LAPCE_BORDER),
+                                        config.get().color(LapceColor::LAPCE_BORDER),
                                     )
                             })
                         },
@@ -209,7 +221,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                             .height_pct(100.0)
                             .border_right(1.0)
                             .border_color(
-                                *config.get().get_color(LapceColor::LAPCE_BORDER),
+                                config.get().color(LapceColor::LAPCE_BORDER),
                             )
                             .apply_if(!modal.get(), |s| s.hide())
                     })
@@ -260,27 +272,24 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                     .height(ui_line_height() as f32)
                     .width_pct(100.0)
                     .apply_if(i % 2 > 0, |s| {
-                        s.background(
-                            *config.get_color(LapceColor::EDITOR_CURRENT_LINE),
-                        )
+                        s.background(config.color(LapceColor::EDITOR_CURRENT_LINE))
                     })
                     .border_bottom(1.0)
-                    .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
+                    .border_color(config.color(LapceColor::LAPCE_BORDER))
             })
         };
 
     stack((
         container(
-            text_input(editor, || false)
+            text_input_view
                 .placeholder(|| "Search Key Bindings".to_string())
                 .keyboard_navigatable()
+                .request_focus(|| {})
                 .style(move |s| {
                     s.width_pct(100.0)
                         .border_radius(6.0)
                         .border(1.0)
-                        .border_color(
-                            *config.get().get_color(LapceColor::LAPCE_BORDER),
-                        )
+                        .border_color(config.get().color(LapceColor::LAPCE_BORDER))
                 }),
         )
         .style(|s| s.padding_bottom(10.0).width_pct(100.0)),
@@ -295,7 +304,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                     .flex_basis(0.0)
                     .flex_grow(1.0)
                     .border_right(1.0)
-                    .border_color(*config.get().get_color(LapceColor::LAPCE_BORDER))
+                    .border_color(config.get().color(LapceColor::LAPCE_BORDER))
             }),
             text("Key Binding").style(move |s| {
                 s.width(200.0)
@@ -303,7 +312,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                     .padding_horiz(10.0)
                     .height_pct(100.0)
                     .border_right(1.0)
-                    .border_color(*config.get().get_color(LapceColor::LAPCE_BORDER))
+                    .border_color(config.get().color(LapceColor::LAPCE_BORDER))
             }),
             text("Modes").style(move |s| {
                 s.width(200.0)
@@ -311,7 +320,7 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                     .padding_horiz(10.0)
                     .height_pct(100.0)
                     .border_right(1.0)
-                    .border_color(*config.get().get_color(LapceColor::LAPCE_BORDER))
+                    .border_color(config.get().color(LapceColor::LAPCE_BORDER))
                     .apply_if(!modal.get(), |s| s.hide())
             }),
             container(text("When").style(move |s| {
@@ -332,14 +341,14 @@ pub fn keymap_view(common: Rc<CommonData>) -> impl View {
                 .width_pct(100.0)
                 .border_top(1.0)
                 .border_bottom(1.0)
-                .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
-                .background(*config.get_color(LapceColor::EDITOR_CURRENT_LINE))
+                .border_color(config.color(LapceColor::LAPCE_BORDER))
+                .background(config.color(LapceColor::EDITOR_CURRENT_LINE))
         }),
         container(
             scroll(
-                virtual_list(
-                    VirtualListDirection::Vertical,
-                    VirtualListItemSize::Fixed(Box::new(ui_line_height)),
+                virtual_stack(
+                    VirtualDirection::Vertical,
+                    VirtualItemSize::Fixed(Box::new(ui_line_height)),
                     items,
                     |(i, (cmd, keymap)): &(
                         usize,
@@ -384,13 +393,13 @@ fn keyboard_picker_view(
                         .unwrap_or_default()
                 })
             }),
-            list(
+            dyn_stack(
                 move || {
                     picker
                         .keys
                         .get()
                         .iter()
-                        .map(|(key, _)| key.label().trim().to_string())
+                        .map(|(key, _)| key.label())
                         .filter(|l| !l.is_empty())
                         .enumerate()
                         .collect::<Vec<(usize, String)>>()
@@ -404,7 +413,7 @@ fn keyboard_picker_view(
                             .border(1.0)
                             .border_radius(6.0)
                             .border_color(
-                                *config.get().get_color(LapceColor::LAPCE_BORDER),
+                                config.get().color(LapceColor::LAPCE_BORDER),
                             )
                     })
                 },
@@ -418,8 +427,8 @@ fn keyboard_picker_view(
                     .height(ui_line_height.get() as f32 + 16.0)
                     .border(1.0)
                     .border_radius(6.0)
-                    .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
-                    .background(*config.get_color(LapceColor::EDITOR_BACKGROUND))
+                    .border_color(config.color(LapceColor::LAPCE_BORDER))
+                    .background(config.color(LapceColor::EDITOR_BACKGROUND))
             }),
             stack((
                 text("Save")
@@ -430,18 +439,15 @@ fn keyboard_picker_view(
                             .padding_vert(8.0)
                             .border(1.0)
                             .border_radius(6.0)
-                            .border_color(
-                                *config.get_color(LapceColor::LAPCE_BORDER),
-                            )
+                            .border_color(config.color(LapceColor::LAPCE_BORDER))
                             .hover(|s| {
                                 s.cursor(CursorStyle::Pointer).background(
-                                    *config.get_color(
-                                        LapceColor::PANEL_HOVERED_BACKGROUND,
-                                    ),
+                                    config
+                                        .color(LapceColor::PANEL_HOVERED_BACKGROUND),
                                 )
                             })
                             .active(|s| {
-                                s.background(*config.get_color(
+                                s.background(config.color(
                                     LapceColor::PANEL_HOVERED_ACTIVE_BACKGROUND,
                                 ))
                             })
@@ -456,7 +462,7 @@ fn keyboard_picker_view(
                                 &keys
                                     .iter()
                                     .map(|(key, _)| key.clone())
-                                    .collect::<Vec<KeyPress>>(),
+                                    .collect::<Vec<KeyMapPress>>(),
                             );
                         }
                     }),
@@ -469,18 +475,15 @@ fn keyboard_picker_view(
                             .padding_vert(8.0)
                             .border(1.0)
                             .border_radius(6.0)
-                            .border_color(
-                                *config.get_color(LapceColor::LAPCE_BORDER),
-                            )
+                            .border_color(config.color(LapceColor::LAPCE_BORDER))
                             .hover(|s| {
                                 s.cursor(CursorStyle::Pointer).background(
-                                    *config.get_color(
-                                        LapceColor::PANEL_HOVERED_BACKGROUND,
-                                    ),
+                                    config
+                                        .color(LapceColor::PANEL_HOVERED_BACKGROUND),
                                 )
                             })
                             .active(|s| {
-                                s.background(*config.get_color(
+                                s.background(config.color(
                                     LapceColor::PANEL_HOVERED_ACTIVE_BACKGROUND,
                                 ))
                             })
@@ -495,7 +498,7 @@ fn keyboard_picker_view(
                     .justify_center()
                     .width_pct(100.0)
                     .margin_top(20.0)
-                    .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
+                    .border_color(config.color(LapceColor::LAPCE_BORDER))
             }),
         ))
         .style(move |s| {
@@ -506,14 +509,15 @@ fn keyboard_picker_view(
                 .width(400.0)
                 .border(1.0)
                 .border_radius(6.0)
-                .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
-                .background(*config.get_color(LapceColor::PANEL_BACKGROUND))
+                .border_color(config.color(LapceColor::LAPCE_BORDER))
+                .background(config.color(LapceColor::PANEL_BACKGROUND))
         }),
     )
     .keyboard_navigatable()
     .on_event_stop(EventListener::KeyDown, move |event| {
         if let Event::KeyDown(key_event) = event {
             if let Some(keypress) = KeyPressData::keypress(key_event) {
+                let keypress = keypress.keymap_press();
                 picker.keys.update(|keys| {
                     if let Some((last_key, last_key_confirmed)) = keys.last() {
                         if !*last_key_confirmed && last_key.is_modifiers() {
