@@ -14,13 +14,14 @@ use anyhow::Result;
 use crossbeam_channel::{Receiver, Sender, TryRecvError};
 use floem::{
     ext_event::{create_ext_action, create_signal_from_channel},
-    keyboard::ModifiersState,
+    keyboard::Modifiers,
     reactive::{use_context, ReadSignal, RwSignal, Scope},
 };
 use itertools::Itertools;
 use lapce_core::{
     buffer::rope_text::RopeText, command::FocusCommand, language::LapceLanguage,
-    mode::Mode, movement::Movement, selection::Selection, syntax::Syntax,
+    line_ending::LineEnding, mode::Mode, movement::Movement, selection::Selection,
+    syntax::Syntax,
 };
 use lapce_rpc::proxy::ProxyResponse;
 use lapce_xi_rope::Rope;
@@ -43,7 +44,6 @@ use crate::{
         location::{EditorLocation, EditorPosition},
         EditorData,
     },
-    id::EditorId,
     keypress::{condition::Condition, KeyPressData, KeyPressFocus},
     main_split::MainSplitData,
     proxy::path_from_url,
@@ -91,7 +91,7 @@ pub struct PaletteData {
     pub input: RwSignal<PaletteInput>,
     kind: RwSignal<PaletteKind>,
     pub input_editor: EditorData,
-    pub preview_editor: Rc<EditorData>,
+    pub preview_editor: EditorData,
     pub has_preview: RwSignal<bool>,
     pub keypress: ReadSignal<KeyPressData>,
     /// Listened on for which entry in the palette has been clicked
@@ -102,6 +102,7 @@ pub struct PaletteData {
     pub references: RwSignal<Vec<EditorLocation>>,
     pub source_control: SourceControlData,
     pub common: Rc<CommonData>,
+    left_diff_path: RwSignal<Option<PathBuf>>,
 }
 
 impl PaletteData {
@@ -123,11 +124,8 @@ impl PaletteData {
             kind: PaletteKind::File,
         });
         let kind = cx.create_rw_signal(PaletteKind::File);
-        let input_editor =
-            EditorData::new_local(cx, EditorId::next(), common.clone());
-        let preview_editor =
-            EditorData::new_local(cx, EditorId::next(), common.clone());
-        let preview_editor = Rc::new(preview_editor);
+        let input_editor = main_split.editors.make_local(cx, common.clone());
+        let preview_editor = main_split.editors.make_local(cx, common.clone());
         let has_preview = cx.create_rw_signal(false);
         let run_id = cx.create_rw_signal(0);
         let run_id_counter = Arc::new(AtomicU64::new(0));
@@ -196,6 +194,7 @@ impl PaletteData {
         }
 
         let clicked_index = cx.create_rw_signal(Option::<usize>::None);
+        let left_diff_path = cx.create_rw_signal(None);
 
         let palette = Self {
             run_id_counter,
@@ -219,6 +218,7 @@ impl PaletteData {
             references,
             source_control,
             common,
+            left_diff_path,
         };
 
         {
@@ -235,7 +235,7 @@ impl PaletteData {
 
         {
             let palette = palette.clone();
-            let doc = palette.input_editor.view.doc.get_untracked();
+            let doc = palette.input_editor.doc();
             let input = palette.input;
             let status = palette.status.read_only();
             let preset_kind = palette.kind.read_only();
@@ -316,16 +316,25 @@ impl PaletteData {
         let symbol = kind.symbol();
         self.kind.set(kind);
         // Refresh the palette input with only the symbol prefix, losing old content.
+        self.input_editor.doc().reload(Rope::from(symbol), true);
         self.input_editor
-            .view
-            .doc
-            .get_untracked()
-            .reload(Rope::from(symbol), true);
-        self.input_editor
-            .cursor
+            .cursor()
             .update(|cursor| cursor.set_insert(Selection::caret(symbol.len())));
     }
 
+    /// Get the placeholder text to use in the palette input field.
+    pub fn placeholder_text(&self) -> &'static str {
+        if self.kind.get() == PaletteKind::DiffFiles {
+            if self.left_diff_path.with(Option::is_some) {
+                "Select right file"
+            } else {
+                "Seleft left file"
+            }
+        } else {
+            ""
+        }
+    }
+
     /// Execute the internal behavior of the palette for the given kind. This ignores updating and
     /// focusing the palette input.
     fn run_inner(&self, kind: PaletteKind) {
@@ -336,7 +345,7 @@ impl PaletteData {
 
         match kind {
             PaletteKind::PaletteHelp => self.get_palette_help(),
-            PaletteKind::File => {
+            PaletteKind::File | PaletteKind::DiffFiles => {
                 self.get_files();
             }
             PaletteKind::Line => {
@@ -360,6 +369,10 @@ impl PaletteData {
             PaletteKind::SshHost => {
                 self.get_ssh_hosts();
             }
+            #[cfg(windows)]
+            PaletteKind::WslHost => {
+                self.get_wsl_hosts();
+            }
             PaletteKind::RunAndDebug => {
                 self.get_run_configs();
             }
@@ -372,6 +385,9 @@ impl PaletteData {
             PaletteKind::Language => {
                 self.get_languages();
             }
+            PaletteKind::LineEnding => {
+                self.get_line_endings();
+            }
             PaletteKind::SCMReferences => {
                 self.get_scm_references();
             }
@@ -421,18 +437,18 @@ impl PaletteData {
             create_ext_action(self.common.scope, move |items: Vec<PathBuf>| {
                 let items = items
                     .into_iter()
-                    .map(|path| {
-                        let full_path = path.clone();
+                    .map(|full_path| {
                         // Strip the workspace prefix off the path, to avoid clutter
                         let path =
                             if let Some(workspace_path) = workspace.path.as_ref() {
-                                path.strip_prefix(workspace_path)
+                                full_path
+                                    .strip_prefix(workspace_path)
                                     .unwrap_or(&full_path)
                                     .to_path_buf()
                             } else {
-                                path
+                                full_path.clone()
                             };
-                        let filter_text = path.to_str().unwrap_or("").to_string();
+                        let filter_text = path.to_string_lossy().into_owned();
                         PaletteItem {
                             content: PaletteItemContent::File { path, full_path },
                             filter_text,
@@ -454,7 +470,7 @@ impl PaletteData {
     fn get_lines(&self) {
         let editor = self.main_split.active_editor.get_untracked();
         let doc = match editor {
-            Some(editor) => editor.view.doc.get_untracked(),
+            Some(editor) => editor.doc(),
             None => {
                 return;
             }
@@ -549,12 +565,12 @@ impl PaletteData {
                 let text = w.path.as_ref()?.to_str()?.to_string();
                 let filter_text = match &w.kind {
                     LapceWorkspaceType::Local => text,
-                    LapceWorkspaceType::RemoteSSH(ssh) => {
-                        format!("[{ssh}] {text}")
+                    LapceWorkspaceType::RemoteSSH(remote) => {
+                        format!("[{remote}] {text}")
                     }
                     #[cfg(windows)]
-                    LapceWorkspaceType::RemoteWSL => {
-                        format!("[wsl] {text}")
+                    LapceWorkspaceType::RemoteWSL(remote) => {
+                        format!("[{remote}] {text}")
                     }
                 };
                 Some(PaletteItem {
@@ -600,14 +616,13 @@ impl PaletteData {
     fn get_document_symbols(&self) {
         let editor = self.main_split.active_editor.get_untracked();
         let doc = match editor {
-            Some(editor) => editor.view.doc,
+            Some(editor) => editor.doc(),
             None => {
                 self.items.update(|items| items.clear());
                 return;
             }
         };
         let path = doc
-            .get_untracked()
             .content
             .with_untracked(|content| content.path().cloned());
         let path = match path {
@@ -721,16 +736,76 @@ impl PaletteData {
         let workspaces = db.recent_workspaces().unwrap_or_default();
         let mut hosts = HashSet::new();
         for workspace in workspaces.iter() {
-            if let LapceWorkspaceType::RemoteSSH(ssh) = &workspace.kind {
-                hosts.insert(ssh.clone());
+            if let LapceWorkspaceType::RemoteSSH(host) = &workspace.kind {
+                hosts.insert(host.clone());
             }
         }
 
         let items = hosts
             .iter()
-            .map(|ssh| PaletteItem {
-                content: PaletteItemContent::SshHost { host: ssh.clone() },
-                filter_text: ssh.to_string(),
+            .map(|host| PaletteItem {
+                content: PaletteItemContent::SshHost { host: host.clone() },
+                filter_text: host.to_string(),
+                score: 0,
+                indices: vec![],
+            })
+            .collect();
+        self.items.set(items);
+    }
+
+    #[cfg(windows)]
+    fn get_wsl_hosts(&self) {
+        use std::os::windows::process::CommandExt;
+        use std::process;
+        let cmd = process::Command::new("wsl")
+            .creation_flags(0x08000000) // CREATE_NO_WINDOW
+            .arg("-l")
+            .arg("-v")
+            .stdout(process::Stdio::piped())
+            .output();
+
+        let distros = if let Ok(proc) = cmd {
+            let distros = String::from_utf16(bytemuck::cast_slice(&proc.stdout))
+                .unwrap_or_default()
+                .lines()
+                .skip(1)
+                .filter_map(|line| {
+                    let line = line.trim_start();
+                    // let default = line.starts_with('*');
+                    let name = line
+                        .trim_start_matches('*')
+                        .trim_start()
+                        .split(' ')
+                        .next()?;
+                    Some(name.to_string())
+                })
+                .collect();
+
+            distros
+        } else {
+            vec![]
+        };
+
+        let db: Arc<LapceDb> = use_context().unwrap();
+        let workspaces = db.recent_workspaces().unwrap_or_default();
+        let mut hosts = HashSet::new();
+        for distro in distros {
+            hosts.insert(distro);
+        }
+
+        for workspace in workspaces.iter() {
+            if let LapceWorkspaceType::RemoteWSL(host) = &workspace.kind {
+                hosts.insert(host.host.clone());
+            }
+        }
+
+        let items = hosts
+            .iter()
+            .map(|host| PaletteItem {
+                content: PaletteItemContent::WslHost {
+                    host: crate::workspace::WslHost { host: host.clone() },
+                },
+                filter_text: host.to_string(),
                 score: 0,
                 indices: vec![],
             })
@@ -802,7 +877,7 @@ impl PaletteData {
     fn get_run_configs(&self) {
         if let Some(workspace) = self.common.workspace.path.as_deref() {
             let run_toml = workspace.join(".lapce").join("run.toml");
-            let (doc, new_doc) = self.main_split.get_doc(run_toml.clone());
+            let (doc, new_doc) = self.main_split.get_doc(run_toml.clone(), None);
             if !new_doc {
                 let content = doc.buffer.with_untracked(|b| b.to_string());
                 self.set_run_configs(content);
@@ -880,14 +955,32 @@ impl PaletteData {
             })
             .collect();
         if let Some(editor) = self.main_split.active_editor.get_untracked() {
-            let doc = editor.view.doc.get_untracked();
+            let doc = editor.doc();
             let language =
-                doc.syntax.with_untracked(|syntax| syntax.language.name());
+                doc.syntax().with_untracked(|syntax| syntax.language.name());
             self.preselect_matching(&items, language);
         }
         self.items.set(items);
     }
 
+    fn get_line_endings(&self) {
+        let items = [LineEnding::Lf, LineEnding::CrLf]
+            .iter()
+            .map(|l| PaletteItem {
+                content: PaletteItemContent::LineEnding { kind: *l },
+                filter_text: l.as_str().to_string(),
+                score: 0,
+                indices: Vec::new(),
+            })
+            .collect();
+        if let Some(editor) = self.main_split.active_editor.get_untracked() {
+            let doc = editor.doc();
+            let line_ending = doc.line_ending();
+            self.preselect_matching(&items, line_ending.as_str());
+        }
+        self.items.set(items);
+    }
+
     fn get_scm_references(&self) {
         let branches = self.source_control.branches.get_untracked();
         let tags = self.source_control.tags.get_untracked();
@@ -977,22 +1070,37 @@ impl PaletteData {
                     self.common.lapce_command.send(cmd);
                 }
                 PaletteItemContent::File { full_path, .. } => {
-                    self.common
-                        .internal_command
-                        .send(InternalCommand::OpenFile {
-                            path: full_path.to_owned(),
-                        });
+                    if self.kind.get_untracked() == PaletteKind::DiffFiles {
+                        if let Some(left_path) =
+                            self.left_diff_path.try_update(Option::take).flatten()
+                        {
+                            self.common.internal_command.send(
+                                InternalCommand::OpenDiffFiles {
+                                    left_path,
+                                    right_path: full_path.clone(),
+                                },
+                            );
+                        } else {
+                            self.left_diff_path.set(Some(full_path.clone()));
+                            self.run(PaletteKind::DiffFiles);
+                        }
+                    } else {
+                        self.common.internal_command.send(
+                            InternalCommand::OpenFile {
+                                path: full_path.clone(),
+                            },
+                        );
+                    }
                 }
                 PaletteItemContent::Line { line, .. } => {
                     let editor = self.main_split.active_editor.get_untracked();
                     let doc = match editor {
-                        Some(editor) => editor.view.doc,
+                        Some(editor) => editor.doc(),
                         None => {
                             return;
                         }
                     };
                     let path = doc
-                        .get_untracked()
                         .content
                         .with_untracked(|content| content.path().cloned());
                     let path = match path {
@@ -1039,16 +1147,27 @@ impl PaletteData {
                         },
                     );
                 }
+                #[cfg(windows)]
+                PaletteItemContent::WslHost { host } => {
+                    self.common.window_common.window_command.send(
+                        WindowCommand::SetWorkspace {
+                            workspace: LapceWorkspace {
+                                kind: LapceWorkspaceType::RemoteWSL(host.clone()),
+                                path: None,
+                                last_open: 0,
+                            },
+                        },
+                    );
+                }
                 PaletteItemContent::DocumentSymbol { range, .. } => {
                     let editor = self.main_split.active_editor.get_untracked();
                     let doc = match editor {
-                        Some(editor) => editor.view.doc,
+                        Some(editor) => editor.doc(),
                         None => {
                             return;
                         }
                     };
                     let path = doc
-                        .get_untracked()
                         .content
                         .with_untracked(|content| content.path().cloned());
                     let path = match path {
@@ -1101,7 +1220,7 @@ impl PaletteData {
                 PaletteItemContent::Language { name } => {
                     let editor = self.main_split.active_editor.get_untracked();
                     let doc = match editor {
-                        Some(editor) => editor.view.doc.get_untracked(),
+                        Some(editor) => editor.doc(),
                         None => {
                             return;
                         }
@@ -1117,6 +1236,17 @@ impl PaletteData {
                     }
                     doc.trigger_syntax_change(None);
                 }
+                PaletteItemContent::LineEnding { kind } => {
+                    let Some(editor) = self.main_split.active_editor.get_untracked()
+                    else {
+                        return;
+                    };
+                    let doc = editor.doc();
+
+                    doc.buffer.update(|buffer| {
+                        buffer.set_line_ending(*kind);
+                    });
+                }
                 PaletteItemContent::SCMReference { name } => {
                     self.common
                         .lapce_command
@@ -1165,7 +1295,7 @@ impl PaletteData {
                     self.has_preview.set(true);
                     let editor = self.main_split.active_editor.get_untracked();
                     let doc = match editor {
-                        Some(editor) => editor.view.doc.get_untracked(),
+                        Some(editor) => editor.doc(),
                         None => {
                             return;
                         }
@@ -1194,11 +1324,14 @@ impl PaletteData {
                 PaletteItemContent::Workspace { .. } => {}
                 PaletteItemContent::RunAndDebug { .. } => {}
                 PaletteItemContent::SshHost { .. } => {}
+                #[cfg(windows)]
+                PaletteItemContent::WslHost { .. } => {}
                 PaletteItemContent::Language { .. } => {}
+                PaletteItemContent::LineEnding { .. } => {}
                 PaletteItemContent::Reference { location, .. } => {
                     self.has_preview.set(true);
                     let (doc, new_doc) =
-                        self.main_split.get_doc(location.path.clone());
+                        self.main_split.get_doc(location.path.clone(), None);
                     self.preview_editor.update_doc(doc);
                     self.preview_editor.go_to_location(
                         location.clone(),
@@ -1210,7 +1343,7 @@ impl PaletteData {
                     self.has_preview.set(true);
                     let editor = self.main_split.active_editor.get_untracked();
                     let doc = match editor {
-                        Some(editor) => editor.view.doc.get_untracked(),
+                        Some(editor) => editor.doc(),
                         None => {
                             return;
                         }
@@ -1238,7 +1371,7 @@ impl PaletteData {
                 PaletteItemContent::WorkspaceSymbol { location, .. } => {
                     self.has_preview.set(true);
                     let (doc, new_doc) =
-                        self.main_split.get_doc(location.path.clone());
+                        self.main_split.get_doc(location.path.clone(), None);
                     self.preview_editor.update_doc(doc);
                     self.preview_editor.go_to_location(
                         location.clone(),
@@ -1277,6 +1410,7 @@ impl PaletteData {
                 .send(InternalCommand::ReloadConfig);
         }
 
+        self.left_diff_path.set(None);
         self.close();
     }
 
@@ -1288,13 +1422,9 @@ impl PaletteData {
         }
         self.has_preview.set(false);
         self.items.update(|items| items.clear());
+        self.input_editor.doc().reload(Rope::from(""), true);
         self.input_editor
-            .view
-            .doc
-            .get_untracked()
-            .reload(Rope::from(""), true);
-        self.input_editor
-            .cursor
+            .cursor()
             .update(|cursor| cursor.set_insert(Selection::caret(0)));
     }
 
@@ -1361,6 +1491,7 @@ impl PaletteData {
         let pattern = nucleo::pattern::Pattern::parse(
             input,
             nucleo::pattern::CaseMatching::Ignore,
+            nucleo::pattern::Normalization::Smart,
         );
 
         // NOTE: We collect into a Vec to sort as we are hitting a worst-case behavior in
@@ -1475,10 +1606,11 @@ impl KeyPressFocus for PaletteData {
         &self,
         command: &crate::command::LapceCommand,
         count: Option<usize>,
-        mods: ModifiersState,
+        mods: Modifiers,
     ) -> CommandExecuted {
         match &command.kind {
             CommandKind::Workbench(_) => {}
+            CommandKind::Scroll(_) => {}
             CommandKind::Focus(cmd) => {
                 self.run_focus_command(cmd);
             }
