@@ -8,8 +8,8 @@ use floem::{
     style::CursorStyle,
     view::View,
     views::{
-        container, container_box, label, list, scroll, stack, svg, text,
-        virtual_list, Decorators, VirtualListDirection, VirtualListItemSize,
+        container, dyn_stack, label, scroll, stack, svg, text, virtual_stack,
+        Decorators, VirtualDirection, VirtualItemSize,
     },
 };
 use lapce_rpc::{
@@ -17,7 +17,7 @@ use lapce_rpc::{
     terminal::TermId,
 };
 
-use super::{position::PanelPosition, view::panel_header};
+use super::{data::PanelSection, position::PanelPosition, view::PanelBuilder};
 use crate::{
     app::clickable_icon,
     command::InternalCommand,
@@ -38,35 +38,30 @@ pub fn debug_panel(
     let terminal = window_tab_data.terminal.clone();
     let internal_command = window_tab_data.common.internal_command;
 
-    stack((
-        {
-            let terminal = terminal.clone();
-            stack((
-                panel_header("Processes".to_string(), config),
-                debug_processes(terminal, config),
-            ))
-            .style(|s| s.width_pct(100.0).flex_col().height(150.0))
-        },
-        stack((
-            panel_header("Variables".to_string(), config),
+    PanelBuilder::new(config, position)
+        .add_height(
+            "Processes",
+            150.0,
+            debug_processes(terminal.clone(), config),
+            window_tab_data.panel.section_open(PanelSection::Process),
+        )
+        .add(
+            "Variables",
             variables_view(window_tab_data.clone()),
-        ))
-        .style(|s| s.width_pct(100.0).flex_grow(1.0).flex_basis(0.0).flex_col()),
-        stack((
-            panel_header("Stack Frames".to_string(), config),
-            debug_stack_traces(terminal, internal_command, config),
-        ))
-        .style(|s| s.width_pct(100.0).flex_grow(1.0).flex_basis(0.0).flex_col()),
-        stack((
-            panel_header("Breakpoints".to_string(), config),
+            window_tab_data.panel.section_open(PanelSection::Variable),
+        )
+        .add(
+            "Stack Frames",
+            debug_stack_traces(terminal.clone(), internal_command, config),
+            window_tab_data.panel.section_open(PanelSection::StackFrame),
+        )
+        .add_height(
+            "Breakpoints",
+            150.0,
             breakpoints_view(window_tab_data.clone()),
-        ))
-        .style(|s| s.width_pct(100.0).flex_col().height(150.0)),
-    ))
-    .style(move |s| {
-        s.width_pct(100.0)
-            .apply_if(!position.is_bottom(), |s| s.flex_col())
-    })
+            window_tab_data.panel.section_open(PanelSection::Breakpoint),
+        )
+        .build()
 }
 
 fn debug_process_icons(
@@ -85,7 +80,7 @@ fn debug_process_icons(
         stopped.map(|stopped| stopped.get()).unwrap_or(false)
     };
     match mode {
-        RunDebugMode::Run => container_box(stack((
+        RunDebugMode::Run => container(stack((
             {
                 let terminal = terminal.clone();
                 clickable_icon(
@@ -95,6 +90,7 @@ fn debug_process_icons(
                     },
                     || false,
                     || false,
+                    || "Restart",
                     config,
                 )
                 .style(|s| s.margin_horiz(4.0))
@@ -108,6 +104,7 @@ fn debug_process_icons(
                     },
                     || false,
                     move || stopped,
+                    || "Stop",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -121,12 +118,13 @@ fn debug_process_icons(
                     },
                     || false,
                     || false,
+                    || "Close",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
             },
         ))),
-        RunDebugMode::Debug => container_box(stack((
+        RunDebugMode::Debug => container(stack((
             {
                 let terminal = terminal.clone();
                 clickable_icon(
@@ -136,6 +134,7 @@ fn debug_process_icons(
                     },
                     || false,
                     move || !paused() || stopped,
+                    || "Continue",
                     config,
                 )
                 .style(|s| s.margin_horiz(6.0))
@@ -149,6 +148,7 @@ fn debug_process_icons(
                     },
                     || false,
                     move || paused() || stopped,
+                    || "Pause",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -162,6 +162,7 @@ fn debug_process_icons(
                     },
                     || false,
                     move || !paused() || stopped,
+                    || "Step Over",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -175,6 +176,7 @@ fn debug_process_icons(
                     },
                     || false,
                     move || !paused() || stopped,
+                    || "Step Into",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -188,6 +190,7 @@ fn debug_process_icons(
                     },
                     || false,
                     move || !paused() || stopped,
+                    || "Step Out",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -201,6 +204,7 @@ fn debug_process_icons(
                     },
                     || false,
                     || false,
+                    || "Restart",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -214,6 +218,7 @@ fn debug_process_icons(
                     },
                     || false,
                     move || stopped,
+                    || "Stop",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -227,6 +232,7 @@ fn debug_process_icons(
                     },
                     || false,
                     || false,
+                    || "Close",
                     config,
                 )
                 .style(|s| s.margin_right(4.0))
@@ -242,7 +248,7 @@ fn debug_processes(
     scroll({
         let terminal = terminal.clone();
         let local_terminal = terminal.clone();
-        list(
+        dyn_stack(
             move || local_terminal.run_debug_process(true),
             |(term_id, p)| (*term_id, p.stopped),
             move |(term_id, p)| {
@@ -267,9 +273,7 @@ fn debug_processes(
                             s.size(size, size)
                                 .margin_vert(5.0)
                                 .margin_horiz(10.0)
-                                .color(
-                                    *config.get_color(LapceColor::LAPCE_ICON_ACTIVE),
-                                )
+                                .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
                         })
                     },
                     label(move || p.config.name.clone()).style(|s| {
@@ -307,16 +311,13 @@ fn debug_processes(
                         .items_center()
                         .apply_if(is_active(), |s| {
                             s.background(
-                                *config
-                                    .get_color(LapceColor::PANEL_CURRENT_BACKGROUND),
+                                config.color(LapceColor::PANEL_CURRENT_BACKGROUND),
                             )
                         })
                         .hover(|s| {
                             s.cursor(CursorStyle::Pointer).background(
-                                (*config.get_color(
-                                    LapceColor::PANEL_HOVERED_BACKGROUND,
-                                ))
-                                .with_alpha_factor(0.3),
+                                (config.color(LapceColor::PANEL_HOVERED_BACKGROUND))
+                                    .with_alpha_factor(0.3),
                             )
                         })
                 })
@@ -333,9 +334,9 @@ fn variables_view(window_tab_data: Rc<WindowTabData>) -> impl View {
     let config = window_tab_data.common.config;
     container(
         scroll(
-            virtual_list(
-                VirtualListDirection::Vertical,
-                VirtualListItemSize::Fixed(Box::new(move || ui_line_height.get())),
+            virtual_stack(
+                VirtualDirection::Vertical,
+                VirtualItemSize::Fixed(Box::new(move || ui_line_height.get())),
                 move || {
                     let dap = terminal.get_active_dap(true);
                     dap.map(|dap| {
@@ -385,7 +386,7 @@ fn variables_view(window_tab_data: Rc<WindowTabData>) -> impl View {
                             let size = config.ui.icon_size() as f32;
 
                             let color = if reference > 0 {
-                                *config.get_color(LapceColor::LAPCE_ICON_ACTIVE)
+                                config.color(LapceColor::LAPCE_ICON_ACTIVE)
                             } else {
                                 Color::TRANSPARENT
                             };
@@ -396,7 +397,7 @@ fn variables_view(window_tab_data: Rc<WindowTabData>) -> impl View {
                             s.apply_if(!type_exists || reference == 0, |s| s.hide())
                         }),
                         text(node.item.ty().unwrap_or("")).style(move |s| {
-                            s.color(*config.get().get_style_color("type").unwrap())
+                            s.color(config.get().style_color("type").unwrap())
                                 .apply_if(!type_exists || reference == 0, |s| {
                                     s.hide()
                                 })
@@ -431,9 +432,11 @@ fn variables_view(window_tab_data: Rc<WindowTabData>) -> impl View {
                             .min_width_pct(100.0)
                             .hover(|s| {
                                 s.apply_if(reference > 0, |s| {
-                                    s.background(*config.get().get_color(
-                                        LapceColor::PANEL_HOVERED_BACKGROUND,
-                                    ))
+                                    s.background(
+                                        config.get().color(
+                                            LapceColor::PANEL_HOVERED_BACKGROUND,
+                                        ),
+                                    )
                                 })
                             })
                     })
@@ -465,13 +468,11 @@ fn debug_stack_frames(
             .style(move |s| {
                 s.padding_horiz(10.0).min_width_pct(100.0).hover(move |s| {
                     s.cursor(CursorStyle::Pointer).background(
-                        *config
-                            .get()
-                            .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                        config.get().color(LapceColor::PANEL_HOVERED_BACKGROUND),
                     )
                 })
             }),
-        list(
+        dyn_stack(
             move || {
                 let expanded = stack_trace.expanded.get() && stopped.get();
                 if expanded {
@@ -501,15 +502,15 @@ fn debug_stack_frames(
                     label(move || frame.name.clone()).style(move |s| {
                         s.hover(|s| {
                             s.background(
-                                *config
+                                config
                                     .get()
-                                    .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                                    .color(LapceColor::PANEL_HOVERED_BACKGROUND),
                             )
                         })
                     }),
                     label(move || source_path.clone()).style(move |s| {
                         s.margin_left(10.0)
-                            .color(*config.get().get_color(LapceColor::EDITOR_DIM))
+                            .color(config.get().color(LapceColor::EDITOR_DIM))
                             .font_style(FontStyle::Italic)
                             .apply_if(!has_source, |s| s.hide())
                     }),
@@ -542,12 +543,11 @@ fn debug_stack_frames(
                         .padding_right(10.0)
                         .min_width_pct(100.0)
                         .apply_if(!has_source, |s| {
-                            s.color(*config.get_color(LapceColor::EDITOR_DIM))
+                            s.color(config.color(LapceColor::EDITOR_DIM))
                         })
                         .hover(|s| {
                             s.background(
-                                *config
-                                    .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                                config.color(LapceColor::PANEL_HOVERED_BACKGROUND),
                             )
                             .apply_if(has_source, |s| s.cursor(CursorStyle::Pointer))
                         })
@@ -567,7 +567,7 @@ fn debug_stack_traces(
     container(
         scroll({
             let local_terminal = terminal.clone();
-            list(
+            dyn_stack(
                 move || {
                     let dap = local_terminal.get_active_dap(true);
                     if let Some(dap) = dap {
@@ -628,7 +628,7 @@ fn breakpoints_view(window_tab_data: Rc<WindowTabData>) -> impl View {
     let internal_command = window_tab_data.common.internal_command;
     container(
         scroll(
-            list(
+            dyn_stack(
                 move || {
                     breakpoints
                         .get()
@@ -674,6 +674,7 @@ fn breakpoints_view(window_tab_data: Rc<WindowTabData>) -> impl View {
                             },
                             || false,
                             || false,
+                            || "Remove",
                             config,
                         )
                         .on_event_stop(EventListener::PointerDown, |_| {}),
@@ -711,9 +712,7 @@ fn breakpoints_view(window_tab_data: Rc<WindowTabData>) -> impl View {
                             s.text_ellipsis()
                                 .flex_grow(1.0)
                                 .flex_basis(0.0)
-                                .color(
-                                    *config.get().get_color(LapceColor::EDITOR_DIM),
-                                )
+                                .color(config.get().color(LapceColor::EDITOR_DIM))
                                 .min_width(0.0)
                                 .margin_left(6.0)
                                 .apply_if(folder_empty, |s| s.hide())
@@ -723,9 +722,9 @@ fn breakpoints_view(window_tab_data: Rc<WindowTabData>) -> impl View {
                         s.items_center().padding_horiz(10.0).width_pct(100.0).hover(
                             |s| {
                                 s.background(
-                                    *config.get().get_color(
-                                        LapceColor::PANEL_HOVERED_BACKGROUND,
-                                    ),
+                                    config
+                                        .get()
+                                        .color(LapceColor::PANEL_HOVERED_BACKGROUND),
                                 )
                             },
                         )
