@@ -6,8 +6,8 @@ use floem::{
     style::{CursorStyle, Style},
     view::View,
     views::{
-        container, label, scroll, stack, svg, virtual_list, Decorators,
-        VirtualListDirection, VirtualListItemSize,
+        container, label, scroll, stack, svg, virtual_stack, Decorators,
+        VirtualDirection, VirtualItemSize,
     },
 };
 use lapce_xi_rope::find::CaseMatching;
@@ -21,7 +21,7 @@ use crate::{
     focus_text::focus_text,
     global_search::{GlobalSearchData, SearchMatchData},
     listener::Listener,
-    text_input::text_input,
+    text_input::TextInputBuilder,
     window_tab::{Focus, WindowTabData},
     workspace::LapceWorkspace,
 };
@@ -45,7 +45,10 @@ pub fn global_search_panel(
     stack((
         container(
             stack((
-                text_input(editor, is_focused).style(|s| s.width_pct(100.0)),
+                TextInputBuilder::new()
+                    .is_focused(is_focused)
+                    .build_editor(editor.clone())
+                    .style(|s| s.width_pct(100.0)),
                 clickable_icon(
                     || LapceIcons::SEARCH_CASE_SENSITIVE,
                     move || {
@@ -57,6 +60,7 @@ pub fn global_search_panel(
                     },
                     move || case_matching.get() == CaseMatching::Exact,
                     || false,
+                    || "Case Sensitive",
                     config,
                 )
                 .style(|s| s.padding_vert(4.0)),
@@ -69,6 +73,7 @@ pub fn global_search_panel(
                     },
                     move || whole_word.get(),
                     || false,
+                    || "Whole Word",
                     config,
                 )
                 .style(|s| s.padding_left(6.0)),
@@ -81,6 +86,7 @@ pub fn global_search_panel(
                     },
                     move || is_regex.get(),
                     || false,
+                    || "Use Regex",
                     config,
                 )
                 .style(|s| s.padding_left(6.0)),
@@ -94,7 +100,7 @@ pub fn global_search_panel(
                     .items_center()
                     .border(1.0)
                     .border_radius(6.0)
-                    .border_color(*config.get().get_color(LapceColor::LAPCE_BORDER))
+                    .border_color(config.get().color(LapceColor::LAPCE_BORDER))
             }),
         )
         .style(|s| s.width_pct(100.0).padding(10.0)),
@@ -112,9 +118,9 @@ fn search_result(
     let ui_line_height = global_search_data.common.ui_line_height;
     container({
         scroll({
-            virtual_list(
-                VirtualListDirection::Vertical,
-                VirtualListItemSize::Fn(Box::new(
+            virtual_stack(
+                VirtualDirection::Vertical,
+                VirtualItemSize::Fn(Box::new(
                     |(_, match_data): &(PathBuf, SearchMatchData)| {
                         match_data.height()
                     },
@@ -164,17 +170,14 @@ fn search_result(
                                     .size(size, size)
                                     .min_size(size, size)
                                     .color(
-                                        *config.get_color(
-                                            LapceColor::LAPCE_ICON_ACTIVE,
-                                        ),
+                                        config.color(LapceColor::LAPCE_ICON_ACTIVE),
                                     )
                             }),
                             svg(move || config.get().file_svg(&path).0).style(
                                 move |s| {
                                     let config = config.get();
                                     let size = config.ui.icon_size() as f32;
-                                    let color =
-                                        config.file_svg(&style_path).1.copied();
+                                    let color = config.file_svg(&style_path).1;
                                     s.margin_right(6.0)
                                         .size(size, size)
                                         .min_size(size, size)
@@ -189,9 +192,7 @@ fn search_result(
                                 }),
                                 label(move || folder.clone()).style(move |s| {
                                     s.color(
-                                        *config
-                                            .get()
-                                            .get_color(LapceColor::EDITOR_DIM),
+                                        config.get().color(LapceColor::EDITOR_DIM),
                                     )
                                     .min_width(0.0)
                                     .text_ellipsis()
@@ -208,15 +209,15 @@ fn search_result(
                                 .items_center()
                                 .hover(|s| {
                                     s.cursor(CursorStyle::Pointer).background(
-                                        *config.get().get_color(
+                                        config.get().color(
                                             LapceColor::PANEL_HOVERED_BACKGROUND,
                                         ),
                                     )
                                 })
                         }),
-                        virtual_list(
-                            VirtualListDirection::Vertical,
-                            VirtualListItemSize::Fixed(Box::new(move || {
+                        virtual_stack(
+                            VirtualDirection::Vertical,
+                            VirtualItemSize::Fixed(Box::new(move || {
                                 ui_line_height.get()
                             })),
                             move || {
@@ -266,9 +267,7 @@ fn search_result(
                                             .collect()
                                     },
                                     move || {
-                                        *config
-                                            .get()
-                                            .get_color(LapceColor::EDITOR_FOCUS)
+                                        config.get().color(LapceColor::EDITOR_FOCUS)
                                     },
                                 )
                                 .style(move |s| {
@@ -277,7 +276,7 @@ fn search_result(
                                     s.margin_left(10.0 + icon_size + 6.0).hover(
                                         |s| {
                                             s.cursor(CursorStyle::Pointer)
-                                                .background(*config.get_color(
+                                                .background(config.color(
                                                 LapceColor::PANEL_HOVERED_BACKGROUND,
                                             ))
                                         },
