@@ -7,20 +7,24 @@ use floem::{
     style::CursorStyle,
     view::View,
     views::{
-        container, dyn_container, img, label, scroll, stack, svg, virtual_list,
-        Decorators, VirtualListDirection, VirtualListItemSize, VirtualListVector,
+        container, dyn_container, img, label,
+        scroll::{scroll, HideBar},
+        stack, svg, virtual_stack, Decorators, VirtualDirection, VirtualItemSize,
+        VirtualVector,
     },
 };
 use indexmap::IndexMap;
 use lapce_rpc::plugin::{VoltID, VoltInfo};
 
-use super::{kind::PanelKind, position::PanelPosition, view::panel_header};
+use super::{
+    data::PanelSection, kind::PanelKind, position::PanelPosition, view::PanelBuilder,
+};
 use crate::{
-    app::clickable_icon,
+    app::not_clickable_icon,
     command::InternalCommand,
     config::{color::LapceColor, icon::LapceIcons},
     plugin::{AvailableVoltData, InstalledVoltData, PluginData, VoltIcon},
-    text_input::text_input,
+    text_input::TextInputBuilder,
     window_tab::{Focus, WindowTabData},
 };
 
@@ -40,16 +44,14 @@ impl<K: Clone, V: Clone> IndexMapItems<K, V> {
     }
 }
 
-impl<K: Clone + 'static, V: Clone + 'static> VirtualListVector<(usize, K, V)>
+impl<K: Clone + 'static, V: Clone + 'static> VirtualVector<(usize, K, V)>
     for IndexMapItems<K, V>
 {
-    type ItemIterator = Box<dyn Iterator<Item = (usize, K, V)>>;
-
     fn total_len(&self) -> usize {
         self.0.len()
     }
 
-    fn slice(&mut self, range: Range<usize>) -> Self::ItemIterator {
+    fn slice(&mut self, range: Range<usize>) -> impl Iterator<Item = (usize, K, V)> {
         let start = range.start;
         Box::new(
             self.items(range)
@@ -67,22 +69,18 @@ pub fn plugin_panel(
     let config = window_tab_data.common.config;
     let plugin = window_tab_data.plugin.clone();
 
-    stack((
-        stack((
-            panel_header("Installed".to_string(), config),
+    PanelBuilder::new(config, position)
+        .add(
+            "Installed",
             installed_view(plugin.clone()),
-        ))
-        .style(|s| s.flex_col().width_pct(100.0).flex_grow(1.0).flex_basis(0.0)),
-        stack((
-            panel_header("Available".to_string(), config),
+            window_tab_data.panel.section_open(PanelSection::Installed),
+        )
+        .add(
+            "Available",
             available_view(plugin.clone()),
-        ))
-        .style(|s| s.flex_col().width_pct(100.0).flex_grow(1.0).flex_basis(0.0)),
-    ))
-    .style(move |s| {
-        s.width_pct(100.0)
-            .apply_if(!position.is_bottom(), |s| s.flex_col())
-    })
+            window_tab_data.panel.section_open(PanelSection::Available),
+        )
+        .build()
 }
 
 fn installed_view(plugin: PluginData) -> impl View {
@@ -102,15 +100,14 @@ fn installed_view(plugin: PluginData) -> impl View {
             dyn_container(
                 move || icon.get(),
                 move |icon| match icon {
-                    None => Box::new(
-                        img(move || VOLT_DEFAULT_PNG.to_vec())
-                            .style(|s| s.size_full()),
-                    ),
-                    Some(VoltIcon::Svg(svg_str)) => Box::new(
-                        svg(move || svg_str.clone()).style(|s| s.size_full()),
-                    ),
+                    None => img(move || VOLT_DEFAULT_PNG.to_vec())
+                        .style(|s| s.size_full())
+                        .any(),
+                    Some(VoltIcon::Svg(svg_str)) => {
+                        svg(move || svg_str.clone()).style(|s| s.size_full()).any()
+                    }
                     Some(VoltIcon::Img(buf)) => {
-                        Box::new(img(move || buf.clone()).style(|s| s.size_full()))
+                        img(move || buf.clone()).style(|s| s.size_full()).any()
                     }
                 },
             )
@@ -151,11 +148,11 @@ fn installed_view(plugin: PluginData) -> impl View {
                             .flex_basis(0.0)
                             .min_width(0.0)
                     }),
-                    clickable_icon(
+                    not_clickable_icon(
                         || LapceIcons::SETTINGS,
-                        || {},
                         || false,
                         || false,
+                        || "Options",
                         config,
                     )
                     .style(|s| s.padding_left(6.0))
@@ -178,9 +175,7 @@ fn installed_view(plugin: PluginData) -> impl View {
                 .padding_vert(5.0)
                 .hover(|s| {
                     s.background(
-                        *config
-                            .get()
-                            .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                        config.get().color(LapceColor::PANEL_HOVERED_BACKGROUND),
                     )
                 })
         })
@@ -188,9 +183,9 @@ fn installed_view(plugin: PluginData) -> impl View {
 
     container(
         scroll(
-            virtual_list(
-                VirtualListDirection::Vertical,
-                VirtualListItemSize::Fixed(Box::new(move || {
+            virtual_stack(
+                VirtualDirection::Vertical,
+                VirtualItemSize::Fixed(Box::new(move || {
                     ui_line_height.get() * 3.0 + 10.0
                 })),
                 move || IndexMapItems(volts.get()),
@@ -217,54 +212,51 @@ fn available_view(plugin: PluginData) -> impl View {
     let internal_command = plugin.common.internal_command;
 
     let local_plugin = plugin.clone();
-    let install_button = move |id: VoltID,
-                               info: RwSignal<VoltInfo>,
-                               installing: RwSignal<bool>| {
-        let plugin = local_plugin.clone();
-        let installed = create_memo(move |_| {
-            installed.with(|installed| installed.contains_key(&id))
-        });
-        label(move || {
-            if installed.get() {
-                "Installed".to_string()
-            } else if installing.get() {
-                "Installing".to_string()
-            } else {
-                "Install".to_string()
-            }
-        })
-        .disabled(move || installed.get() || installing.get())
-        .on_click_stop(move |_| {
-            plugin.install_volt(info.get_untracked());
-        })
-        .style(move |s| {
-            let config = config.get();
-            s.color(*config.get_color(LapceColor::LAPCE_BUTTON_PRIMARY_FOREGROUND))
-                .background(
-                    *config.get_color(LapceColor::LAPCE_BUTTON_PRIMARY_BACKGROUND),
-                )
-                .margin_left(6.0)
-                .padding_horiz(6.0)
-                .border_radius(6.0)
-                .hover(|s| {
-                    s.cursor(CursorStyle::Pointer).background(
-                        config
-                            .get_color(LapceColor::LAPCE_BUTTON_PRIMARY_BACKGROUND)
-                            .with_alpha_factor(0.8),
+    let install_button =
+        move |id: VoltID, info: RwSignal<VoltInfo>, installing: RwSignal<bool>| {
+            let plugin = local_plugin.clone();
+            let installed = create_memo(move |_| {
+                installed.with(|installed| installed.contains_key(&id))
+            });
+            label(move || {
+                if installed.get() {
+                    "Installed".to_string()
+                } else if installing.get() {
+                    "Installing".to_string()
+                } else {
+                    "Install".to_string()
+                }
+            })
+            .disabled(move || installed.get() || installing.get())
+            .on_click_stop(move |_| {
+                plugin.install_volt(info.get_untracked());
+            })
+            .style(move |s| {
+                let config = config.get();
+                s.color(config.color(LapceColor::LAPCE_BUTTON_PRIMARY_FOREGROUND))
+                    .background(
+                        config.color(LapceColor::LAPCE_BUTTON_PRIMARY_BACKGROUND),
                     )
-                })
-                .active(|s| {
-                    s.background(
-                        config
-                            .get_color(LapceColor::LAPCE_BUTTON_PRIMARY_BACKGROUND)
-                            .with_alpha_factor(0.6),
-                    )
-                })
-                .disabled(|s| {
-                    s.background(*config.get_color(LapceColor::EDITOR_DIM))
-                })
-        })
-    };
+                    .margin_left(6.0)
+                    .padding_horiz(6.0)
+                    .border_radius(6.0)
+                    .hover(|s| {
+                        s.cursor(CursorStyle::Pointer).background(
+                            config
+                                .color(LapceColor::LAPCE_BUTTON_PRIMARY_BACKGROUND)
+                                .with_alpha_factor(0.8),
+                        )
+                    })
+                    .active(|s| {
+                        s.background(
+                            config
+                                .color(LapceColor::LAPCE_BUTTON_PRIMARY_BACKGROUND)
+                                .with_alpha_factor(0.6),
+                        )
+                    })
+                    .disabled(|s| s.background(config.color(LapceColor::EDITOR_DIM)))
+            })
+        };
 
     let view_fn = move |(_, id, volt): (usize, VoltID, AvailableVoltData)| {
         let info = volt.info.get_untracked();
@@ -274,15 +266,14 @@ fn available_view(plugin: PluginData) -> impl View {
             dyn_container(
                 move || icon.get(),
                 move |icon| match icon {
-                    None => Box::new(
-                        img(move || VOLT_DEFAULT_PNG.to_vec())
-                            .style(|s| s.size_full()),
-                    ),
-                    Some(VoltIcon::Svg(svg_str)) => Box::new(
-                        svg(move || svg_str.clone()).style(|s| s.size_full()),
-                    ),
+                    None => img(move || VOLT_DEFAULT_PNG.to_vec())
+                        .style(|s| s.size_full())
+                        .any(),
+                    Some(VoltIcon::Svg(svg_str)) => {
+                        svg(move || svg_str.clone()).style(|s| s.size_full()).any()
+                    }
                     Some(VoltIcon::Img(buf)) => {
-                        Box::new(img(move || buf.clone()).style(|s| s.size_full()))
+                        img(move || buf.clone()).style(|s| s.size_full()).any()
                     }
                 },
             )
@@ -322,9 +313,7 @@ fn available_view(plugin: PluginData) -> impl View {
                 .padding_vert(5.0)
                 .hover(|s| {
                     s.background(
-                        *config
-                            .get()
-                            .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                        config.get().color(LapceColor::PANEL_HOVERED_BACKGROUND),
                     )
                 })
         })
@@ -340,7 +329,9 @@ fn available_view(plugin: PluginData) -> impl View {
     stack((
         container({
             scroll(
-                text_input(editor, is_focused)
+                TextInputBuilder::new()
+                    .is_focused(is_focused)
+                    .build_editor(editor.clone())
                     .on_cursor_pos(move |point| {
                         cursor_x.set(point.x);
                     })
@@ -348,8 +339,7 @@ fn available_view(plugin: PluginData) -> impl View {
                         s.padding_vert(4.0).padding_horiz(10.0).min_width_pct(100.0)
                     }),
             )
-            .hide_bar(|| true)
-            .on_ensure_visible(move || {
+            .ensure_visible(move || {
                 Size::new(20.0, 0.0)
                     .to_rect()
                     .with_origin(Point::new(cursor_x.get(), 0.0))
@@ -359,21 +349,22 @@ fn available_view(plugin: PluginData) -> impl View {
             })
             .style(move |s| {
                 let config = config.get();
-                s.width_pct(100.0)
+                s.set(HideBar, true)
+                    .width_pct(100.0)
                     .cursor(CursorStyle::Text)
                     .items_center()
-                    .background(*config.get_color(LapceColor::EDITOR_BACKGROUND))
+                    .background(config.color(LapceColor::EDITOR_BACKGROUND))
                     .border(1.0)
                     .border_radius(6.0)
-                    .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
+                    .border_color(config.color(LapceColor::LAPCE_BORDER))
             })
         })
         .style(|s| s.padding(10.0).width_pct(100.0)),
         container({
             scroll({
-                virtual_list(
-                    VirtualListDirection::Vertical,
-                    VirtualListItemSize::Fixed(Box::new(move || {
+                virtual_stack(
+                    VirtualDirection::Vertical,
+                    VirtualItemSize::Fixed(Box::new(move || {
                         ui_line_height.get() * 3.0 + 10.0
                     })),
                     move || IndexMapItems(volts.get()),
