@@ -2,14 +2,14 @@ use std::{path::PathBuf, rc::Rc, sync::Arc};
 
 use floem::{
     peniko::Color,
-    reactive::{create_memo, create_rw_signal, ReadSignal},
+    reactive::{create_effect, create_rw_signal, ReadSignal},
     style::{CursorStyle, Style},
     view::View,
-    views::{container, label, list, scroll, stack, svg, Decorators},
+    views::{container, dyn_stack, label, scroll, stack, svg, Decorators},
 };
 use lsp_types::{DiagnosticRelatedInformation, DiagnosticSeverity};
 
-use super::{position::PanelPosition, view::panel_header};
+use super::{data::PanelSection, position::PanelPosition, view::PanelBuilder};
 use crate::{
     command::InternalCommand,
     config::{color::LapceColor, icon::LapceIcons, LapceConfig},
@@ -27,30 +27,23 @@ pub fn problem_panel(
 ) -> impl View {
     let config = window_tab_data.common.config;
     let is_bottom = position.is_bottom();
-    stack((
-        stack((
-            panel_header("Errors".to_string(), config),
+    PanelBuilder::new(config, position)
+        .add_style(
+            "Errors",
             problem_section(window_tab_data.clone(), DiagnosticSeverity::ERROR),
-        ))
-        .style(move |s| {
-            let config = config.get();
-            s.flex_col()
-                .flex_basis(0.0)
-                .flex_grow(1.0)
-                .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
-                .apply_if(is_bottom, |s| s.border_right(1.0))
-                .apply_if(!is_bottom, |s| s.border_bottom(1.0))
-        }),
-        stack((
-            panel_header("Warnings".to_string(), config),
-            problem_section(window_tab_data, DiagnosticSeverity::WARNING),
-        ))
-        .style(|s| s.flex_col().flex_basis(0.0).flex_grow(1.0)),
-    ))
-    .style(move |s| {
-        s.size_pct(100.0, 100.0)
-            .apply_if(!is_bottom, |s| s.flex_col())
-    })
+            window_tab_data.panel.section_open(PanelSection::Error),
+            move |s| {
+                s.border_color(config.get().color(LapceColor::LAPCE_BORDER))
+                    .apply_if(is_bottom, |s| s.border_right(1.0))
+                    .apply_if(!is_bottom, |s| s.border_bottom(1.0))
+            },
+        )
+        .add(
+            "Warnings",
+            problem_section(window_tab_data.clone(), DiagnosticSeverity::WARNING),
+            window_tab_data.panel.section_open(PanelSection::Warn),
+        )
+        .build()
 }
 
 fn problem_section(
@@ -62,7 +55,7 @@ fn problem_section(
     let internal_command = window_tab_data.common.internal_command;
     container({
         scroll(
-            list(
+            dyn_stack(
                 move || main_split.diagnostics.get(),
                 |(p, _)| p.clone(),
                 move |(path, diagnostic_data)| {
@@ -93,19 +86,40 @@ fn file_view(
 ) -> impl View {
     let collpased = create_rw_signal(false);
 
-    let diagnostics = create_memo(move |_| {
-        let diagnostics = diagnostic_data.diagnostics.get();
-        let diagnostics: im::Vector<EditorDiagnostic> = diagnostics
-            .into_iter()
-            .filter_map(|d| {
-                if d.diagnostic.severity == Some(severity) {
-                    Some(d)
-                } else {
-                    None
-                }
-            })
-            .collect();
-        diagnostics
+    let diagnostics = create_rw_signal(im::Vector::new());
+    create_effect(move |_| {
+        let span = diagnostic_data.diagnostics_span.get();
+        let d = if !span.is_empty() {
+            span.iter()
+                .filter_map(|(iv, diag)| {
+                    if diag.severity == Some(severity) {
+                        Some(EditorDiagnostic {
+                            range: Some((iv.start, iv.end)),
+                            diagnostic: diag.to_owned(),
+                        })
+                    } else {
+                        None
+                    }
+                })
+                .collect::<im::Vector<EditorDiagnostic>>()
+        } else {
+            let diagnostics = diagnostic_data.diagnostics.get();
+            let diagnostics: im::Vector<EditorDiagnostic> = diagnostics
+                .into_iter()
+                .filter_map(|d| {
+                    if d.severity == Some(severity) {
+                        Some(EditorDiagnostic {
+                            range: None,
+                            diagnostic: d,
+                        })
+                    } else {
+                        None
+                    }
+                })
+                .collect();
+            diagnostics
+        };
+        diagnostics.set(d);
     });
 
     let full_path = path.clone();
@@ -125,8 +139,8 @@ fn file_view(
     let icon_color = move || {
         let config = config.get();
         match severity {
-            DiagnosticSeverity::ERROR => *config.get_color(LapceColor::LAPCE_ERROR),
-            _ => *config.get_color(LapceColor::LAPCE_WARN),
+            DiagnosticSeverity::ERROR => config.color(LapceColor::LAPCE_ERROR),
+            _ => config.color(LapceColor::LAPCE_WARN),
         }
     };
 
@@ -150,7 +164,7 @@ fn file_view(
                         s.margin_right(6.0).max_width_pct(100.0).text_ellipsis()
                     }),
                     label(move || folder.clone()).style(move |s| {
-                        s.color(*config.get().get_color(LapceColor::EDITOR_DIM))
+                        s.color(config.get().color(LapceColor::EDITOR_DIM))
                             .min_width(0.0)
                             .text_ellipsis()
                     }),
@@ -168,7 +182,7 @@ fn file_view(
                     .padding_right(10.0)
                     .hover(|s| {
                         s.cursor(CursorStyle::Pointer).background(
-                            *config.get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                            config.color(LapceColor::PANEL_HOVERED_BACKGROUND),
                         )
                     })
             }),
@@ -185,12 +199,12 @@ fn file_view(
                     let size = config.ui.icon_size() as f32;
                     s.margin_right(6.0)
                         .size(size, size)
-                        .color(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE))
+                        .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
                 }),
                 svg(move || config.get().file_svg(&path).0).style(move |s| {
                     let config = config.get();
                     let size = config.ui.icon_size() as f32;
-                    let color = config.file_svg(&style_path).1.copied();
+                    let color = config.file_svg(&style_path).1;
                     s.min_width(size)
                         .size(size, size)
                         .apply_opt(color, Style::color)
@@ -200,7 +214,7 @@ fn file_view(
             .style(|s| s.absolute().items_center().margin_left(10.0)),
         ))
         .style(move |s| s.width_pct(100.0).min_width(0.0)),
-        list(
+        dyn_stack(
             move || {
                 if collpased.get() {
                     im::Vector::new()
@@ -208,7 +222,7 @@ fn file_view(
                     diagnostics.get()
                 }
             },
-            |_| 0,
+            |d| (d.range, d.diagnostic.range),
             move |d| {
                 item_view(
                     full_path.clone(),
@@ -239,9 +253,14 @@ fn item_view(
     config: ReadSignal<Arc<LapceConfig>>,
 ) -> impl View {
     let related = d.diagnostic.related_information.unwrap_or_default();
+    let position = if let Some((start, _)) = d.range {
+        EditorPosition::Offset(start)
+    } else {
+        EditorPosition::Position(d.diagnostic.range.start)
+    };
     let location = EditorLocation {
         path,
-        position: Some(EditorPosition::Position(d.diagnostic.range.start)),
+        position: Some(position),
         scroll_offset: None,
         ignore_unconfirmed: false,
         same_editor_tab: false,
@@ -274,9 +293,7 @@ fn item_view(
             .style(move |s| {
                 s.width_pct(100.0).min_width(0.0).hover(|s| {
                     s.cursor(CursorStyle::Pointer).background(
-                        *config
-                            .get()
-                            .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                        config.get().color(LapceColor::PANEL_HOVERED_BACKGROUND),
                     )
                 })
             })
@@ -299,7 +316,7 @@ fn related_view(
 ) -> impl View {
     let is_empty = related.is_empty();
     stack((
-        list(
+        dyn_stack(
             move || related.clone(),
             |_| 0,
             move |related| {
@@ -342,8 +359,7 @@ fn related_view(
                         .min_width(0.0)
                         .hover(|s| {
                             s.cursor(CursorStyle::Pointer).background(
-                                *config
-                                    .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                                config.color(LapceColor::PANEL_HOVERED_BACKGROUND),
                             )
                         })
                 })
@@ -355,7 +371,7 @@ fn related_view(
                 let config = config.get();
                 let size = config.ui.icon_size() as f32;
                 s.size(size, size)
-                    .color(*config.get_color(LapceColor::EDITOR_DIM))
+                    .color(config.color(LapceColor::EDITOR_DIM))
             }),
             label(|| " ".to_string()),
         ))
@@ -369,7 +385,7 @@ fn related_view(
         s.width_pct(100.0)
             .min_width(0.0)
             .items_start()
-            .color(*config.get().get_color(LapceColor::EDITOR_DIM))
+            .color(config.get().color(LapceColor::EDITOR_DIM))
             .apply_if(is_empty, |s| s.hide())
     })
 }
