@@ -8,16 +8,23 @@ use floem::{
     reactive::{create_memo, create_rw_signal},
     style::{CursorStyle, Style},
     view::View,
-    views::{container, label, list, scroll, stack, svg, Decorators},
+    views::{
+        container, dyn_stack,
+        editor::view::{cursor_caret, LineRegion},
+        label, scroll, stack, svg, text, Decorators,
+    },
 };
 use lapce_core::buffer::rope_text::RopeText;
 use lapce_rpc::source_control::FileDiff;
 
-use super::{kind::PanelKind, position::PanelPosition, view::panel_header};
+use super::{
+    data::PanelSection, kind::PanelKind, position::PanelPosition,
+    view::foldable_panel_section,
+};
 use crate::{
     command::{CommandKind, InternalCommand, LapceCommand, LapceWorkbenchCommand},
     config::{color::LapceColor, icon::LapceIcons},
-    editor::view::{cursor_caret, editor_view, LineRegion},
+    editor::view::editor_view,
     settings::checkbox,
     source_control::SourceControlData,
     window_tab::{Focus, WindowTabData},
@@ -31,10 +38,10 @@ pub fn source_control_panel(
     let source_control = window_tab_data.source_control.clone();
     let focus = source_control.common.focus;
     let editor = source_control.editor.clone();
-    let doc = editor.view.doc;
-    let cursor = editor.cursor;
-    let viewport = editor.viewport;
-    let window_origin = editor.window_origin;
+    let doc = editor.doc_signal();
+    let cursor = editor.cursor();
+    let viewport = editor.viewport();
+    let window_origin = editor.window_origin();
     let editor = create_rw_signal(editor);
     let is_active = move |tracked| {
         let focus = if tracked {
@@ -59,14 +66,13 @@ pub fn source_control_panel(
                             editor.get_untracked(),
                             debug_breakline,
                             is_active,
-                        )
-                        .style(|s| s.min_size_pct(100.0, 100.0)),
+                        ),
                         label(|| "Commit Message".to_string()).style(move |s| {
                             let config = config.get();
                             s.absolute()
                                 .items_center()
                                 .height(config.editor.line_height() as f32)
-                                .color(*config.get_color(LapceColor::EDITOR_DIM))
+                                .color(config.color(LapceColor::EDITOR_DIM))
                                 .apply_if(!is_empty.get(), |s| s.hide())
                         }),
                     ))
@@ -107,20 +113,24 @@ pub fn source_control_panel(
                 .on_scroll(move |rect| {
                     viewport.set(rect);
                 })
-                .on_ensure_visible(move || {
+                .ensure_visible(move || {
                     let cursor = cursor.get();
                     let offset = cursor.offset();
-                    let editor = editor.get_untracked();
-                    let editor_view = editor.view.clone();
-                    editor_view.doc.track();
-                    editor_view.kind.track();
-                    let LineRegion { x, width, line } =
-                        cursor_caret(&editor_view, offset, !cursor.is_insert());
+                    let e_data = editor.get_untracked();
+                    e_data.doc_signal().track();
+                    e_data.kind.track();
+                    let LineRegion { x, width, rvline } = cursor_caret(
+                        &e_data.editor,
+                        offset,
+                        !cursor.is_insert(),
+                        cursor.affinity,
+                    );
                     let config = config.get_untracked();
                     let line_height = config.editor.line_height();
-
+                    // TODO: is there a way to avoid the calculation of the vline here?
+                    let vline = e_data.editor.vline_of_rvline(rvline);
                     Rect::from_origin_size(
-                        (x, (line * line_height) as f64),
+                        (x, (vline.get() * line_height) as f64),
                         (width, line_height as f64),
                     )
                     .inflate(30.0, 10.0)
@@ -134,8 +144,8 @@ pub fn source_control_panel(
                     .border(1.0)
                     .padding(-1.0)
                     .border_radius(6.0)
-                    .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
-                    .background(*config.get_color(LapceColor::EDITOR_BACKGROUND))
+                    .border_color(config.color(LapceColor::LAPCE_BORDER))
+                    .background(config.color(LapceColor::EDITOR_BACKGROUND))
             }),
             {
                 let source_control = source_control.clone();
@@ -151,18 +161,15 @@ pub fn source_control_panel(
                             .justify_center()
                             .border(1.0)
                             .border_radius(6.0)
-                            .border_color(
-                                *config.get_color(LapceColor::LAPCE_BORDER),
-                            )
+                            .border_color(config.color(LapceColor::LAPCE_BORDER))
                             .hover(|s| {
                                 s.cursor(CursorStyle::Pointer).background(
-                                    *config.get_color(
-                                        LapceColor::PANEL_HOVERED_BACKGROUND,
-                                    ),
+                                    config
+                                        .color(LapceColor::PANEL_HOVERED_BACKGROUND),
                                 )
                             })
                             .active(|s| {
-                                s.background(*config.get_color(
+                                s.background(config.color(
                                     LapceColor::PANEL_HOVERED_ACTIVE_BACKGROUND,
                                 ))
                             })
@@ -170,10 +177,12 @@ pub fn source_control_panel(
             },
         ))
         .style(|s| s.flex_col().width_pct(100.0).padding(10.0)),
-        stack((
-            panel_header("Changes".to_string(), config),
+        foldable_panel_section(
+            text("Changes"),
             file_diffs_view(source_control),
-        ))
+            window_tab_data.panel.section_open(PanelSection::Changes),
+            config,
+        )
         .style(|s| s.flex_col().size_pct(100.0, 100.0)),
     ))
     .on_event_stop(EventListener::PointerDown, move |_| {
@@ -230,7 +239,7 @@ fn file_diffs_view(source_control: SourceControlData) -> impl View {
             svg(move || config.get().file_svg(&path).0).style(move |s| {
                 let config = config.get();
                 let size = config.ui.icon_size() as f32;
-                let color = config.file_svg(&style_path).1.copied();
+                let color = config.file_svg(&style_path).1;
                 s.min_width(size)
                     .size(size, size)
                     .margin(6.0)
@@ -254,7 +263,7 @@ fn file_diffs_view(source_control: SourceControlData) -> impl View {
                 s.text_ellipsis()
                     .flex_grow(1.0)
                     .flex_basis(0.0)
-                    .color(*config.get().get_color(LapceColor::EDITOR_DIM))
+                    .color(config.get().color(LapceColor::EDITOR_DIM))
                     .min_width(0.0)
             }),
             container({
@@ -278,8 +287,8 @@ fn file_diffs_view(source_control: SourceControlData) -> impl View {
                             LapceColor::SOURCE_CONTROL_MODIFIED
                         }
                     };
-                    let color = config.get_color(color);
-                    s.min_width(size).size(size, size).color(*color)
+                    let color = config.color(color);
+                    s.min_width(size).size(size, size).color(color)
                 })
             })
             .style(|s| {
@@ -323,16 +332,14 @@ fn file_diffs_view(source_control: SourceControlData) -> impl View {
                 .width_pct(100.0)
                 .items_center()
                 .hover(|s| {
-                    s.background(
-                        *config.get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
-                    )
+                    s.background(config.color(LapceColor::PANEL_HOVERED_BACKGROUND))
                 })
         })
     };
 
     container({
         scroll({
-            list(
+            dyn_stack(
                 move || file_diffs.get(),
                 |(path, (diff, checked))| {
                     (path.to_path_buf(), diff.clone(), *checked)
