@@ -6,8 +6,8 @@ use floem::{
     reactive::create_rw_signal,
     view::View,
     views::{
-        container, empty, label, list,
-        scroll::{scroll, Thickness},
+        container, dyn_stack, empty, label,
+        scroll::{scroll, Thickness, VerticalScrollAsHorizontal},
         stack, svg, tab, Decorators,
     },
     EventPropagation,
@@ -52,7 +52,7 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
     let workbench_command = window_tab_data.common.workbench_command;
 
     stack((
-        scroll(list(
+        scroll(dyn_stack(
             move || {
                 let tabs = terminal.tab_info.with(|info| info.tabs.clone());
                 for (i, (index, _)) in tabs.iter().enumerate() {
@@ -115,9 +115,11 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
                                     .style(move |s| {
                                         let config = config.get();
                                         let size = config.ui.icon_size() as f32;
-                                        s.size(size, size).color(*config.get_color(
-                                            LapceColor::LAPCE_ICON_ACTIVE,
-                                        ))
+                                        s.size(size, size).color(
+                                            config.color(
+                                                LapceColor::LAPCE_ICON_ACTIVE,
+                                            ),
+                                        )
                                     }),
                             )
                             .style(|s| s.padding_horiz(10.0).padding_vert(12.0)),
@@ -134,6 +136,7 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
                                 },
                                 || false,
                                 || false,
+                                || "Close",
                                 config,
                             )
                             .style(|s| s.margin_horiz(6.0)),
@@ -143,15 +146,13 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
                                     .height(header_height.get() - 15.0)
                                     .border_right(1.0)
                                     .border_color(
-                                        *config
-                                            .get()
-                                            .get_color(LapceColor::LAPCE_BORDER),
+                                        config.get().color(LapceColor::LAPCE_BORDER),
                                     )
                             }),
                         ))
                         .style(move |s| {
                             s.items_center().width(200.0).border_color(
-                                *config.get().get_color(LapceColor::LAPCE_BORDER),
+                                config.get().color(LapceColor::LAPCE_BORDER),
                             )
                         })
                     })
@@ -164,7 +165,7 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
                                 } else {
                                     0.0
                                 })
-                                .border_color(*config.get().get_color(
+                                .border_color(config.get().color(
                                     if focus.get()
                                         == Focus::Panel(PanelKind::Terminal)
                                     {
@@ -194,7 +195,6 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
                 )
             },
         ))
-        .vertical_scroll_as_horizontal(|| true)
         .on_resize(move |rect| {
             if rect.size() != scroll_size.get_untracked() {
                 scroll_size.set(rect.size());
@@ -203,7 +203,8 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
         .style(move |s| {
             let header_width = header_width.get();
             let icon_width = icon_width.get();
-            s.absolute()
+            s.set(VerticalScrollAsHorizontal, true)
+                .absolute()
                 .max_width(header_width - icon_width)
                 .set(Thickness, 3)
         }),
@@ -218,6 +219,7 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
             },
             || false,
             || false,
+            || "New Terminal",
             config,
         ))
         .on_resize(move |rect| {
@@ -242,7 +244,7 @@ fn terminal_tab_header(window_tab_data: Rc<WindowTabData>) -> impl View {
         s.width_pct(100.0)
             .items_center()
             .border_bottom(1.0)
-            .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
+            .border_color(config.color(LapceColor::LAPCE_BORDER))
     })
 }
 
@@ -251,9 +253,11 @@ fn terminal_tab_split(
     terminal_tab_data: TerminalTabData,
 ) -> impl View {
     let config = terminal_panel_data.common.config;
+    let internal_command = terminal_panel_data.common.internal_command;
+    let workspace = terminal_panel_data.workspace.clone();
     let active = terminal_tab_data.active;
     let terminal_tab_scope = terminal_tab_data.scope;
-    list(
+    dyn_stack(
         move || {
             let terminals = terminal_tab_data.terminals.get();
             for (i, (index, _)) in terminals.iter().enumerate() {
@@ -275,6 +279,8 @@ fn terminal_tab_split(
                     terminal.run_debug.read_only(),
                     terminal_panel_data,
                     terminal.launch_error,
+                    internal_command,
+                    workspace.clone(),
                 )
                 .on_event_cont(EventListener::PointerDown, move |_| {
                     active.set(index.get_untracked());
@@ -297,7 +303,7 @@ fn terminal_tab_split(
                     index.get() > 0,
                     |s| {
                         s.border_left(1.0).border_color(
-                            *config.get().get_color(LapceColor::LAPCE_BORDER),
+                            config.get().color(LapceColor::LAPCE_BORDER),
                         )
                     },
                 )
