@@ -1,14 +1,11 @@
 use std::{collections::BTreeMap, rc::Rc, sync::Arc, time::Duration};
 
 use floem::{
-    action::{exec_after, TimerToken},
+    action::{add_overlay, exec_after, remove_overlay, TimerToken},
     cosmic_text::{Attrs, AttrsList, FamilyOwned, TextLayout},
     event::EventListener,
-    keyboard::ModifiersState,
-    peniko::{
-        kurbo::{Point, Rect, Size},
-        Color,
-    },
+    keyboard::Modifiers,
+    peniko::kurbo::{Point, Rect, Size},
     reactive::{
         create_effect, create_memo, create_rw_signal, Memo, ReadSignal, RwSignal,
         Scope,
@@ -16,9 +13,10 @@ use floem::{
     style::CursorStyle,
     view::View,
     views::{
-        container, container_box, empty, label, list, scroll, stack, svg, text,
-        virtual_list, Decorators, VirtualListDirection, VirtualListItemSize,
-        VirtualListVector,
+        container, dyn_stack, empty, label,
+        scroll::{scroll, PropagatePointerWheel},
+        stack, svg, text, virtual_stack, Container, Decorators, VirtualDirection,
+        VirtualItemSize, VirtualVector,
     },
 };
 use indexmap::IndexMap;
@@ -34,11 +32,10 @@ use crate::{
         color::LapceColor, core::CoreConfig, editor::EditorConfig, icon::LapceIcons,
         terminal::TerminalConfig, ui::UIConfig, DropdownInfo, LapceConfig,
     },
-    editor::EditorData,
-    id::EditorId,
     keypress::KeyPressFocus,
+    main_split::Editors,
     plugin::InstalledVoltData,
-    text_input::text_input,
+    text_input::TextInputBuilder,
     window_tab::CommonData,
 };
 
@@ -109,7 +106,7 @@ impl KeyPressFocus for SettingsData {
         &self,
         _command: &crate::command::LapceCommand,
         _count: Option<usize>,
-        _mods: ModifiersState,
+        _mods: Modifiers,
     ) -> crate::command::CommandExecuted {
         CommandExecuted::No
     }
@@ -117,14 +114,15 @@ impl KeyPressFocus for SettingsData {
     fn receive_char(&self, _c: &str) {}
 }
 
-impl VirtualListVector<SettingsItem> for SettingsData {
-    type ItemIterator = Box<dyn Iterator<Item = SettingsItem>>;
-
+impl VirtualVector<SettingsItem> for SettingsData {
     fn total_len(&self) -> usize {
         self.filtered_items.get_untracked().len()
     }
 
-    fn slice(&mut self, _range: std::ops::Range<usize>) -> Self::ItemIterator {
+    fn slice(
+        &mut self,
+        _range: std::ops::Range<usize>,
+    ) -> impl Iterator<Item = SettingsItem> {
         Box::new(self.filtered_items.get().into_iter())
     }
 }
@@ -308,6 +306,7 @@ impl SettingsData {
 
 pub fn settings_view(
     installed_plugins: RwSignal<IndexMap<VoltID, InstalledVoltData>>,
+    editors: Editors,
     common: Rc<CommonData>,
 ) -> impl View {
     let config = common.config;
@@ -317,8 +316,8 @@ pub fn settings_view(
     let view_settings_data = settings_data.clone();
     let plugin_kinds = settings_data.plugin_kinds;
 
-    let search_editor = EditorData::new_local(cx, EditorId::next(), common);
-    let doc = search_editor.view.doc;
+    let search_editor = editors.make_local(cx, common);
+    let doc = search_editor.doc_signal();
 
     let items = settings_data.items.clone();
     let kinds = settings_data.kinds.clone();
@@ -399,19 +398,16 @@ pub fn settings_view(
             s.padding_horiz(20.0)
                 .width_pct(100.0)
                 .apply_if(kind == current_kind.get(), |s| {
-                    s.background(
-                        *config.get_color(LapceColor::PANEL_CURRENT_BACKGROUND),
-                    )
+                    s.background(config.color(LapceColor::PANEL_CURRENT_BACKGROUND))
                 })
                 .hover(|s| {
                     s.cursor(CursorStyle::Pointer).background(
-                        *config.get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                        config.color(LapceColor::PANEL_HOVERED_BACKGROUND),
                     )
                 })
                 .active(|s| {
                     s.background(
-                        *config
-                            .get_color(LapceColor::PANEL_HOVERED_ACTIVE_BACKGROUND),
+                        config.color(LapceColor::PANEL_HOVERED_ACTIVE_BACKGROUND),
                     )
                 })
         })
@@ -419,7 +415,7 @@ pub fn settings_view(
 
     let switcher = || {
         stack((
-            list(
+            dyn_stack(
                 move || kinds.clone(),
                 |(k, _)| k.clone(),
                 move |(k, pos)| switcher_item(k, Box::new(move || Some(pos)), 0.0),
@@ -434,7 +430,7 @@ pub fn settings_view(
                     }),
                     0.0,
                 ),
-                list(
+                dyn_stack(
                     move || plugin_kinds.get(),
                     |(k, _)| k.clone(),
                     move |(k, pos)| {
@@ -469,11 +465,12 @@ pub fn settings_view(
             s.height_pct(100.0)
                 .width(200.0)
                 .border_right(1.0)
-                .border_color(*config.get().get_color(LapceColor::LAPCE_BORDER))
+                .border_color(config.get().color(LapceColor::LAPCE_BORDER))
         }),
         stack((
             container({
-                text_input(search_editor, || false)
+                TextInputBuilder::new()
+                    .build_editor(search_editor)
                     .placeholder(|| "Search Settings".to_string())
                     .keyboard_navigatable()
                     .style(move |s| {
@@ -481,22 +478,23 @@ pub fn settings_view(
                             .border_radius(6.0)
                             .border(1.0)
                             .border_color(
-                                *config.get().get_color(LapceColor::LAPCE_BORDER),
+                                config.get().color(LapceColor::LAPCE_BORDER),
                             )
                     })
+                    .request_focus(|| {})
             })
             .style(|s| s.padding_horiz(50.0).padding_vert(20.0)),
             container({
                 scroll({
-                    virtual_list(
-                        VirtualListDirection::Vertical,
-                        floem::views::VirtualListItemSize::Fn(Box::new(
-                            |item: &SettingsItem| item.size.get().height.max(50.0),
-                        )),
-                        move || settings_data.clone(),
+                    dyn_stack(
+                        move || filtered_items_signal.get(),
                         |item| (item.kind.clone(), item.name.clone()),
                         move |item| {
-                            settings_item_view(view_settings_data.clone(), item)
+                            settings_item_view(
+                                editors,
+                                view_settings_data.clone(),
+                                item,
+                            )
                         },
                     )
                     .style(|s| {
@@ -509,7 +507,7 @@ pub fn settings_view(
                 .on_scroll(move |rect| {
                     scroll_pos.set(rect.origin());
                 })
-                .on_ensure_visible(move || ensure_visible.get())
+                .ensure_visible(move || ensure_visible.get())
                 .on_resize(move |rect| {
                     settings_content_size.set(rect.size());
                 })
@@ -522,7 +520,11 @@ pub fn settings_view(
     .style(|s| s.absolute().size_pct(100.0, 100.0))
 }
 
-fn settings_item_view(settings_data: SettingsData, item: SettingsItem) -> impl View {
+fn settings_item_view(
+    editors: Editors,
+    settings_data: SettingsData,
+    item: SettingsItem,
+) -> impl View {
     let config = settings_data.common.config;
 
     let is_ticked = if let SettingsValue::Bool(is_ticked) = &item.value {
@@ -547,18 +549,17 @@ fn settings_item_view(settings_data: SettingsData, item: SettingsItem) -> impl V
         move || {
             let cx = Scope::current();
             if let Some(editor_value) = editor_value {
-                let editor = EditorData::new_local(
-                    cx,
-                    EditorId::next(),
-                    settings_data.common,
-                );
-                let doc = editor.view.doc.get_untracked();
-                doc.reload(Rope::from(editor_value), true);
+                let text_input_view = TextInputBuilder::new()
+                    .value(editor_value)
+                    .build(cx, editors, settings_data.common);
+
+                let doc = text_input_view.doc_signal();
 
                 let kind = item.kind.clone();
                 let field = item.field.clone();
                 let item_value = item.value.clone();
                 create_effect(move |last| {
+                    let doc = doc.get_untracked();
                     let rev = doc.buffer.with(|b| b.rev());
                     if last.is_none() {
                         return rev;
@@ -613,21 +614,13 @@ fn settings_item_view(settings_data: SettingsData, item: SettingsItem) -> impl V
                     rev
                 });
 
-                container_box(
-                    text_input(editor, || false).keyboard_navigatable().style(
-                        move |s| {
-                            s.width(300.0)
-                                .border(1.0)
-                                .border_radius(6.0)
-                                .border_color(
-                                    *config
-                                        .get()
-                                        .get_color(LapceColor::LAPCE_BORDER),
-                                )
-                        },
-                    ),
-                )
-            } else if let SettingsValue::Dropdown(dropdown) = item.value {
+                container(text_input_view.keyboard_navigatable().style(move |s| {
+                    s.width(300.0)
+                        .border(1.0)
+                        .border_radius(6.0)
+                        .border_color(config.get().color(LapceColor::LAPCE_BORDER))
+                }))
+            } else if let SettingsValue::Dropdown(dropdown) = &item.value {
                 let expanded = create_rw_signal(false);
                 let current_value = dropdown
                     .items
@@ -637,143 +630,26 @@ fn settings_item_view(settings_data: SettingsData, item: SettingsItem) -> impl V
                     .unwrap_or_default();
                 let current_value = create_rw_signal(current_value);
 
-                let kind = item.kind.clone();
-                let field = item.field.clone();
-                let view_fn = move |item_string: String| {
-                    let kind = kind.clone();
-                    let field = field.clone();
-                    let local_item_string = item_string.clone();
-                    label(move || local_item_string.clone())
-                        .on_click_stop(move |_| {
-                            current_value.set(item_string.clone());
-                            if let Ok(value) = serde::Serialize::serialize(
-                                &item_string,
-                                toml_edit::ser::ValueSerializer::new(),
-                            ) {
-                                LapceConfig::update_file(&kind, &field, value);
-                            }
-                            expanded.set(false);
-                        })
-                        .style(move |s| {
-                            s.text_ellipsis().padding_horiz(10.0).hover(|s| {
-                                s.cursor(CursorStyle::Pointer).background(
-                                    *config.get().get_color(
-                                        LapceColor::PANEL_HOVERED_BACKGROUND,
-                                    ),
-                                )
-                            })
-                        })
-                };
-                container_box(
-                    stack((
-                        stack((
-                            label(move || current_value.get()).style(move |s| {
-                                s.text_ellipsis()
-                                    .width_pct(100.0)
-                                    .padding_horiz(10.0)
-                            }),
-                            container(
-                                svg(move || {
-                                    config.get().ui_svg(LapceIcons::DROPDOWN_ARROW)
-                                })
-                                .style(move |s| {
-                                    let config = config.get();
-                                    let size = config.ui.icon_size() as f32;
-                                    s.size(size, size).color(
-                                        *config.get_color(
-                                            LapceColor::LAPCE_ICON_ACTIVE,
-                                        ),
-                                    )
-                                }),
-                            )
-                            .style(|s| s.padding_right(4.0)),
-                        ))
-                        .on_click_stop(move |_| {
-                            expanded.update(|expanded| {
-                                *expanded = !*expanded;
-                            });
-                        })
-                        .style(move |s| {
-                            s.items_center()
-                                .width_pct(100.0)
-                                .cursor(CursorStyle::Pointer)
-                                .border_color(Color::TRANSPARENT)
-                                .border(1.0)
-                                .border_radius(6.0)
-                                .apply_if(!expanded.get(), |s| {
-                                    s.border_color(
-                                        *config
-                                            .get()
-                                            .get_color(LapceColor::LAPCE_BORDER),
-                                    )
-                                })
-                        }),
-                        stack((
-                            label(|| " ".to_string()),
-                            scroll({
-                                list(
-                                    move || dropdown.items.clone(),
-                                    |item| item.to_string(),
-                                    view_fn,
-                                )
-                                .style(|s| {
-                                    s.flex_col()
-                                        .width_pct(100.0)
-                                        .cursor(CursorStyle::Pointer)
-                                })
-                            })
-                            .style(move |s| {
-                                let config = config.get();
-                                s.background(
-                                    *config.get_color(LapceColor::EDITOR_BACKGROUND),
-                                )
-                                .width_pct(100.0)
-                                .max_height(300.0)
-                                .z_index(1)
-                                .border_top(1.0)
-                                .border_radius(6.0)
-                                .border_color(
-                                    *config.get_color(LapceColor::LAPCE_BORDER),
-                                )
-                                .apply_if(!expanded.get(), |s| s.hide())
-                            }),
-                        ))
-                        .keyboard_navigatable()
-                        .on_event_stop(EventListener::FocusLost, move |_| {
-                            if expanded.get_untracked() {
-                                expanded.set(false);
-                            }
-                        })
-                        .style(move |s| {
-                            s.absolute()
-                                .flex_col()
-                                .width_pct(100.0)
-                                .border(1.0)
-                                .border_radius(6.0)
-                                .border_color(
-                                    *config
-                                        .get()
-                                        .get_color(LapceColor::LAPCE_BORDER),
-                                )
-                                .apply_if(!expanded.get(), |s| {
-                                    s.border_color(Color::TRANSPARENT)
-                                })
-                        }),
-                    ))
-                    .style(move |s| s.width(250.0).line_height(1.8)),
+                dropdown_view(
+                    &item,
+                    current_value,
+                    dropdown,
+                    expanded,
+                    settings_data.common.window_common.size,
+                    config,
                 )
             } else if item.header {
-                container_box(label(move || item.kind.clone()).style(move |s| {
+                container(label(move || item.kind.clone()).style(move |s| {
                     let config = config.get();
                     s.line_height(2.0)
                         .font_bold()
                         .width_pct(100.0)
                         .padding_horiz(10.0)
                         .font_size(config.ui.font_size() as f32 + 2.0)
-                        .background(*config.get_color(LapceColor::PANEL_BACKGROUND))
+                        .background(config.color(LapceColor::PANEL_BACKGROUND))
                 }))
             } else {
-                container_box(empty())
+                container(empty())
             }
         }
     };
@@ -815,7 +691,7 @@ fn settings_item_view(settings_data: SettingsData, item: SettingsItem) -> impl V
                     }
                 });
 
-                container_box(
+                container(
                     stack((
                         checkbox(move || checked.get(), config),
                         label(|| " ".to_string()).style(|s| s.line_height(1.6)),
@@ -834,7 +710,7 @@ fn settings_item_view(settings_data: SettingsData, item: SettingsItem) -> impl V
                         .items_start()
                 })
             } else {
-                container_box(empty()).style(|s| s.hide())
+                container(empty()).style(|s| s.hide())
             },
         )),
         view().style(move |s| s.apply_if(!item.header, |s| s.margin_top(6.0))),
@@ -864,10 +740,10 @@ pub fn checkbox(
     const CHECKBOX_SVG: &str = r#"<svg xmlns="http://www.w3.org/2000/svg" viewBox="-2 -2 16 16"><polygon points="5.19,11.83 0.18,7.44 1.82,5.56 4.81,8.17 10,1.25 12,2.75" /></svg>"#;
     let svg_str = move || if checked() { CHECKBOX_SVG } else { "" }.to_string();
 
-    svg(svg_str).base_style(move |s| {
+    svg(svg_str).style(move |s| {
         let config = config.get();
         let size = config.ui.font_size() as f32;
-        let color = *config.get_color(LapceColor::EDITOR_FOREGROUND);
+        let color = config.color(LapceColor::EDITOR_FOREGROUND);
 
         s.min_width(size)
             .size(size, size)
@@ -880,14 +756,15 @@ pub fn checkbox(
 
 struct BTreeMapVirtualList(BTreeMap<String, String>);
 
-impl VirtualListVector<(String, String)> for BTreeMapVirtualList {
-    type ItemIterator = Box<dyn Iterator<Item = (String, String)>>;
-
+impl VirtualVector<(String, String)> for BTreeMapVirtualList {
     fn total_len(&self) -> usize {
         self.0.len()
     }
 
-    fn slice(&mut self, range: std::ops::Range<usize>) -> Self::ItemIterator {
+    fn slice(
+        &mut self,
+        range: std::ops::Range<usize>,
+    ) -> impl Iterator<Item = (String, String)> {
         Box::new(
             self.0
                 .iter()
@@ -911,6 +788,7 @@ fn color_section_list(
     list: impl Fn() -> BTreeMap<String, String> + 'static,
     max_width: Memo<f64>,
     text_height: Memo<f64>,
+    editors: Editors,
     common: Rc<CommonData>,
 ) -> impl View {
     let config = common.config;
@@ -923,23 +801,24 @@ fn color_section_list(
                 .font_bold()
                 .line_height(2.0)
         }),
-        virtual_list(
-            VirtualListDirection::Vertical,
-            VirtualListItemSize::Fixed(Box::new(move || text_height.get() + 24.0)),
+        virtual_stack(
+            VirtualDirection::Vertical,
+            VirtualItemSize::Fixed(Box::new(move || text_height.get() + 24.0)),
             move || BTreeMapVirtualList(list()),
             move |(key, _)| (key.to_owned()),
             move |(key, value)| {
                 let cx = Scope::current();
-                let editor =
-                    EditorData::new_local(cx, EditorId::next(), common.clone());
-                let doc = editor.view.doc.get_untracked();
-                doc.reload(Rope::from(value.clone()), true);
+                let text_input_view = TextInputBuilder::new()
+                    .value(value.clone())
+                    .build(cx, editors, common.clone());
+                let doc = text_input_view.doc_signal();
 
                 {
                     let kind = kind.clone();
                     let key = key.clone();
-                    let doc = doc.clone();
+                    let doc = text_input_view.doc_signal();
                     create_effect(move |_| {
+                        let doc = doc.get_untracked();
                         let config = config.get();
                         let current = doc.buffer.with_untracked(|b| b.to_string());
 
@@ -962,8 +841,9 @@ fn color_section_list(
                     let timer = create_rw_signal(TimerToken::INVALID);
                     let kind = kind.clone();
                     let field = key.clone();
-                    let doc = doc.clone();
                     create_effect(move |last| {
+                        let doc = doc.get_untracked();
+
                         let rev = doc.buffer.with(|b| b.rev());
                         if last.is_none() {
                             return rev;
@@ -984,15 +864,15 @@ fn color_section_list(
                                         let config = config.get_untracked();
                                         let default = match kind.as_str() {
                                             "base" => config
-                                                .default_color_theme
+                                                .default_color_theme()
                                                 .base
                                                 .get(&field),
                                             "ui" => config
-                                                .default_color_theme
+                                                .default_color_theme()
                                                 .ui
                                                 .get(&field),
                                             "syntax" => config
-                                                .default_color_theme
+                                                .default_color_theme()
                                                 .syntax
                                                 .get(&field),
                                             _ => None,
@@ -1034,37 +914,31 @@ fn color_section_list(
                     text(&key).style(move |s| {
                         s.width(max_width.get()).margin_left(20).margin_right(10)
                     }),
-                    text_input(editor, || false).keyboard_navigatable().style(
-                        move |s| {
-                            s.width(150.0)
-                                .margin_vert(6)
-                                .border(1)
-                                .border_radius(6)
-                                .border_color(
-                                    *config
-                                        .get()
-                                        .get_color(LapceColor::LAPCE_BORDER),
-                                )
-                        },
-                    ),
+                    text_input_view.keyboard_navigatable().style(move |s| {
+                        s.width(150.0)
+                            .margin_vert(6)
+                            .border(1)
+                            .border_radius(6)
+                            .border_color(
+                                config.get().color(LapceColor::LAPCE_BORDER),
+                            )
+                    }),
                     empty().style(move |s| {
                         let size = text_height.get() + 12.0;
                         let config = config.get();
                         let color = match local_kind.as_str() {
                             "base" => config.color.base.get(&local_key),
-                            "ui" => config.color.ui.get(&local_key),
-                            "syntax" => config.color.syntax.get(&local_key),
+                            "ui" => config.color.ui.get(&local_key).copied(),
+                            "syntax" => config.color.syntax.get(&local_key).copied(),
                             _ => None,
                         };
                         s.border(1)
                             .border_radius(6)
                             .size(size, size)
                             .margin_left(10)
-                            .border_color(
-                                *config.get_color(LapceColor::LAPCE_BORDER),
-                            )
-                            .background(*color.unwrap_or_else(|| {
-                                config.get_color(LapceColor::EDITOR_FOREGROUND)
+                            .border_color(config.color(LapceColor::LAPCE_BORDER))
+                            .background(color.unwrap_or_else(|| {
+                                config.color(LapceColor::EDITOR_FOREGROUND)
                             }))
                     }),
                     {
@@ -1080,21 +954,22 @@ fn color_section_list(
                                 );
                             })
                             .style(move |s| {
+                                let doc = doc.get_untracked();
                                 let config = config.get();
                                 let buffer = doc.buffer;
                                 let content = buffer.with(|b| b.to_string());
 
                                 let same = match kind.as_str() {
                                     "base" => {
-                                        config.default_color_theme.base.get(&key)
+                                        config.default_color_theme().base.get(&key)
                                             == Some(&content)
                                     }
                                     "ui" => {
-                                        config.default_color_theme.ui.get(&key)
+                                        config.default_color_theme().ui.get(&key)
                                             == Some(&content)
                                     }
                                     "syntax" => {
-                                        config.default_color_theme.syntax.get(&key)
+                                        config.default_color_theme().syntax.get(&key)
                                             == Some(&content)
                                     }
                                     _ => false,
@@ -1106,14 +981,13 @@ fn color_section_list(
                                     .border(1)
                                     .border_radius(6)
                                     .border_color(
-                                        *config.get_color(LapceColor::LAPCE_BORDER),
+                                        config.color(LapceColor::LAPCE_BORDER),
                                     )
                                     .apply_if(same, |s| s.hide())
                                     .active(|s| {
                                         s.background(
-                                            *config.get_color(
-                                                LapceColor::PANEL_BACKGROUND,
-                                            ),
+                                            config
+                                                .color(LapceColor::PANEL_BACKGROUND),
                                         )
                                     })
                             })
@@ -1127,7 +1001,10 @@ fn color_section_list(
     .style(|s| s.flex_col())
 }
 
-pub fn theme_color_settings_view(common: Rc<CommonData>) -> impl View {
+pub fn theme_color_settings_view(
+    editors: Editors,
+    common: Rc<CommonData>,
+) -> impl View {
     let config = common.config;
 
     let text_height = create_memo(move |_| {
@@ -1179,6 +1056,7 @@ pub fn theme_color_settings_view(common: Rc<CommonData>) -> impl View {
                 move || config.with(|c| c.color_theme.base.key_values()),
                 max_width,
                 text_height,
+                editors,
                 common.clone(),
             ),
             color_section_list(
@@ -1187,6 +1065,7 @@ pub fn theme_color_settings_view(common: Rc<CommonData>) -> impl View {
                 move || config.with(|c| c.color_theme.syntax.clone()),
                 max_width,
                 text_height,
+                editors,
                 common.clone(),
             ),
             color_section_list(
@@ -1195,6 +1074,7 @@ pub fn theme_color_settings_view(common: Rc<CommonData>) -> impl View {
                 move || config.with(|c| c.color_theme.ui.clone()),
                 max_width,
                 text_height,
+                editors,
                 common.clone(),
             ),
         ))
@@ -1202,3 +1082,220 @@ pub fn theme_color_settings_view(common: Rc<CommonData>) -> impl View {
     )
     .style(|s| s.absolute().size_full())
 }
+
+fn dropdown_view(
+    item: &SettingsItem,
+    current_value: RwSignal<String>,
+    dropdown: &DropdownInfo,
+    expanded: RwSignal<bool>,
+    window_size: RwSignal<Size>,
+    config: ReadSignal<Arc<LapceConfig>>,
+) -> Container {
+    let window_origin = create_rw_signal(Point::ZERO);
+    let size = create_rw_signal(Size::ZERO);
+    let overlay_id = create_rw_signal(None);
+    let dropdown_input_focus = create_rw_signal(false);
+    let dropdown_scroll_focus = create_rw_signal(true);
+
+    {
+        let item = item.to_owned();
+        let dropdown = dropdown.to_owned();
+        create_effect(move |_| {
+            if expanded.get() {
+                let item = item.clone();
+                let dropdown = dropdown.clone();
+                let id = add_overlay(Point::ZERO, move |_| {
+                    dropdown_scroll(
+                        &item.clone(),
+                        current_value,
+                        &dropdown.clone(),
+                        expanded,
+                        dropdown_scroll_focus,
+                        dropdown_input_focus,
+                        window_origin,
+                        size,
+                        window_size,
+                        config,
+                    )
+                    .build()
+                });
+                overlay_id.set(Some(id));
+            } else if let Some(id) = overlay_id.get_untracked() {
+                remove_overlay(id);
+                overlay_id.set(None);
+            }
+        });
+    }
+
+    container(
+        stack((
+            label(move || current_value.get()).style(move |s| {
+                s.text_ellipsis().width_pct(100.0).padding_horiz(10.0)
+            }),
+            container(
+                svg(move || {
+                    if expanded.get() {
+                        config.get().ui_svg(LapceIcons::CLOSE)
+                    } else {
+                        config.get().ui_svg(LapceIcons::DROPDOWN_ARROW)
+                    }
+                })
+                .style(move |s| {
+                    let config = config.get();
+                    let size = config.ui.icon_size() as f32;
+                    s.size(size, size)
+                        .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
+                }),
+            )
+            .style(|s| s.padding_right(4.0)),
+        ))
+        .on_click_stop(move |_| {
+            expanded.update(|expanded| {
+                *expanded = !*expanded;
+            });
+        })
+        .on_move(move |point| {
+            window_origin.set(point);
+            if expanded.get_untracked() {
+                expanded.set(false);
+            }
+        })
+        .on_resize(move |rect| {
+            size.set(rect.size());
+        })
+        .style(move |s| {
+            s.items_center()
+                .width_pct(100.0)
+                .cursor(CursorStyle::Pointer)
+                .border_color(config.get().color(LapceColor::LAPCE_BORDER))
+                .border(1.0)
+                .border_radius(6.0)
+        }),
+    )
+    .keyboard_navigatable()
+    .on_event_stop(EventListener::FocusGained, move |_| {
+        dropdown_input_focus.set(true);
+    })
+    .on_event_stop(EventListener::FocusLost, move |_| {
+        dropdown_input_focus.set(false);
+        if expanded.get_untracked() && !dropdown_scroll_focus.get_untracked() {
+            expanded.set(false);
+        }
+    })
+    .style(move |s| s.width(250.0).line_height(1.8))
+    .on_cleanup(move || {
+        if let Some(id) = overlay_id.get_untracked() {
+            remove_overlay(id);
+        }
+    })
+}
+
+#[allow(clippy::too_many_arguments)]
+fn dropdown_scroll(
+    item: &SettingsItem,
+    current_value: RwSignal<String>,
+    dropdown: &DropdownInfo,
+    expanded: RwSignal<bool>,
+    dropdown_scroll_focus: RwSignal<bool>,
+    dropdown_input_focus: RwSignal<bool>,
+    window_origin: RwSignal<Point>,
+    input_size: RwSignal<Size>,
+    window_size: RwSignal<Size>,
+    config: ReadSignal<Arc<LapceConfig>>,
+) -> impl View {
+    dropdown_scroll_focus.set(true);
+
+    let kind = item.kind.clone();
+    let field = item.field.clone();
+    let view_fn = move |item_string: String| {
+        let kind = kind.clone();
+        let field = field.clone();
+        let local_item_string = item_string.clone();
+        label(move || local_item_string.clone())
+            .on_click_stop(move |_| {
+                current_value.set(item_string.clone());
+                if let Ok(value) = serde::Serialize::serialize(
+                    &item_string,
+                    toml_edit::ser::ValueSerializer::new(),
+                ) {
+                    LapceConfig::update_file(&kind, &field, value);
+                }
+                expanded.set(false);
+            })
+            .style(move |s| {
+                s.text_ellipsis().padding_horiz(10.0).hover(|s| {
+                    s.cursor(CursorStyle::Pointer).background(
+                        config.get().color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                    )
+                })
+            })
+    };
+
+    let items = dropdown.items.clone();
+
+    let scroll_size = create_rw_signal(Size::ZERO);
+
+    scroll({
+        dyn_stack(move || items.clone(), |item| item.to_string(), view_fn)
+            .style(|s| s.flex_col().width_pct(100.0).cursor(CursorStyle::Pointer))
+    })
+    .style(move |s| {
+        s.width_pct(100.0)
+            .max_height(200.0)
+            .set(PropagatePointerWheel, false)
+    })
+    .keyboard_navigatable()
+    .request_focus(|| {})
+    .on_event_stop(EventListener::FocusGained, move |_| {
+        dropdown_scroll_focus.set(true);
+    })
+    .on_event_stop(EventListener::FocusLost, move |_| {
+        dropdown_scroll_focus.set(false);
+        if expanded.get_untracked() && !dropdown_input_focus.get_untracked() {
+            expanded.set(false);
+        }
+    })
+    .on_event_stop(EventListener::PointerMove, move |_| {})
+    .on_event_stop(EventListener::PointerDown, move |_| {})
+    .on_resize(move |rect| {
+        scroll_size.set(rect.size());
+    })
+    .style(move |s| {
+        let config = config.get();
+        let window_origin = window_origin.get();
+        let window_size = window_size.get();
+        let input_size = input_size.get();
+        let scroll_size = scroll_size.get();
+
+        let x = if window_origin.x + scroll_size.width + 5.0 > window_size.width {
+            window_size.width - scroll_size.width - 5.0
+        } else {
+            window_origin.x
+        };
+
+        let y = if window_origin.y + input_size.height + scroll_size.height + 5.0
+            > window_size.height
+        {
+            window_origin.y - scroll_size.height + 1.0
+        } else {
+            window_origin.y + input_size.height - 1.0
+        };
+
+        s.width(250.0)
+            .line_height(1.8)
+            .font_size(config.ui.font_size() as f32)
+            .font_family(config.ui.font_family.clone())
+            .color(config.color(LapceColor::EDITOR_FOREGROUND))
+            .background(config.color(LapceColor::EDITOR_BACKGROUND))
+            .class(floem::views::scroll::Handle, |s| {
+                s.background(config.color(LapceColor::LAPCE_SCROLL_BAR))
+            })
+            .border(1)
+            .border_radius(6.0)
+            .border_color(config.color(LapceColor::LAPCE_BORDER))
+            .box_shadow_blur(3.0)
+            .box_shadow_color(config.color(LapceColor::LAPCE_DROPDOWN_SHADOW))
+            .inset_left(x)
+            .inset_top(y)
+    })
+}
