@@ -4,10 +4,10 @@ use std::{
 };
 
 use floem::{
-    reactive::{create_memo, ReadSignal, RwSignal},
+    reactive::{create_memo, Memo, ReadSignal, RwSignal},
     style::{AlignItems, CursorStyle, Display},
     view::View,
-    views::{label, list, stack, svg, Decorators},
+    views::{dyn_stack, label, stack, svg, Decorators},
 };
 use indexmap::IndexMap;
 use lapce_core::mode::{Mode, VisualMode};
@@ -17,6 +17,7 @@ use crate::{
     app::clickable_icon,
     command::LapceWorkbenchCommand,
     config::{color::LapceColor, icon::LapceIcons, LapceConfig},
+    editor::EditorData,
     listener::Listener,
     palette::kind::PaletteKind,
     panel::{kind::PanelKind, position::PanelContainerPosition},
@@ -41,7 +42,7 @@ pub fn status(
         let mut warnings = 0;
         for (_, diagnostics) in diagnostics.get().iter() {
             for diagnostic in diagnostics.diagnostics.get().iter() {
-                if let Some(severity) = diagnostic.diagnostic.severity {
+                if let Some(severity) = diagnostic.severity {
                     match severity {
                         DiagnosticSeverity::ERROR => errors += 1,
                         DiagnosticSeverity::WARNING => warnings += 1,
@@ -68,6 +69,7 @@ pub fn status(
 
     let progresses = window_tab_data.progresses;
     let mode = create_memo(move |_| window_tab_data.mode());
+    let pointer_down = floem::reactive::create_rw_signal(false);
 
     stack((
         stack((
@@ -108,8 +110,8 @@ pub fn status(
                     ),
                 };
 
-                let bg = *config.get_color(bg);
-                let fg = *config.get_color(fg);
+                let bg = config.color(bg);
+                let fg = config.color(fg);
 
                 s.display(display)
                     .padding_horiz(10.0)
@@ -123,12 +125,11 @@ pub fn status(
                     let config = config.get();
                     let icon_size = config.ui.icon_size() as f32;
                     s.size(icon_size, icon_size)
-                        .color(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE))
+                        .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
                 }),
                 label(branch).style(move |s| {
-                    s.margin_left(10.0).color(
-                        *config.get().get_color(LapceColor::STATUS_FOREGROUND),
-                    )
+                    s.margin_left(10.0)
+                        .color(config.get().color(LapceColor::STATUS_FOREGROUND))
                 }),
             ))
             .style(move |s| {
@@ -142,15 +143,24 @@ pub fn status(
                 .align_items(Some(AlignItems::Center))
                 .hover(|s| {
                     s.cursor(CursorStyle::Pointer).background(
-                        *config
-                            .get()
-                            .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                        config.get().color(LapceColor::PANEL_HOVERED_BACKGROUND),
                     )
                 })
             })
-            .on_click_stop(move |_| {
-                workbench_command.send(LapceWorkbenchCommand::PaletteSCMReferences);
-            }),
+            .on_event_cont(floem::event::EventListener::PointerDown, move |_| {
+                pointer_down.set(true);
+            })
+            .on_event(
+                floem::event::EventListener::PointerUp,
+                move |_| {
+                    if pointer_down.get() {
+                        workbench_command
+                            .send(LapceWorkbenchCommand::PaletteSCMReferences);
+                    }
+                    pointer_down.set(false);
+                    floem::EventPropagation::Continue
+                },
+            ),
             {
                 let panel = panel.clone();
                 stack((
@@ -158,17 +168,14 @@ pub fn status(
                         move |s| {
                             let config = config.get();
                             let size = config.ui.icon_size() as f32;
-                            s.size(size, size).color(
-                                *config.get_color(LapceColor::LAPCE_ICON_ACTIVE),
-                            )
+                            s.size(size, size)
+                                .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
                         },
                     ),
                     label(move || diagnostic_count.get().0.to_string()).style(
                         move |s| {
                             s.margin_left(5.0).color(
-                                *config
-                                    .get()
-                                    .get_color(LapceColor::STATUS_FOREGROUND),
+                                config.get().color(LapceColor::STATUS_FOREGROUND),
                             )
                         },
                     ),
@@ -176,17 +183,15 @@ pub fn status(
                         move |s| {
                             let config = config.get();
                             let size = config.ui.icon_size() as f32;
-                            s.size(size, size).margin_left(5.0).color(
-                                *config.get_color(LapceColor::LAPCE_ICON_ACTIVE),
-                            )
+                            s.size(size, size)
+                                .margin_left(5.0)
+                                .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
                         },
                     ),
                     label(move || diagnostic_count.get().1.to_string()).style(
                         move |s| {
                             s.margin_left(5.0).color(
-                                *config
-                                    .get()
-                                    .get_color(LapceColor::STATUS_FOREGROUND),
+                                config.get().color(LapceColor::STATUS_FOREGROUND),
                             )
                         },
                     ),
@@ -200,9 +205,9 @@ pub fn status(
                         .items_center()
                         .hover(|s| {
                             s.cursor(CursorStyle::Pointer).background(
-                                *config
+                                config
                                     .get()
-                                    .get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                                    .color(LapceColor::PANEL_HOVERED_BACKGROUND),
                             )
                         })
                 })
@@ -237,6 +242,7 @@ pub fn status(
                     },
                     || false,
                     || false,
+                    || "Toggle Left Panel",
                     config,
                 )
             },
@@ -263,6 +269,7 @@ pub fn status(
                     },
                     || false,
                     || false,
+                    || "Toggle Bottom Panel",
                     config,
                 )
             },
@@ -287,6 +294,7 @@ pub fn status(
                     },
                     || false,
                     || false,
+                    || "Toggle Right Panel",
                     config,
                 )
             },
@@ -294,17 +302,16 @@ pub fn status(
         .style(move |s| {
             s.height_pct(100.0)
                 .items_center()
-                .color(*config.get().get_color(LapceColor::STATUS_FOREGROUND))
+                .color(config.get().color(LapceColor::STATUS_FOREGROUND))
         }),
         stack({
             let palette_clone = palette.clone();
-            let cursor_info = label(move || {
+            let cursor_info = status_text(config, editor, move || {
                 if let Some(editor) = editor.get() {
                     let mut status = String::new();
-                    let cursor = editor.cursor.get();
+                    let cursor = editor.cursor().get();
                     if let Some((line, column, character)) = editor
-                        .view
-                        .doc
+                        .doc_signal()
                         .get()
                         .buffer
                         .with(|buffer| cursor.get_line_col_char(buffer))
@@ -332,70 +339,32 @@ pub fn status(
             })
             .on_click_stop(move |_| {
                 palette_clone.run(PaletteKind::Line);
-            })
-            .style(move |s| {
-                let config = config.get();
-                s.display(
-                    if editor
-                        .get()
-                        .map(|editor| {
-                            editor.view.doc.get().content.with(|c| c.is_file())
-                        })
-                        .unwrap_or(false)
-                    {
-                        Display::Flex
-                    } else {
-                        Display::None
-                    },
-                )
-                .height_pct(100.0)
-                .padding_horiz(10.0)
-                .items_center()
-                .color(*config.get_color(LapceColor::STATUS_FOREGROUND))
-                .hover(|s| {
-                    s.cursor(CursorStyle::Pointer).background(
-                        *config.get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
-                    )
-                })
             });
             let palette_clone = palette.clone();
-            let language_info = label(move || {
+            let line_ending_info = status_text(config, editor, move || {
                 if let Some(editor) = editor.get() {
-                    let doc = editor.view.doc.get();
-                    doc.syntax.with(|s| s.language.name())
+                    let doc = editor.doc_signal().get();
+                    doc.buffer.with(|b| b.line_ending()).as_str()
+                } else {
+                    ""
+                }
+            })
+            .on_click_stop(move |_| {
+                palette_clone.run(PaletteKind::LineEnding);
+            });
+            let palette_clone = palette.clone();
+            let language_info = status_text(config, editor, move || {
+                if let Some(editor) = editor.get() {
+                    let doc = editor.doc_signal().get();
+                    doc.syntax().with(|s| s.language.name())
                 } else {
                     "unknown"
                 }
             })
             .on_click_stop(move |_| {
                 palette_clone.run(PaletteKind::Language);
-            })
-            .style(move |s| {
-                let config = config.get();
-                s.display(
-                    if editor
-                        .get()
-                        .map(|editor| {
-                            editor.view.doc.get().content.with(|c| c.is_file())
-                        })
-                        .unwrap_or(false)
-                    {
-                        Display::Flex
-                    } else {
-                        Display::None
-                    },
-                )
-                .height_pct(100.0)
-                .padding_horiz(10.0)
-                .items_center()
-                .color(*config.get_color(LapceColor::STATUS_FOREGROUND))
-                .hover(|s| {
-                    s.cursor(CursorStyle::Pointer).background(
-                        *config.get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
-                    )
-                })
             });
-            (cursor_info, language_info)
+            (cursor_info, line_ending_info, language_info)
         })
         .style(|s| {
             s.height_pct(100.0)
@@ -413,8 +382,8 @@ pub fn status(
     .style(move |s| {
         let config = config.get();
         s.border_top(1.0)
-            .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
-            .background(*config.get_color(LapceColor::STATUS_BACKGROUND))
+            .border_color(config.color(LapceColor::LAPCE_BORDER))
+            .background(config.color(LapceColor::STATUS_BACKGROUND))
             .height(config.ui.status_height() as f32)
             .align_items(Some(AlignItems::Center))
     })
@@ -425,7 +394,7 @@ fn progress_view(
     progresses: RwSignal<IndexMap<ProgressToken, WorkProgress>>,
 ) -> impl View {
     let id = AtomicU64::new(0);
-    list(
+    dyn_stack(
         move || progresses.get(),
         move |_| id.fetch_add(1, std::sync::atomic::Ordering::Relaxed),
         move |(_, p)| {
@@ -440,8 +409,37 @@ fn progress_view(
             }))
             .style(move |s| {
                 s.margin_left(10.0)
-                    .color(*config.get().get_color(LapceColor::STATUS_FOREGROUND))
+                    .color(config.get().color(LapceColor::STATUS_FOREGROUND))
             })
         },
     )
 }
+
+fn status_text<S: std::fmt::Display + 'static>(
+    config: ReadSignal<Arc<LapceConfig>>,
+    editor: Memo<Option<EditorData>>,
+    text: impl Fn() -> S + 'static,
+) -> impl View {
+    label(text).style(move |s| {
+        let config = config.get();
+        let display = if editor
+            .get()
+            .map(|editor| editor.doc_signal().get().content.with(|c| c.is_file()))
+            .unwrap_or(false)
+        {
+            Display::Flex
+        } else {
+            Display::None
+        };
+
+        s.display(display)
+            .height_full()
+            .padding_horiz(10.0)
+            .items_center()
+            .color(config.color(LapceColor::STATUS_FOREGROUND))
+            .hover(|s| {
+                s.cursor(CursorStyle::Pointer)
+                    .background(config.color(LapceColor::PANEL_HOVERED_BACKGROUND))
+            })
+    })
+}
