@@ -10,15 +10,18 @@ use floem::{
         kurbo::{Line, Point, Rect, Size, Vec2},
         Color,
     },
-    prop_extracter,
-    reactive::{create_effect, create_memo, create_rw_signal, ReadSignal, RwSignal},
+    prop_extractor,
+    reactive::{
+        create_effect, create_memo, create_rw_signal, Memo, ReadSignal, RwSignal,
+        Scope,
+    },
     style::{
         CursorStyle, FontFamily, FontSize, FontStyle, FontWeight, LineHeight,
         PaddingLeft, Style, TextColor,
     },
-    taffy::prelude::Node,
+    taffy::prelude::NodeId,
     unit::PxPct,
-    view::{View, ViewData},
+    view::{AnyWidget, View, ViewData, Widget},
     views::Decorators,
     EventPropagation, Renderer,
 };
@@ -27,16 +30,19 @@ use lapce_core::{
     cursor::{Cursor, CursorMode},
     selection::Selection,
 };
+use lapce_xi_rope::Rope;
 
 use crate::{
     config::{color::LapceColor, LapceConfig},
-    doc::Document,
-    editor::EditorData,
+    doc::Doc,
+    editor::{view::editor_style, DocSignal, EditorData},
     keypress::KeyPressFocus,
+    main_split::Editors,
+    window_tab::CommonData,
 };
 
-prop_extracter! {
-    Extracter {
+prop_extractor! {
+    Extractor {
         color: TextColor,
         font_size: FontSize,
         font_family: FontFamily,
@@ -46,21 +52,103 @@ prop_extracter! {
     }
 }
 
-pub fn text_input(
-    editor: EditorData,
-    is_focused: impl Fn() -> bool + 'static,
+/// Builder for creating a [`TextInput`] easily.
+pub struct TextInputBuilder {
+    is_focused: Option<Memo<bool>>,
+    // TODO: it'd be nice to not need to box this
+    key_focus: Option<Box<dyn KeyPressFocus>>,
+    value: Option<Rope>,
+    keyboard_focus: RwSignal<bool>,
+}
+
+impl Default for TextInputBuilder {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+impl TextInputBuilder {
+    pub fn new() -> Self {
+        Self {
+            is_focused: None,
+            key_focus: None,
+            value: None,
+            keyboard_focus: create_rw_signal(false),
+        }
+    }
+
+    pub fn is_focused(mut self, is_focused: impl Fn() -> bool + 'static) -> Self {
+        let keyboard_focus = self.keyboard_focus;
+        self.is_focused =
+            Some(create_memo(move |_| is_focused() || keyboard_focus.get()));
+        self
+    }
+
+    /// Initialize with a specific value.  
+    /// If this is set it will apply the value via reloading the editor's doc as pristine.
+    pub fn value(mut self, value: impl Into<Rope>) -> Self {
+        self.value = Some(value.into());
+        self
+    }
+
+    pub fn key_focus(mut self, key_focus: impl KeyPressFocus + 'static) -> Self {
+        self.key_focus = Some(Box::new(key_focus));
+        self
+    }
+
+    pub fn build(
+        self,
+        cx: Scope,
+        editors: Editors,
+        common: Rc<CommonData>,
+    ) -> TextInput {
+        let editor = editors.make_local(cx, common);
+        let id = editor.id();
+
+        self.build_editor(editor).on_cleanup(move || {
+            editors.remove(id);
+        })
+    }
+
+    /// Build the text input with a specific editor.  
+    /// This function does *not* perform add/cleanup the editor to/from [`Editors`]
+    pub fn build_editor(self, editor: EditorData) -> TextInput {
+        let keyboard_focus = self.keyboard_focus;
+        let is_focused = if let Some(is_focused) = self.is_focused {
+            is_focused
+        } else {
+            create_memo(move |_| keyboard_focus.get())
+        };
+
+        if let Some(value) = self.value {
+            editor.doc().reload(value, true);
+        }
+
+        text_input_full(editor, self.key_focus, is_focused, keyboard_focus)
+    }
+}
+
+/// Create a basic single line text input  
+/// `e_data` is the editor data that this input is associated with.  
+/// `supplied_editor`
+/// `key_focus` is what receives the keydown events, leave as `None` to default to editor.  
+/// `is_focused` is a function that returns if the input is focused, used for certain events.
+fn text_input_full<T: KeyPressFocus + 'static>(
+    e_data: EditorData,
+    key_focus: Option<T>,
+    is_focused: Memo<bool>,
+    keyboard_focus: RwSignal<bool>,
 ) -> TextInput {
     let id = Id::next();
 
-    let doc = editor.view.doc;
-    let cursor = editor.cursor;
-    let config = editor.common.config;
-    let keypress = editor.common.keypress;
+    let doc = e_data.doc_signal();
+    let cursor = e_data.cursor();
+    let config = e_data.common.config;
+    let keypress = e_data.common.keypress;
     let window_origin = create_rw_signal(Point::ZERO);
     let cursor_line = create_rw_signal(Line::new(Point::ZERO, Point::ZERO));
-    let keyboard_focus = create_rw_signal(false);
-    let is_focused = create_memo(move |_| is_focused() || keyboard_focus.get());
-    let local_editor = editor.clone();
+    let local_editor = e_data.clone();
+    let editor = local_editor.editor.clone();
 
     {
         let doc = doc.get();
@@ -68,7 +156,7 @@ pub fn text_input(
             let offset = cursor.with(|c| c.offset());
             let (content, offset, preedit_range) = {
                 let content = doc.buffer.with(|b| b.to_string());
-                if let Some(preedit) = doc.preedit.get().as_ref() {
+                if let Some(preedit) = doc.preedit.preedit.get().as_ref() {
                     let mut new_content = String::new();
                     new_content.push_str(&content[..offset]);
                     new_content.push_str(&preedit.text);
@@ -84,25 +172,22 @@ pub fn text_input(
                     (content, offset, None)
                 }
             };
-            id.update_state(
-                TextInputState::Content {
-                    text: content,
-                    offset,
-                    preedit_range,
-                },
-                false,
-            );
+            id.update_state(TextInputState::Content {
+                text: content,
+                offset,
+                preedit_range,
+            });
         });
     }
 
     {
         create_effect(move |_| {
             let focus = is_focused.get();
-            id.update_state(TextInputState::Focus(focus), false);
+            id.update_state(TextInputState::Focus(focus));
         });
 
         let editor = editor.clone();
-        let ime_allowed = editor.common.window_common.ime_allowed;
+        let ime_allowed = editor.ime_allowed;
         create_effect(move |_| {
             let focus = is_focused.get();
             if focus {
@@ -124,8 +209,10 @@ pub fn text_input(
         });
     }
 
-    let common_keyboard_focus = editor.common.keyboard_focus;
+    let common_keyboard_focus = e_data.common.keyboard_focus;
 
+    let ed1 = editor.clone();
+    let ed2 = editor.clone();
     TextInput {
         id,
         data: ViewData::new(id),
@@ -142,15 +229,15 @@ pub fn text_input(
         cursor_line,
         placeholder: "".to_string(),
         placeholder_text_layout: None,
-        cursor,
-        doc,
+        editor: e_data.clone(),
         cursor_pos: Point::ZERO,
         on_cursor_pos: None,
-        hide_cursor: editor.common.window_common.hide_cursor,
+        hide_cursor: editor.cursor_info.hidden,
         style: Default::default(),
     }
-    .base_style(|s| {
-        s.cursor(CursorStyle::Text)
+    .style(move |s| {
+        editor_style(config, doc, s)
+            .cursor(CursorStyle::Text)
             .padding_horiz(10.0)
             .padding_vert(6.0)
     })
@@ -170,7 +257,11 @@ pub fn text_input(
     .on_event(EventListener::KeyDown, move |event| {
         if let Event::KeyDown(key_event) = event {
             let keypress = keypress.get_untracked();
-            if keypress.key_down(key_event, &editor) {
+            let key_focus = key_focus
+                .as_ref()
+                .map(|k| k as &dyn KeyPressFocus)
+                .unwrap_or(&e_data);
+            if keypress.key_down(key_event, key_focus) {
                 EventPropagation::Stop
             } else {
                 EventPropagation::Continue
@@ -189,12 +280,11 @@ pub fn text_input(
             cursor: ime_cursor,
         } = event
         {
-            let doc = local_editor.view.doc.get_untracked();
             if text.is_empty() {
-                doc.clear_preedit();
+                ed1.clear_preedit();
             } else {
                 let offset = cursor.with_untracked(|c| c.offset());
-                doc.set_preedit(text.clone(), *ime_cursor, offset);
+                ed1.set_preedit(text.clone(), ime_cursor.to_owned(), offset);
             }
         }
         EventPropagation::Stop
@@ -205,9 +295,8 @@ pub fn text_input(
         }
 
         if let Event::ImeCommit(text) = event {
-            let doc = local_editor.view.doc.get_untracked();
-            doc.clear_preedit();
-            local_editor.receive_char(text);
+            ed2.clear_preedit();
+            ed2.receive_char(text.as_str());
         }
         EventPropagation::Stop
     })
@@ -229,10 +318,9 @@ pub struct TextInput {
     content: String,
     offset: usize,
     preedit_range: Option<(usize, usize)>,
-    doc: RwSignal<Rc<Document>>,
-    cursor: RwSignal<Cursor>,
+    editor: EditorData,
     focus: bool,
-    text_node: Option<Node>,
+    text_node: Option<NodeId>,
     text_layout: RwSignal<Option<TextLayout>>,
     text_rect: Rect,
     text_viewport: Rect,
@@ -244,7 +332,7 @@ pub struct TextInput {
     on_cursor_pos: Option<Box<dyn Fn(Point)>>,
     hide_cursor: RwSignal<bool>,
     config: ReadSignal<Arc<LapceConfig>>,
-    style: Extracter,
+    style: Extractor,
 }
 
 impl TextInput {
@@ -252,7 +340,7 @@ impl TextInput {
         let id = self.id;
         create_effect(move |_| {
             let placeholder = placeholder();
-            id.update_state(TextInputState::Placeholder(placeholder), false);
+            id.update_state(TextInputState::Placeholder(placeholder));
         });
         self
     }
@@ -262,6 +350,22 @@ impl TextInput {
         self
     }
 
+    pub fn editor(&self) -> EditorData {
+        self.editor.clone()
+    }
+
+    pub fn doc_signal(&self) -> DocSignal {
+        self.editor.doc_signal()
+    }
+
+    pub fn doc(&self) -> Rc<Doc> {
+        self.editor.doc()
+    }
+
+    pub fn cursor(&self) -> RwSignal<Cursor> {
+        self.editor.cursor()
+    }
+
     fn set_text_layout(&mut self) {
         let mut text_layout = TextLayout::new();
         let mut attrs =
@@ -425,6 +529,19 @@ impl View for TextInput {
         &mut self.data
     }
 
+    fn build(self) -> AnyWidget {
+        Box::new(self)
+    }
+}
+impl Widget for TextInput {
+    fn view_data(&self) -> &ViewData {
+        &self.data
+    }
+
+    fn view_data_mut(&mut self) -> &mut ViewData {
+        &mut self.data
+    }
+
     fn update(
         &mut self,
         cx: &mut floem::context::UpdateCx,
@@ -464,7 +581,7 @@ impl View for TextInput {
     fn layout(
         &mut self,
         cx: &mut floem::context::LayoutCx,
-    ) -> floem::taffy::prelude::Node {
+    ) -> floem::taffy::prelude::NodeId {
         cx.layout_node(self.id, true, |cx| {
             if self
                 .text_layout
@@ -478,7 +595,7 @@ impl View for TextInput {
             text_layout.with_untracked(|text_layout| {
                 let text_layout = text_layout.as_ref().unwrap();
 
-                let offset = self.cursor.get_untracked().offset();
+                let offset = self.cursor().get_untracked().offset();
                 let cursor_point = text_layout.hit_position(offset).point;
                 if cursor_point != self.cursor_pos {
                     self.cursor_pos = cursor_point;
@@ -531,7 +648,7 @@ impl View for TextInput {
         self.layout_rect = size
             .to_rect()
             .with_origin(Point::new(location.x as f64, location.y as f64));
-        let offset = self.cursor.with_untracked(|c| c.offset());
+        let offset = self.cursor().with_untracked(|c| c.offset());
         let cursor_line = self.text_layout.with_untracked(|text_layout| {
             let hit_position = text_layout.as_ref().unwrap().hit_position(offset);
             let point = Point::new(location.x as f64, location.y as f64)
@@ -565,21 +682,20 @@ impl View for TextInput {
         match event {
             Event::PointerDown(pointer) => {
                 let offset = self.hit_index(cx, pointer.pos);
-                self.cursor.update(|cursor| {
+                self.cursor().update(|cursor| {
                     cursor.set_insert(Selection::caret(offset));
                 });
                 if pointer.button.is_primary() && pointer.count == 2 {
                     let offset = self.hit_index(cx, pointer.pos);
                     let (start, end) = self
-                        .doc
-                        .get_untracked()
+                        .doc()
                         .buffer
                         .with_untracked(|buffer| buffer.select_word(offset));
-                    self.cursor.update(|cursor| {
+                    self.cursor().update(|cursor| {
                         cursor.set_insert(Selection::region(start, end));
                     });
                 } else if pointer.button.is_primary() && pointer.count == 3 {
-                    self.cursor.update(|cursor| {
+                    self.cursor().update(|cursor| {
                         cursor.set_insert(Selection::region(0, self.content.len()));
                     });
                 }
@@ -588,7 +704,7 @@ impl View for TextInput {
             Event::PointerMove(pointer) => {
                 if cx.is_active(self.id) {
                     let offset = self.hit_index(cx, pointer.pos);
-                    self.cursor.update(|cursor| {
+                    self.cursor().update(|cursor| {
                         cursor.set_offset(offset, true, false);
                     });
                 }
@@ -621,7 +737,7 @@ impl View for TextInput {
             let height = text_layout.size().height;
             let config = self.config.get_untracked();
 
-            let cursor = self.cursor.get_untracked();
+            let cursor = self.cursor().get_untracked();
 
             if let CursorMode::Insert(selection) = &cursor.mode {
                 for region in selection.regions() {
@@ -632,7 +748,7 @@ impl View for TextInput {
                             &Rect::ZERO
                                 .with_size(Size::new(max - min, height))
                                 .with_origin(Point::new(min + point.x, point.y)),
-                            *config.get_color(LapceColor::EDITOR_SELECTION),
+                            config.color(LapceColor::EDITOR_SELECTION),
                             0.0,
                         );
                     }
@@ -665,11 +781,7 @@ impl View for TextInput {
                         end_point.y + end_position.glyph_descent,
                     ),
                 );
-                cx.stroke(
-                    &line,
-                    *config.get_color(LapceColor::EDITOR_FOREGROUND),
-                    1.0,
-                );
+                cx.stroke(&line, config.color(LapceColor::EDITOR_FOREGROUND), 1.0);
             }
 
             if !self.hide_cursor.get_untracked()
@@ -695,10 +807,7 @@ impl View for TextInput {
 
                 cx.stroke(
                     &line,
-                    *self
-                        .config
-                        .get_untracked()
-                        .get_color(LapceColor::EDITOR_CARET),
+                    self.config.get_untracked().color(LapceColor::EDITOR_CARET),
                     2.0,
                 );
             }
