@@ -13,7 +13,7 @@ use lapce_core::meta;
 use lapce_rpc::proxy::ProxyStatus;
 
 use crate::{
-    app::{clickable_icon, window_menu},
+    app::{clickable_icon, not_clickable_icon, tooltip_label, window_menu},
     command::{LapceCommand, LapceWorkbenchCommand, WindowCommand},
     config::{color::LapceColor, icon::LapceIcons, LapceConfig},
     listener::Listener,
@@ -46,31 +46,41 @@ fn left(
             move |s| {
                 let config = config.get();
                 s.size(16.0, 16.0)
-                    .color(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE))
+                    .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
             },
         ))
         .style(move |s| s.margin_horiz(10.0).apply_if(is_macos, |s| s.hide())),
-        clickable_icon(|| LapceIcons::MENU, move || {}, || false, || false, config)
-            .popout_menu(move || window_menu(lapce_command, workbench_command))
-            .style(move |s| {
-                s.margin_left(4.0)
-                    .margin_right(6.0)
-                    .apply_if(is_macos, |s| s.hide())
-            }),
-        container(svg(move || config.get().ui_svg(LapceIcons::REMOTE)).style(
-            move |s| {
-                let config = config.get();
-                let size = (config.ui.icon_size() as f32 + 2.0).min(30.0);
-                s.size(size, size).color(if is_local {
-                    *config.get_color(LapceColor::LAPCE_ICON_ACTIVE)
-                } else {
-                    match proxy_status.get() {
-                        Some(_) => Color::WHITE,
-                        None => *config.get_color(LapceColor::LAPCE_ICON_ACTIVE),
-                    }
-                })
-            },
-        ))
+        not_clickable_icon(
+            || LapceIcons::MENU,
+            || false,
+            || false,
+            || "Menu",
+            config,
+        )
+        .popout_menu(move || window_menu(lapce_command, workbench_command))
+        .style(move |s| {
+            s.margin_left(4.0)
+                .margin_right(6.0)
+                .apply_if(is_macos, |s| s.hide())
+        }),
+        tooltip_label(
+            config,
+            container(svg(move || config.get().ui_svg(LapceIcons::REMOTE)).style(
+                move |s| {
+                    let config = config.get();
+                    let size = (config.ui.icon_size() as f32 + 2.0).min(30.0);
+                    s.size(size, size).color(if is_local {
+                        config.color(LapceColor::LAPCE_ICON_ACTIVE)
+                    } else {
+                        match proxy_status.get() {
+                            Some(_) => Color::WHITE,
+                            None => config.color(LapceColor::LAPCE_ICON_ACTIVE),
+                        }
+                    })
+                },
+            )),
+            || "Connect to Remote",
+        )
         .popout_menu(move || {
             #[allow(unused_mut)]
             let mut menu = Menu::new("").entry(
@@ -78,12 +88,26 @@ fn left(
                     workbench_command.send(LapceWorkbenchCommand::ConnectSshHost);
                 }),
             );
+            if !is_local
+                && proxy_status.get().is_some_and(|p| {
+                    matches!(p, ProxyStatus::Connecting | ProxyStatus::Connected)
+                })
+            {
+                menu = menu.entry(MenuItem::new("Disconnect remote").action(
+                    move || {
+                        workbench_command
+                            .send(LapceWorkbenchCommand::DisconnectRemote);
+                    },
+                ));
+            }
             #[cfg(windows)]
             {
-                menu =
-                    menu.entry(MenuItem::new("Connect to WSL").action(move || {
-                        workbench_command.send(LapceWorkbenchCommand::ConnectWsl);
-                    }));
+                menu = menu.entry(MenuItem::new("Connect to WSL Host").action(
+                    move || {
+                        workbench_command
+                            .send(LapceWorkbenchCommand::ConnectWslHost);
+                    },
+                ));
             }
             menu
         })
@@ -94,13 +118,13 @@ fn left(
             } else {
                 match proxy_status.get() {
                     Some(ProxyStatus::Connected) => {
-                        *config.get_color(LapceColor::LAPCE_REMOTE_CONNECTED)
+                        config.color(LapceColor::LAPCE_REMOTE_CONNECTED)
                     }
                     Some(ProxyStatus::Connecting) => {
-                        *config.get_color(LapceColor::LAPCE_REMOTE_CONNECTING)
+                        config.color(LapceColor::LAPCE_REMOTE_CONNECTING)
                     }
                     Some(ProxyStatus::Disconnected) => {
-                        *config.get_color(LapceColor::LAPCE_REMOTE_DISCONNECTED)
+                        config.color(LapceColor::LAPCE_REMOTE_DISCONNECTED)
                     }
                     None => Color::TRANSPARENT,
                 }
@@ -111,13 +135,12 @@ fn left(
                 .background(color)
                 .hover(|s| {
                     s.cursor(CursorStyle::Pointer).background(
-                        *config.get_color(LapceColor::PANEL_HOVERED_BACKGROUND),
+                        config.color(LapceColor::PANEL_HOVERED_BACKGROUND),
                     )
                 })
                 .active(|s| {
                     s.cursor(CursorStyle::Pointer).background(
-                        *config
-                            .get_color(LapceColor::PANEL_HOVERED_ACTIVE_BACKGROUND),
+                        config.color(LapceColor::PANEL_HOVERED_ACTIVE_BACKGROUND),
                     )
                 })
         }),
@@ -154,6 +177,7 @@ fn middle(
             },
             || false,
             move || !can_jump_backward.get(),
+            || "Jump Backward",
             config,
         )
         .style(move |s| s.margin_horiz(6.0))
@@ -166,17 +190,18 @@ fn middle(
             },
             || false,
             move || !can_jump_forward.get(),
+            || "Jump Forward",
             config,
         )
         .style(move |s| s.margin_right(6.0))
     };
 
     let open_folder = move || {
-        clickable_icon(
+        not_clickable_icon(
             || LapceIcons::PALETTE_MENU,
-            move || {},
             || false,
             || false,
+            || "Open Folder / Recent Workspace",
             config,
         )
         .popout_menu(move || {
@@ -209,7 +234,7 @@ fn middle(
                         let config = config.get();
                         let icon_size = config.ui.icon_size() as f32;
                         s.size(icon_size, icon_size)
-                            .color(*config.get_color(LapceColor::LAPCE_ICON_ACTIVE))
+                            .color(config.color(LapceColor::LAPCE_ICON_ACTIVE))
                     },
                 ),
                 label(move || {
@@ -242,9 +267,9 @@ fn middle(
                 .justify_content(Some(JustifyContent::Center))
                 .align_items(Some(AlignItems::Center))
                 .border(1.0)
-                .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
+                .border_color(config.color(LapceColor::LAPCE_BORDER))
                 .border_radius(6.0)
-                .background(*config.get_color(LapceColor::EDITOR_BACKGROUND))
+                .background(config.color(LapceColor::EDITOR_BACKGROUND))
         }),
         stack((
             clickable_icon(
@@ -254,6 +279,7 @@ fn middle(
                 },
                 || false,
                 || false,
+                || "Run and Debug",
                 config,
             )
             .style(move |s| s.margin_horiz(6.0)),
@@ -302,11 +328,11 @@ fn right(
         drag_window_area(empty())
             .style(|s| s.height_pct(100.0).flex_basis(0.0).flex_grow(1.0)),
         stack((
-            clickable_icon(
+            not_clickable_icon(
                 || LapceIcons::SETTINGS,
-                || {},
                 || false,
                 || false,
+                || "Settings",
                 config,
             )
             .popout_menu(move || {
@@ -354,11 +380,11 @@ fn right(
             container(label(|| "1".to_string()).style(move |s| {
                 let config = config.get();
                 s.font_size(10.0)
-                    .color(*config.get_color(LapceColor::EDITOR_BACKGROUND))
+                    .color(config.color(LapceColor::EDITOR_BACKGROUND))
                     .border_radius(100.0)
                     .margin_left(5.0)
                     .margin_top(10.0)
-                    .background(*config.get_color(LapceColor::EDITOR_CARET))
+                    .background(config.color(LapceColor::EDITOR_CARET))
             }))
             .style(move |s| {
                 let has_update = has_update();
@@ -433,9 +459,9 @@ pub fn title(window_tab_data: Rc<WindowTabData>) -> impl View {
         s.width_pct(100.0)
             .height(37.0)
             .items_center()
-            .background(*config.get_color(LapceColor::PANEL_BACKGROUND))
+            .background(config.color(LapceColor::PANEL_BACKGROUND))
             .border_bottom(1.0)
-            .border_color(*config.get_color(LapceColor::LAPCE_BORDER))
+            .border_color(config.color(LapceColor::LAPCE_BORDER))
     })
 }
 
@@ -454,6 +480,7 @@ pub fn window_controls_view(
             },
             || false,
             || false,
+            || "Minimize",
             config,
         )
         .style(|s| s.margin_right(16.0).margin_left(10.0)),
@@ -472,6 +499,7 @@ pub fn window_controls_view(
             },
             || false,
             || false,
+            || "Maximize",
             config,
         )
         .style(|s| s.margin_right(16.0)),
@@ -482,6 +510,7 @@ pub fn window_controls_view(
             },
             || false,
             || false,
+            || "Close Window",
             config,
         )
         .style(|s| s.margin_right(6.0)),
