@@ -2,28 +2,34 @@ use floem::{
     id::Id,
     peniko::kurbo::{BezPath, Point, Size},
     style::TextColor,
-    view::{View, ViewData},
+    view::{AnyWidget, View, ViewData, Widget},
     Renderer,
 };
 
 pub fn wave_box() -> WaveBox {
-    let id = Id::next();
     WaveBox {
-        id,
-        data: ViewData::new(id),
+        data: ViewData::new(Id::next()),
     }
 }
 
 pub struct WaveBox {
-    id: Id,
     data: ViewData,
 }
 
 impl View for WaveBox {
-    fn id(&self) -> Id {
-        self.id
+    fn view_data(&self) -> &ViewData {
+        &self.data
     }
 
+    fn view_data_mut(&mut self) -> &mut ViewData {
+        &mut self.data
+    }
+
+    fn build(self) -> AnyWidget {
+        Box::new(self)
+    }
+}
+impl Widget for WaveBox {
     fn view_data(&self) -> &ViewData {
         &self.data
     }
@@ -33,8 +39,8 @@ impl View for WaveBox {
     }
 
     fn paint(&mut self, cx: &mut floem::context::PaintCx) {
-        if let Some(color) = cx.get_computed_style(self.id).get(TextColor) {
-            let layout = cx.get_layout(self.id).unwrap();
+        if let Some(color) = cx.get_computed_style(self.data.id()).get(TextColor) {
+            let layout = cx.get_layout(self.data.id()).unwrap();
             let size = layout.size;
             let size = Size::new(size.width as f64, size.height as f64);
             let radius = 4.0;
