@@ -1,4 +1,4 @@
-use std::{rc::Rc, sync::Arc};
+use std::{path::PathBuf, rc::Rc, sync::Arc};
 
 use floem::{
     action::TimerToken,
@@ -47,6 +47,8 @@ pub struct WindowCommonData {
     pub cursor_blink_timer: RwSignal<TimerToken>,
     // the value to be update by curosr blinking
     pub hide_cursor: RwSignal<bool>,
+    pub app_view_id: RwSignal<floem::id::Id>,
+    pub extra_plugin_paths: Arc<Vec<PathBuf>>,
 }
 
 /// `WindowData` is the application model for a top-level window.
@@ -80,13 +82,16 @@ pub struct WindowData {
 impl WindowData {
     pub fn new(
         window_id: WindowId,
+        app_view_id: RwSignal<floem::id::Id>,
         info: WindowInfo,
         window_scale: RwSignal<f64>,
         latest_release: ReadSignal<Arc<Option<ReleaseInfo>>>,
+        extra_plugin_paths: Arc<Vec<PathBuf>>,
         app_command: Listener<AppCommand>,
     ) -> Self {
         let cx = Scope::new();
-        let config = LapceConfig::load(&LapceWorkspace::default(), &[]);
+        let config =
+            LapceConfig::load(&LapceWorkspace::default(), &[], &extra_plugin_paths);
         let config = cx.create_rw_signal(Arc::new(config));
         let root_view_id = cx.create_rw_signal(floem::id::Id::next());
 
@@ -113,6 +118,8 @@ impl WindowData {
             ime_allowed,
             cursor_blink_timer,
             hide_cursor,
+            app_view_id,
+            extra_plugin_paths,
         });
 
         for w in info.tabs.workspaces {
@@ -176,7 +183,11 @@ impl WindowData {
     }
 
     pub fn reload_config(&self) {
-        let config = LapceConfig::load(&LapceWorkspace::default(), &[]);
+        let config = LapceConfig::load(
+            &LapceWorkspace::default(),
+            &[],
+            &self.common.extra_plugin_paths,
+        );
         self.config.set(Arc::new(config));
         let window_tabs = self.window_tabs.get_untracked();
         for (_, window_tab) in window_tabs {
