@@ -1,10 +1,14 @@
-use std::{collections::HashSet, path::Path, str::FromStr};
+use std::{
+    collections::{hash_map::Entry, HashMap, HashSet},
+    path::Path,
+    str::FromStr,
+};
 
-use anyhow::Result;
+use lapce_rpc::style::{LineStyle, Style};
 use once_cell::sync::Lazy;
-use strum_macros::{AsRefStr, AsStaticStr, Display, EnumMessage, EnumString};
+use strum_macros::{AsRefStr, Display, EnumMessage, EnumString, IntoStaticStr};
 use tracing::{debug, error};
-use tree_sitter::TreeCursor;
+use tree_sitter::{Point, TreeCursor};
 
 use crate::{
     directory::Directory,
@@ -95,7 +99,7 @@ struct TreeSitterProperties {
     query: Option<&'static str>,
     /// Lists of tree-sitter node types that control how code lenses are built.
     /// The first is a list of nodes that should be traversed and included in
-    /// the lens, along with thier children. The second is a list of nodes that
+    /// the lens, along with their children. The second is a list of nodes that
     /// should be excluded from the lens, though they will still be traversed.
     /// See `walk_tree` for more details.
     ///
@@ -140,7 +144,7 @@ struct CommentProperties {
     Debug,
     Display,
     AsRefStr,
-    AsStaticStr,
+    IntoStaticStr,
     EnumString,
     EnumMessage,
     Default,
@@ -354,7 +358,7 @@ const LANGUAGES: &[SyntaxProperties] = &[
         id: LapceLanguage::Cmake,
 
         indent: "  ",
-        files: &[],
+        files: &["cmakelists"],
         extensions: &["cmake"],
 
         comment: comment_properties!("#"),
@@ -1555,10 +1559,10 @@ impl LapceLanguage {
     pub fn languages() -> Vec<&'static str> {
         let mut langs = vec![];
         for l in LANGUAGES {
-            langs.push(
-                strum::EnumMessage::get_message(&l.id)
-                    .unwrap_or(strum::AsStaticRef::as_static(&l.id)),
-            )
+            // Get only languages with display name to hide inline grammars
+            if let Some(lang) = strum::EnumMessage::get_message(&l.id) {
+                langs.push(lang)
+            }
         }
         langs
     }
@@ -1579,8 +1583,7 @@ impl LapceLanguage {
     }
 
     pub fn name(&self) -> &'static str {
-        strum::EnumMessage::get_message(self)
-            .unwrap_or(strum::AsStaticRef::as_static(self))
+        strum::EnumMessage::get_message(self).unwrap_or(self.into())
     }
 
     fn tree_sitter(&self) -> Option<TreeSitterProperties> {
@@ -1595,7 +1598,7 @@ impl LapceLanguage {
         }
     }
 
-    pub fn comment_token(&self) -> &str {
+    pub fn comment_token(&self) -> &'static str {
         self.properties()
             .comment
             .single_line_start
@@ -1781,9 +1784,12 @@ fn load_grammar(
         > = match library.get(language_fn_name.as_bytes()) {
             Ok(v) => v,
             Err(e) => {
+                if let Some(e) = library.close().err() {
+                    error!("Failed to drop loaded library: {e}");
+                };
                 return Err(HighlightIssue::Error(format!(
                     "Failed to load '{language_fn_name}': '{e}'"
-                )))
+                )));
             }
         };
         language_fn()
@@ -1822,3 +1828,72 @@ fn walk_tree(
         cursor.goto_parent();
     }
 }
+
+fn add_bracket_pos(
+    bracket_pos: &mut HashMap<usize, Vec<LineStyle>>,
+    start_pos: Point,
+    color: String,
+) {
+    let line_style = LineStyle {
+        start: start_pos.column,
+        end: start_pos.column + 1,
+        style: Style {
+            fg_color: Some(color),
+        },
+    };
+    match bracket_pos.entry(start_pos.row) {
+        Entry::Vacant(v) => _ = v.insert(vec![line_style]),
+        Entry::Occupied(mut o) => o.get_mut().push(line_style),
+    }
+}
+
+pub(crate) fn walk_tree_bracket_ast(
+    cursor: &mut TreeCursor,
+    level: &mut usize,
+    counter: &mut usize,
+    bracket_pos: &mut HashMap<usize, Vec<LineStyle>>,
+    palette: &Vec<String>,
+) {
+    if cursor.node().kind().ends_with('(')
+        || cursor.node().kind().ends_with('{')
+        || cursor.node().kind().ends_with('[')
+    {
+        let row = cursor.node().end_position().row;
+        let col = cursor.node().end_position().column - 1;
+        let start_pos = Point::new(row, col);
+        add_bracket_pos(
+            bracket_pos,
+            start_pos,
+            palette.get(*level % palette.len()).unwrap().clone(),
+        );
+        *level += 1;
+    } else if cursor.node().kind().ends_with(')')
+        || cursor.node().kind().ends_with('}')
+        || cursor.node().kind().ends_with(']')
+    {
+        let (new_level, overflow) = (*level).overflowing_sub(1);
+        let row = cursor.node().end_position().row;
+        let col = cursor.node().end_position().column - 1;
+        let start_pos = Point::new(row, col);
+        if overflow {
+            add_bracket_pos(bracket_pos, start_pos, "bracket.unpaired".to_string());
+        } else {
+            *level = new_level;
+            add_bracket_pos(
+                bracket_pos,
+                start_pos,
+                palette.get(*level % palette.len()).unwrap().clone(),
+            );
+        }
+    }
+    *counter += 1;
+    if cursor.goto_first_child() {
+        loop {
+            walk_tree_bracket_ast(cursor, level, counter, bracket_pos, palette);
+            if !cursor.goto_next_sibling() {
+                break;
+            }
+        }
+        cursor.goto_parent();
+    }
+}
