@@ -8,14 +8,15 @@
 
 use std::{
     cell::RefCell,
-    collections::{HashSet, VecDeque},
+    collections::{hash_map::Entry, HashMap, HashSet, VecDeque},
     mem,
     path::Path,
     sync::{atomic::AtomicUsize, Arc},
 };
 
+use floem_editor_core::util::{matching_bracket_general, matching_pair_direction};
 use itertools::Itertools;
-use lapce_rpc::style::Style;
+use lapce_rpc::style::{LineStyle, Style};
 use lapce_xi_rope::{
     spans::{Spans, SpansBuilder},
     Interval, Rope,
@@ -31,14 +32,16 @@ use self::{
         HighlightConfiguration, HighlightEvent, HighlightIter, HighlightIterLayer,
         IncludedChildren, LocalScope,
     },
-    util::{matching_bracket_general, matching_pair_direction, RopeProvider},
+    util::RopeProvider,
 };
 use crate::{
-    language::LapceLanguage,
+    buffer::rope_text::RopeText,
+    language::{self, LapceLanguage},
     lens::{Lens, LensBuilder},
     style::SCOPES,
 };
 
+use crate::buffer::Buffer;
 pub mod edit;
 pub mod highlight;
 pub mod util;
@@ -68,6 +71,309 @@ pub enum Error {
     Unknown,
 }
 
+#[derive(Clone, Debug)]
+pub enum NodeType {
+    LeftParen,
+    RightParen,
+    LeftBracket,
+    RightBracket,
+    LeftCurly,
+    RightCurly,
+    Pair,
+    Code,
+    Dummy,
+}
+
+#[derive(Clone, Debug)]
+pub enum BracketParserMode {
+    Parsing,
+    NoParsing,
+}
+
+#[derive(Clone, Debug)]
+pub struct ASTNode {
+    pub tt: NodeType,
+    pub len: usize,
+    pub children: Vec<ASTNode>,
+    pub level: usize,
+}
+
+impl ASTNode {
+    pub fn new() -> Self {
+        Self {
+            tt: NodeType::Dummy,
+            len: 0,
+            children: vec![],
+            level: 0,
+        }
+    }
+
+    pub fn new_with_type(tt: NodeType, len: usize) -> Self {
+        Self {
+            tt,
+            len,
+            children: vec![],
+            level: 0,
+        }
+    }
+}
+
+impl Default for ASTNode {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+#[derive(Clone, Debug)]
+pub struct BracketParser {
+    pub code: Vec<char>,
+    pub cur: usize,
+    pub ast: ASTNode,
+    bracket_set: HashMap<char, ASTNode>,
+    pub bracket_pos: HashMap<usize, Vec<LineStyle>>,
+    mode: BracketParserMode,
+    noparsing_token: Vec<char>,
+    pub active: bool,
+    pub limit: u64,
+}
+
+impl BracketParser {
+    pub fn new(code: String, active: bool, limit: u64) -> Self {
+        Self {
+            code: code.chars().collect(),
+            cur: 0,
+            ast: ASTNode::new(),
+            bracket_set: HashMap::from([
+                ('(', ASTNode::new_with_type(NodeType::LeftParen, 1)),
+                (')', ASTNode::new_with_type(NodeType::RightParen, 1)),
+                ('{', ASTNode::new_with_type(NodeType::LeftCurly, 1)),
+                ('}', ASTNode::new_with_type(NodeType::RightCurly, 1)),
+                ('[', ASTNode::new_with_type(NodeType::LeftBracket, 1)),
+                (']', ASTNode::new_with_type(NodeType::RightBracket, 1)),
+            ]),
+            bracket_pos: HashMap::new(),
+            mode: BracketParserMode::Parsing,
+            noparsing_token: vec!['\'', '"', '`'],
+            active,
+            limit,
+        }
+    }
+
+    /*pub fn enable(&self) {
+        *(self.active.borrow_mut()) = true;
+    }
+
+    pub fn disable(&self) {
+        *(self.active.borrow_mut()) = false;
+    }*/
+
+    pub fn update_code(
+        &mut self,
+        code: String,
+        buffer: &Buffer,
+        syntax: Option<&Syntax>,
+    ) {
+        let palette = vec![
+            "bracket.color.1".to_string(),
+            "bracket.color.2".to_string(),
+            "bracket.color.3".to_string(),
+        ];
+        if self.active
+            && code
+                .chars()
+                .fold(0, |i, c| if c == '\n' { i + 1 } else { i })
+                < self.limit as usize
+        {
+            self.bracket_pos = HashMap::new();
+            if let Some(syntax) = syntax {
+                if let Some(layers) = &syntax.layers {
+                    if let Some(tree) = layers.try_tree() {
+                        let mut walk_cursor = tree.walk();
+                        let mut bracket_pos: HashMap<usize, Vec<LineStyle>> =
+                            HashMap::new();
+                        language::walk_tree_bracket_ast(
+                            &mut walk_cursor,
+                            &mut 0,
+                            &mut 0,
+                            &mut bracket_pos,
+                            &palette,
+                        );
+                        self.bracket_pos = bracket_pos;
+                    }
+                }
+            } else {
+                self.code = code.chars().collect();
+                self.cur = 0;
+                self.parse();
+                let mut pos_vec = vec![];
+                Self::highlight_pos(
+                    &self.ast,
+                    &mut pos_vec,
+                    &mut 0usize,
+                    &mut 0usize,
+                    &palette,
+                );
+                if buffer.is_empty() {
+                    return;
+                }
+                for (offset, color) in pos_vec.iter() {
+                    let (line, col) = buffer.offset_to_line_col(*offset);
+                    let line_style = LineStyle {
+                        start: col,
+                        end: col + 1,
+                        style: Style {
+                            fg_color: Some(color.clone()),
+                        },
+                    };
+                    match self.bracket_pos.entry(line) {
+                        Entry::Vacant(v) => _ = v.insert(vec![line_style.clone()]),
+                        Entry::Occupied(mut o) => {
+                            o.get_mut().push(line_style.clone())
+                        }
+                    }
+                }
+            }
+        } else {
+            self.bracket_pos = HashMap::new();
+        }
+    }
+
+    fn is_left(c: &char) -> bool {
+        if *c == '(' || *c == '{' || *c == '[' {
+            return true;
+        }
+        false
+    }
+
+    fn parse(&mut self) {
+        let new_ast = &mut ASTNode::new();
+        self.parse_bracket(0, new_ast);
+        self.ast = new_ast.clone();
+        Self::patch_len(&mut self.ast);
+        self.cur = 0;
+    }
+
+    fn parse_bracket(&mut self, level: usize, parent_node: &mut ASTNode) {
+        let mut counter = 0usize;
+        while self.cur < self.code.len() {
+            if self.noparsing_token.contains(&self.code[self.cur]) {
+                if matches!(self.mode, BracketParserMode::Parsing) {
+                    self.mode = BracketParserMode::NoParsing;
+                } else {
+                    self.mode = BracketParserMode::Parsing;
+                }
+            }
+            if self.bracket_set.contains_key(&self.code[self.cur])
+                && matches!(self.mode, BracketParserMode::Parsing)
+            {
+                if Self::is_left(&self.code[self.cur]) {
+                    let code_node = ASTNode::new_with_type(NodeType::Code, counter);
+                    let left_node =
+                        self.bracket_set.get(&self.code[self.cur]).unwrap().clone();
+                    let mut pair_node =
+                        ASTNode::new_with_type(NodeType::Pair, counter + 1);
+                    pair_node.level = level;
+                    pair_node.children.push(code_node);
+                    pair_node.children.push(left_node);
+                    self.cur += 1;
+                    self.parse_bracket(level + 1, &mut pair_node);
+                    parent_node.children.push(pair_node.clone());
+                    counter = 0;
+                } else if level <= parent_node.level {
+                    let code_node = ASTNode::new_with_type(NodeType::Code, counter);
+                    let right_node =
+                        self.bracket_set.get(&self.code[self.cur]).unwrap().clone();
+                    parent_node.children.push(code_node);
+                    parent_node.children.push(right_node);
+                    let parent_len = parent_node.len;
+                    parent_node.len = parent_len + counter + 1;
+                    counter = 0;
+                    self.cur += 1;
+                } else {
+                    let code_node = ASTNode::new_with_type(NodeType::Code, counter);
+                    let right_node =
+                        self.bracket_set.get(&self.code[self.cur]).unwrap().clone();
+                    parent_node.children.push(code_node);
+                    parent_node.children.push(right_node);
+                    let parent_len = parent_node.len;
+                    parent_node.len = parent_len + counter + 1;
+                    self.cur += 1;
+                    return;
+                }
+            } else {
+                counter += self.code[self.cur].len_utf8();
+                self.cur += 1;
+            }
+        }
+    }
+
+    fn patch_len(ast: &mut ASTNode) {
+        if !ast.children.is_empty() {
+            let mut len = 0usize;
+            for n in ast.children.iter_mut() {
+                match n.tt {
+                    NodeType::LeftCurly => len += 1,
+                    NodeType::RightCurly => len += 1,
+                    NodeType::LeftParen => len += 1,
+                    NodeType::RightParen => len += 1,
+                    NodeType::LeftBracket => len += 1,
+                    NodeType::RightBracket => len += 1,
+                    NodeType::Code => len += n.len,
+                    NodeType::Pair => {
+                        Self::patch_len(n);
+                        len += n.len;
+                    }
+                    _ => break,
+                }
+            }
+            ast.len = len;
+        }
+    }
+
+    fn highlight_pos(
+        ast: &ASTNode,
+        pos_vec: &mut Vec<(usize, String)>,
+        index: &mut usize,
+        level: &mut usize,
+        palette: &Vec<String>,
+    ) {
+        if !ast.children.is_empty() {
+            for n in ast.children.iter() {
+                match n.tt {
+                    NodeType::LeftCurly
+                    | NodeType::LeftParen
+                    | NodeType::LeftBracket => {
+                        pos_vec
+                            .push((*index, palette[*level % palette.len()].clone()));
+                        *level += 1;
+                        *index += 1;
+                    }
+                    NodeType::RightCurly
+                    | NodeType::RightParen
+                    | NodeType::RightBracket => {
+                        let (new_level, overflow) = (*level).overflowing_sub(1);
+                        if overflow {
+                            pos_vec.push((*index, "bracket.unpaired".to_string()));
+                        } else {
+                            *level = new_level;
+                            pos_vec.push((
+                                *index,
+                                palette[*level % palette.len()].clone(),
+                            ));
+                        }
+                        *index += 1;
+                    }
+                    NodeType::Code => *index += n.len,
+                    NodeType::Pair => {
+                        Self::highlight_pos(n, pos_vec, index, level, palette)
+                    }
+                    _ => break,
+                }
+            }
+        }
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct LanguageLayer {
     // mode
@@ -80,10 +386,6 @@ pub struct LanguageLayer {
 }
 
 impl LanguageLayer {
-    pub fn tree(&self) -> &Tree {
-        self.tree.as_ref().unwrap()
-    }
-
     pub fn try_tree(&self) -> Option<&Tree> {
         self.tree.as_ref()
     }
@@ -93,6 +395,7 @@ impl LanguageLayer {
         parser: &mut Parser,
         source: &Rope,
         had_edits: bool,
+        cancellation_flag: &AtomicUsize,
     ) -> Result<(), Error> {
         parser.set_included_ranges(&self.ranges).unwrap();
 
@@ -100,7 +403,7 @@ impl LanguageLayer {
             .set_language(self.config.language)
             .map_err(|_| Error::InvalidLanguage)?;
 
-        // unsafe { syntax.parser.set_cancellation_flag(cancellation_flag) };
+        unsafe { parser.set_cancellation_flag(Some(cancellation_flag)) };
         let tree = parser
             .parse_with(
                 &mut |byte, _| {
@@ -117,7 +420,6 @@ impl LanguageLayer {
                 had_edits.then_some(()).and(self.tree.as_ref()),
             )
             .ok_or(Error::Cancelled)?;
-        // unsafe { ts_parser.parser.set_cancellation_flag(None) };
         self.tree = Some(tree);
         Ok(())
     }
@@ -155,8 +457,9 @@ impl SyntaxLayers {
 
         let mut syntax = SyntaxLayers { root, layers };
 
+        let cancel_flag = AtomicUsize::new(0);
         if let Some(source) = source {
-            let _ = syntax.update(0, 0, source, None);
+            let _ = syntax.update(0, 0, source, None, &cancel_flag);
         }
 
         syntax
@@ -168,6 +471,7 @@ impl SyntaxLayers {
         new_rev: u64,
         source: &Rope,
         syntax_edits: Option<&[SyntaxEdit]>,
+        cancellation_flag: &AtomicUsize,
     ) -> Result<(), Error> {
         let mut queue = VecDeque::new();
         queue.push_back(self.root);
@@ -300,116 +604,118 @@ impl SyntaxLayers {
                 }
 
                 // Re-parse the tree.
-                layer.parse(&mut ts_parser.parser, source, had_edits)?;
+                layer.parse(&mut ts_parser.parser, source, had_edits, cancellation_flag)?;
                 layer.rev = new_rev;
 
                 // Switch to an immutable borrow.
                 let layer = &self.layers[layer_id];
 
                 // Process injections.
-                let matches = cursor.matches(
-                    &layer.config.injections_query,
-                    layer.tree().root_node(),
-                    RopeProvider(source),
-                );
-                let mut injections = Vec::new();
-                for mat in matches {
-                    let (language_name, content_node, included_children) = injection_for_match(
-                        &layer.config,
-                        &layer.config.injections_query,
-                        &mat,
-                        source,
-                    );
-
-                    // Explicitly remove this match so that none of its other captures will remain
-                    // in the stream of captures.
-                    mat.remove();
-
-                    // If a language is found with the given name, then add a new language layer
-                    // to the highlighted document.
-                    if let (Some(language_name), Some(content_node)) = (language_name, content_node)
-                    {
-                        if let Ok(config) = (injection_callback)(&language_name) {
-                            let ranges =
-                                intersect_ranges(&layer.ranges, &[content_node], included_children);
-
-                            if !ranges.is_empty() {
-                                injections.push((config, ranges));
-                            }
-                        }
-                    }
-                }
-
-                // Process combined injections.
-                if let Some(combined_injections_query) = &layer.config.combined_injections_query {
-                    let mut injections_by_pattern_index =
-                        vec![
-                            (None, Vec::new(), IncludedChildren::default());
-                            combined_injections_query.pattern_count()
-                        ];
+                if let Some(tree) = layer.try_tree() {
                     let matches = cursor.matches(
-                        combined_injections_query,
-                        layer.tree().root_node(),
+                        &layer.config.injections_query,
+                        tree.root_node(),
                         RopeProvider(source),
                     );
+                    let mut injections = Vec::new();
                     for mat in matches {
-                        let entry = &mut injections_by_pattern_index[mat.pattern_index];
                         let (language_name, content_node, included_children) = injection_for_match(
                             &layer.config,
-                            combined_injections_query,
+                            &layer.config.injections_query,
                             &mat,
                             source,
                         );
-                        if language_name.is_some() {
-                            entry.0 = language_name;
-                        }
-                        if let Some(content_node) = content_node {
-                            entry.1.push(content_node);
-                        }
-                        entry.2 = included_children;
-                    }
-                    for (lang_name, content_nodes, included_children) in injections_by_pattern_index
-                    {
-                        if let (Some(lang_name), false) = (lang_name, content_nodes.is_empty()) {
-                            if let Ok(config) = (injection_callback)(&lang_name) {
-                                let ranges = intersect_ranges(
-                                    &layer.ranges,
-                                    &content_nodes,
-                                    included_children,
-                                );
+
+                        // Explicitly remove this match so that none of its other captures will remain
+                        // in the stream of captures.
+                        mat.remove();
+
+                        // If a language is found with the given name, then add a new language layer
+                        // to the highlighted document.
+                        if let (Some(language_name), Some(content_node)) = (language_name, content_node)
+                        {
+                            if let Ok(config) = (injection_callback)(&language_name) {
+                                let ranges =
+                                    intersect_ranges(&layer.ranges, &[content_node], included_children);
+
                                 if !ranges.is_empty() {
                                     injections.push((config, ranges));
                                 }
                             }
                         }
                     }
-                }
 
-                let depth = layer.depth + 1;
-                // TODO: can't inline this since matches borrows self.layers
-                for (config, ranges) in injections {
-                    // Find an existing layer
-                    let layer = self
-                        .layers
-                        .iter_mut()
-                        .find(|(_, layer)| {
-                            layer.depth == depth && // TODO: track parent id instead
-                            layer.config.language == config.language && layer.ranges == ranges
-                        })
-                        .map(|(id, _layer)| id);
+                    // Process combined injections.
+                    if let Some(combined_injections_query) = &layer.config.combined_injections_query {
+                        let mut injections_by_pattern_index =
+                            vec![
+                                (None, Vec::new(), IncludedChildren::default());
+                                combined_injections_query.pattern_count()
+                            ];
+                        let matches = cursor.matches(
+                            combined_injections_query,
+                            tree.root_node(),
+                            RopeProvider(source),
+                        );
+                        for mat in matches {
+                            let entry = &mut injections_by_pattern_index[mat.pattern_index];
+                            let (language_name, content_node, included_children) = injection_for_match(
+                                &layer.config,
+                                combined_injections_query,
+                                &mat,
+                                source,
+                            );
+                            if language_name.is_some() {
+                                entry.0 = language_name;
+                            }
+                            if let Some(content_node) = content_node {
+                                entry.1.push(content_node);
+                            }
+                            entry.2 = included_children;
+                        }
+                        for (lang_name, content_nodes, included_children) in injections_by_pattern_index
+                        {
+                            if let (Some(lang_name), false) = (lang_name, content_nodes.is_empty()) {
+                                if let Ok(config) = (injection_callback)(&lang_name) {
+                                    let ranges = intersect_ranges(
+                                        &layer.ranges,
+                                        &content_nodes,
+                                        included_children,
+                                    );
+                                    if !ranges.is_empty() {
+                                        injections.push((config, ranges));
+                                    }
+                                }
+                            }
+                        }
+                    }
 
-                    // ...or insert a new one.
-                    let layer_id = layer.unwrap_or_else(|| {
-                        self.layers.insert(LanguageLayer {
-                            tree: None,
-                            config,
-                            depth,
-                            ranges,
-                            rev: 0,
-                        })
-                    });
+                    let depth = layer.depth + 1;
+                    // TODO: can't inline this since matches borrows self.layers
+                    for (config, ranges) in injections {
+                        // Find an existing layer
+                        let layer = self
+                            .layers
+                            .iter_mut()
+                            .find(|(_, layer)| {
+                                layer.depth == depth && // TODO: track parent id instead
+                                layer.config.language == config.language && layer.ranges == ranges
+                            })
+                            .map(|(id, _layer)| id);
 
-                    queue.push_back(layer_id);
+                        // ...or insert a new one.
+                        let layer_id = layer.unwrap_or_else(|| {
+                            self.layers.insert(LanguageLayer {
+                                tree: None,
+                                config,
+                                depth,
+                                ranges,
+                                rev: 0,
+                            })
+                        });
+
+                        queue.push_back(layer_id);
+                    }
                 }
 
                 // TODO: pre-process local scopes at this time, rather than highlight?
@@ -426,10 +732,6 @@ impl SyntaxLayers {
         })
     }
 
-    pub fn tree(&self) -> &Tree {
-        self.layers[self.root].tree()
-    }
-
     pub fn try_tree(&self) -> Option<&Tree> {
         self.layers[self.root].try_tree()
     }
@@ -466,7 +768,7 @@ impl SyntaxLayers {
                 let mut captures = cursor_ref
                     .captures(
                         &layer.config.query,
-                        layer.tree().root_node(),
+                        layer.try_tree()?.root_node(),
                         RopeProvider(source),
                     )
                     .peekable();
@@ -535,6 +837,7 @@ pub struct Syntax {
     pub line_height: usize,
     pub lens_height: usize,
     pub styles: Option<Spans<Style>>,
+    pub cancel_flag: Arc<AtomicUsize>,
 }
 
 impl std::fmt::Debug for Syntax {
@@ -573,6 +876,7 @@ impl Syntax {
             lens_height: 0,
             normal_lines: Vec::new(),
             styles: None,
+            cancel_flag: Arc::new(AtomicUsize::new(0)),
         }
     }
 
@@ -587,7 +891,8 @@ impl Syntax {
             None => return,
         };
         let edits = edits.filter(|edits| new_rev == self.rev + edits.len() as u64);
-        let _ = layers.update(self.rev, new_rev, &new_text, edits);
+        let _ =
+            layers.update(self.rev, new_rev, &new_text, edits, &self.cancel_flag);
         let tree = layers.try_tree();
 
         let styles = if tree.is_some() {
@@ -597,7 +902,11 @@ impl Syntax {
 
             // TODO: Should we be ignoring highlight errors via flattening them?
             for highlight in layers
-                .highlight_iter(&new_text, Some(0..new_text.len()), None)
+                .highlight_iter(
+                    &new_text,
+                    Some(0..new_text.len()),
+                    Some(&self.cancel_flag),
+                )
                 .flatten()
             {
                 match highlight {
