@@ -9,6 +9,7 @@ use std::{
     thread,
 };
 
+use lapce_rpc::plugin::VoltInfo;
 use lapce_rpc::{
     dap_types::{self, DapId, DapServer, SetBreakpointsResponse},
     plugin::{PluginId, VoltID, VoltMetadata},
@@ -17,6 +18,7 @@ use lapce_rpc::{
     RpcError,
 };
 use lapce_xi_rope::{Rope, RopeDelta};
+use lsp_types::request::Request;
 use lsp_types::{
     notification::DidOpenTextDocument, DidOpenTextDocumentParams, SemanticTokens,
     TextDocumentIdentifier, TextDocumentItem, VersionedTextDocumentIdentifier,
@@ -50,6 +52,7 @@ impl PluginCatalog {
     pub fn new(
         workspace: Option<PathBuf>,
         disabled_volts: Vec<VoltID>,
+        extra_plugin_paths: Vec<PathBuf>,
         plugin_configurations: HashMap<String, HashMap<String, serde_json::Value>>,
         plugin_rpc: PluginCatalogRpcHandler,
     ) -> Self {
@@ -65,7 +68,7 @@ impl PluginCatalog {
         };
 
         thread::spawn(move || {
-            load_all_volts(plugin_rpc, disabled_volts);
+            load_all_volts(plugin_rpc, &extra_plugin_paths, disabled_volts);
         });
 
         plugin
@@ -174,6 +177,31 @@ impl PluginCatalog {
         }
     }
 
+    pub fn shutdown_volt(
+        &mut self,
+        volt: VoltInfo,
+        f: Box<dyn ClonableCallback<Value, RpcError>>,
+    ) {
+        let id = volt.id();
+        for (plugin_id, plugin) in self.plugins.iter() {
+            if plugin.volt_id == id {
+                let f = dyn_clone::clone_box(&*f);
+                let plugin_id = *plugin_id;
+                plugin.server_request_async(
+                    lsp_types::request::Shutdown::METHOD,
+                    Value::Null,
+                    None,
+                    None,
+                    false,
+                    move |result| {
+                        f(plugin_id, result);
+                    },
+                );
+                plugin.shutdown();
+            }
+        }
+    }
+
     fn start_unactivated_volts(&mut self, to_be_activated: Vec<VoltID>) {
         for id in to_be_activated.iter() {
             let workspace = self.workspace.clone();
@@ -376,7 +404,7 @@ impl PluginCatalog {
                     match result {
                         Ok(resp) => {
                             let scopes = resp.scopes.clone();
-                            if let Some(scope) = resp.scopes.get(0) {
+                            if let Some(scope) = resp.scopes.first() {
                                 let scope = scope.to_owned();
                                 thread::spawn(move || {
                                     local_dap.variables_async(
@@ -483,7 +511,7 @@ impl PluginCatalog {
                 let configurations =
                     self.plugin_configurations.get(&volt.name).cloned();
                 let catalog_rpc = self.plugin_rpc.clone();
-                let _ = catalog_rpc.stop_volt(volt.clone());
+                catalog_rpc.stop_volt(volt.clone());
                 thread::spawn(move || {
                     let _ =
                         install_volt(catalog_rpc, workspace, configurations, volt);
