@@ -12,11 +12,9 @@ use std::{
 use anyhow::{anyhow, Result};
 use crossbeam_channel::{Receiver, Sender};
 use dyn_clone::DynClone;
+use floem_editor_core::buffer::rope_text::{RopeText, RopeTextRef};
 use jsonrpc_lite::{Id, JsonRpc, Params};
-use lapce_core::{
-    buffer::rope_text::{RopeText, RopeTextRef},
-    encoding::offset_utf16_to_utf8,
-};
+use lapce_core::{encoding::offset_utf16_to_utf8, rope_text_pos::RopeTextPosition};
 use lapce_rpc::{
     core::CoreRpcHandler,
     plugin::{PluginId, VoltID},
@@ -33,10 +31,10 @@ use lsp_types::{
     request::{
         CodeActionRequest, CodeActionResolveRequest, Completion,
         DocumentSymbolRequest, Formatting, GotoDefinition, GotoTypeDefinition,
-        HoverRequest, Initialize, InlayHintRequest, PrepareRenameRequest,
-        References, RegisterCapability, Rename, ResolveCompletionItem,
-        SelectionRangeRequest, SemanticTokensFullRequest, SignatureHelpRequest,
-        WorkDoneProgressCreate, WorkspaceSymbol,
+        HoverRequest, Initialize, InlayHintRequest, InlineCompletionRequest,
+        PrepareRenameRequest, References, RegisterCapability, Rename,
+        ResolveCompletionItem, SelectionRangeRequest, SemanticTokensFullRequest,
+        SignatureHelpRequest, WorkDoneProgressCreate, WorkspaceSymbolRequest,
     },
     CodeActionProviderCapability, DidChangeTextDocumentParams,
     DidSaveTextDocumentParams, DocumentSelector, HoverProviderCapability,
@@ -417,9 +415,11 @@ impl PluginServerRpcHandler {
     }
 
     pub fn shutdown(&self) {
+        // to kill lsp
         self.handle_rpc(PluginServerRpc::Handler(
             PluginHandlerNotification::Shutdown,
         ));
+        // to end PluginServerRpcHandler::mainloop
         self.handle_rpc(PluginServerRpc::Shutdown);
     }
 
@@ -760,10 +760,14 @@ impl PluginHostHandler {
             InlayHintRequest::METHOD => {
                 self.server_capabilities.inlay_hint_provider.is_some()
             }
+            InlineCompletionRequest::METHOD => self
+                .server_capabilities
+                .inline_completion_provider
+                .is_some(),
             DocumentSymbolRequest::METHOD => {
                 self.server_capabilities.document_symbol_provider.is_some()
             }
-            WorkspaceSymbol::METHOD => {
+            WorkspaceSymbolRequest::METHOD => {
                 self.server_capabilities.workspace_symbol_provider.is_some()
             }
             PrepareRenameRequest::METHOD => {
@@ -1010,11 +1014,12 @@ impl PluginHostHandler {
             // TODO: remove this after the next release and once we convert all the existing plugins to use the request.
             StartLspServer::METHOD => {
                 self.core_rpc.log(
-                    tracing::Level::WARN,
+                    lapce_rpc::core::LogLevel::Warn,
                     format!(
                         "[{}] Usage of startLspServer as a notification is deprecated.",
                         self.volt_display_name
                     ),
+                    Some(format!("lapce_proxy::plugin::psp::{}::{}::StartLspServer", self.volt_id.author, self.volt_id.name)),
                 );
 
                 let params: StartLspServerParams =
@@ -1059,10 +1064,23 @@ impl PluginHostHandler {
             LogMessage::METHOD => {
                 let message: LogMessageParams =
                     serde_json::from_value(serde_json::to_value(params)?)?;
-                self.catalog_rpc.core_rpc.log_message(message);
+                self.catalog_rpc.core_rpc.log_message(
+                    message,
+                    format!(
+                        "lapce_proxy::plugin::psp::{}::{}::LogMessage",
+                        self.volt_id.author, self.volt_id.name
+                    ),
+                );
             }
             _ => {
-                eprintln!("host notificaton {method} not handled");
+                self.core_rpc.log(
+                    lapce_rpc::core::LogLevel::Warn,
+                    format!("host notification {method} not handled"),
+                    Some(format!(
+                        "lapce_proxy::plugin::psp::{}::{}::{method}",
+                        self.volt_id.author, self.volt_id.name
+                    )),
+                );
             }
         }
         Ok(())
@@ -1190,8 +1208,12 @@ impl PluginHostHandler {
         if let Some(info) = self.spawned_lsp.get_mut(&plugin_id) {
             let Some(resp) = info.resp.take() else {
                 self.core_rpc.log(
-                    tracing::Level::WARN,
+                    lapce_rpc::core::LogLevel::Warn,
                     "Spawned lsp initialized twice?".to_string(),
+                    Some(format!(
+                        "{}::{}::handle_spawned_plugin_loaded",
+                        self.volt_id.author, self.volt_id.name
+                    )),
                 );
                 return;
             };
