@@ -13,8 +13,9 @@ use lapce_xi_rope::RopeDelta;
 use lsp_types::{
     request::GotoTypeDefinitionResponse, CodeAction, CodeActionResponse,
     CompletionItem, Diagnostic, DocumentSymbolResponse, GotoDefinitionResponse,
-    Hover, InlayHint, Location, Position, PrepareRenameResponse, SelectionRange,
-    SymbolInformation, TextDocumentItem, TextEdit, WorkspaceEdit,
+    Hover, InlayHint, InlineCompletionResponse, InlineCompletionTriggerKind,
+    Location, Position, PrepareRenameResponse, SelectionRange, SymbolInformation,
+    TextDocumentItem, TextEdit, WorkspaceEdit,
 };
 use parking_lot::Mutex;
 use serde::{Deserialize, Serialize};
@@ -111,6 +112,11 @@ pub enum ProxyRequest {
     GetInlayHints {
         path: PathBuf,
     },
+    GetInlineCompletions {
+        path: PathBuf,
+        position: Position,
+        trigger_kind: InlineCompletionTriggerKind,
+    },
     GetSemanticTokens {
         path: PathBuf,
     },
@@ -176,6 +182,9 @@ pub enum ProxyRequest {
         from: PathBuf,
         to: PathBuf,
     },
+    TestCreateAtPath {
+        path: PathBuf,
+    },
     DapVariable {
         dap_id: DapId,
         reference: usize,
@@ -193,6 +202,8 @@ pub enum ProxyNotification {
     Initialize {
         workspace: Option<PathBuf>,
         disabled_volts: Vec<VoltID>,
+        /// Paths to extra plugins that should be loaded
+        extra_plugin_paths: Vec<PathBuf>,
         plugin_configurations: HashMap<String, HashMap<String, serde_json::Value>>,
         window_id: usize,
         tab_id: usize,
@@ -373,6 +384,9 @@ pub enum ProxyResponse {
     GetInlayHints {
         hints: Vec<InlayHint>,
     },
+    GetInlineCompletions {
+        completions: InlineCompletionResponse,
+    },
     GetSemanticTokens {
         styles: SemanticStyles,
     },
@@ -394,6 +408,9 @@ pub enum ProxyResponse {
     DapGetScopesResponse {
         scopes: Vec<(dap_types::Scope, Vec<dap_types::Variable>)>,
     },
+    CreatePathResponse {
+        path: PathBuf,
+    },
     Success {},
     SaveResponse {},
 }
@@ -556,6 +573,7 @@ impl ProxyRpcHandler {
         &self,
         workspace: Option<PathBuf>,
         disabled_volts: Vec<VoltID>,
+        extra_plugin_paths: Vec<PathBuf>,
         plugin_configurations: HashMap<String, HashMap<String, serde_json::Value>>,
         window_id: usize,
         tab_id: usize,
@@ -563,6 +581,7 @@ impl ProxyRpcHandler {
         self.notification(ProxyNotification::Initialize {
             workspace,
             disabled_volts,
+            extra_plugin_paths,
             plugin_configurations,
             window_id,
             tab_id,
@@ -666,6 +685,14 @@ impl ProxyRpcHandler {
         self.request_async(ProxyRequest::RenamePath { from, to }, f);
     }
 
+    pub fn test_create_at_path(
+        &self,
+        path: PathBuf,
+        f: impl ProxyCallback + 'static,
+    ) {
+        self.request_async(ProxyRequest::TestCreateAtPath { path }, f);
+    }
+
     pub fn save_buffer_as(
         &self,
         buffer_id: BufferId,
@@ -917,6 +944,23 @@ impl ProxyRpcHandler {
         self.request_async(ProxyRequest::GetInlayHints { path }, f);
     }
 
+    pub fn get_inline_completions(
+        &self,
+        path: PathBuf,
+        position: Position,
+        trigger_kind: InlineCompletionTriggerKind,
+        f: impl ProxyCallback + 'static,
+    ) {
+        self.request_async(
+            ProxyRequest::GetInlineCompletions {
+                path,
+                position,
+                trigger_kind,
+            },
+            f,
+        );
+    }
+
     pub fn update(&self, path: PathBuf, delta: RopeDelta, rev: u64) {
         self.notification(ProxyNotification::Update { path, delta, rev });
     }
